@@ -7,8 +7,6 @@ import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.core.Version;
 import com.fasterxml.jackson.core.Versioned;
 
-import com.fasterxml.jackson.databind.JsonDeserializer;
-import com.fasterxml.jackson.databind.JsonSerializer;
 import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
 import com.fasterxml.jackson.databind.cfg.MapperConfig;
@@ -101,6 +99,8 @@ public abstract class AnnotationIntrospector
      * Factory method for accessing "no operation" implementation
      * of introspector: instance that will never find any annotation-based
      * configuration.
+     *
+     * @return "no operation" instance
      */
     public static AnnotationIntrospector nopInstance() {
         return NopAnnotationIntrospector.instance;
@@ -126,6 +126,9 @@ public abstract class AnnotationIntrospector
      * as contents.
      * This usually works for sub-classes, except for proxy or delegating "container
      * introspectors" which need to override implementation.
+     *
+     * @return Collection of all introspectors starting with this one, in case
+     *    multiple introspectors are chained
      */
     public Collection<AnnotationIntrospector> allIntrospectors() {
         return Collections.singletonList(this);
@@ -140,6 +143,11 @@ public abstract class AnnotationIntrospector
      * Default implementation adds this introspector in result; this usually
      * works for sub-classes, except for proxy or delegating "container
      * introspectors" which need to override implementation.
+     *
+     * @param result Container to add introspectors to
+     *
+     * @return Passed in {@code Collection} filled with introspectors as explained
+     *    above
      */
     public Collection<AnnotationIntrospector> allIntrospectors(Collection<AnnotationIntrospector> result) {
         result.add(this);
@@ -165,8 +173,11 @@ public abstract class AnnotationIntrospector
      * Method for checking whether given annotation is considered an
      * annotation bundle: if so, all meta-annotations it has will
      * be used instead of annotation ("bundle") itself.
-     * 
-     * @since 2.0
+     *
+     * @param ann Annotated entity to introspect
+     *
+     * @return True if given annotation is considered an annotation
+     *    bundle; false if not
      */
     public boolean isAnnotationBundle(Annotation ann) {
         return false;
@@ -185,8 +196,11 @@ public abstract class AnnotationIntrospector
      * values referenced by annotated property; latter
      * having precedence) should include Object Identifier,
      * and if so, specify details of Object Identity used.
+     *
+     * @param ann Annotated entity to introspect
      * 
-     * @since 2.0
+     * @return Details of Object Id as explained above, if Object Id
+     *    handling to be applied; {@code null} otherwise.
      */
     public ObjectIdInfo findObjectIdInfo(Annotated ann) {
         return null;
@@ -194,6 +208,11 @@ public abstract class AnnotationIntrospector
 
     /**
      * Method for figuring out additional properties of an Object Identity reference
+     *
+     * @param ann Annotated entity to introspect
+     * @param objectIdInfo (optional) Base Object Id information, if any; {@code null} if none
+     *
+     * @return {@link ObjectIdInfo} augmented with possible additional information
      * 
      * @since 2.1
      */
@@ -216,11 +235,28 @@ public abstract class AnnotationIntrospector
      *<p> 
      * NOTE: method signature changed in 2.1, to return {@link PropertyName}
      * instead of String.
+     *
+     * @param ac Annotated class to introspect
+     *
+     * @return Root name to use, if any; {@code null} if not
      */
     public PropertyName findRootName(AnnotatedClass ac) {
         return null;
     }
 
+    /**
+     * Method for checking whether properties that have specified type
+     * (class, not generics aware) should be completely ignored for
+     * serialization and deserialization purposes.
+     * 
+     * @param ac Annotated class to introspect
+     * 
+     * @return Boolean.TRUE if properties of type should be ignored;
+     *   Boolean.FALSE if they are not to be ignored, null for default
+     *   handling (which is 'do not ignore')
+     */
+    public Boolean isIgnorableType(AnnotatedClass ac) { return null; }
+
     /**
      * Method for finding information about properties to ignore either by
      * name, or by more general specification ("ignore all unknown").
@@ -229,31 +265,37 @@ public abstract class AnnotationIntrospector
      * {@link #findPropertiesToIgnore(Annotated, boolean)} and
      * {@link #findIgnoreUnknownProperties(AnnotatedClass)}.
      *
-     * @since 2.8
+     * @param config Configuration settings in effect (for serialization or deserialization)
+     * @param ann Annotated entity (Class, Accessor) to introspect
+     *
+     * @since 2.12 (to replace {@code findPropertyIgnorals()})
      */
-    public JsonIgnoreProperties.Value findPropertyIgnorals(Annotated ac)
+    public JsonIgnoreProperties.Value findPropertyIgnoralByName(MapperConfig<?> config, Annotated ann)
     {
-        // 18-Oct-2016, tatu: Used to call deprecated methods for backwards
-        //   compatibility in 2.8, but not any more in 2.9
-        return JsonIgnoreProperties.Value.empty();
+        // In 2.12, remove redirection in future
+        return findPropertyIgnorals(ann);
     }
 
     /**
-     * Method for checking whether properties that have specified type
-     * (class, not generics aware) should be completely ignored for
-     * serialization and deserialization purposes.
-     * 
-     * @param ac Type to check
-     * 
-     * @return Boolean.TRUE if properties of type should be ignored;
-     *   Boolean.FALSE if they are not to be ignored, null for default
-     *   handling (which is 'do not ignore')
+     * Method for finding information about names of properties to included.
+     * This is typically used to strictly limit properties to include based
+     * on fully defined set of names ("allow-listing"), as opposed to excluding
+     * potential properties by exclusion ("deny-listing").
+     *
+     * @param config Configuration settings in effect (for serialization or deserialization)
+     * @param ann Annotated entity (Class, Accessor) to introspect
+     *
+     * @since 2.12
      */
-    public Boolean isIgnorableType(AnnotatedClass ac) { return null; }
+    public JsonIncludeProperties.Value findPropertyInclusionByName(MapperConfig<?> config, Annotated ann) {
+        return JsonIncludeProperties.Value.all();
+    }
 
     /**
      * Method for finding if annotated class has associated filter; and if so,
      * to return id that is used to locate filter.
+     *
+     * @param ann Annotated entity to introspect
      * 
      * @return Id of the filter to use for filtering properties of annotated
      *    class, if any; or null if none found.
@@ -265,7 +307,9 @@ public abstract class AnnotationIntrospector
      * class, if any specified by annotations; and if so, either return
      * a {@link PropertyNamingStrategy} instance, or Class to use for
      * creating instance
-     * 
+     *
+     * @param ac Annotated class to introspect
+     *
      * @return Sub-class or instance of {@link PropertyNamingStrategy}, if one
      *   is specified for given class; null if not.
      * 
@@ -279,6 +323,8 @@ public abstract class AnnotationIntrospector
      * There are no further definitions for contents; for example, whether
      * these may be marked up using HTML (or something like wiki format like Markup)
      * is not defined.
+     *
+     * @param ac Annotated class to introspect
      * 
      * @return Human-readable description, if any.
      * 
@@ -289,32 +335,41 @@ public abstract class AnnotationIntrospector
     /**
      * @param forSerialization True if requesting properties to ignore for serialization;
      *   false if for deserialization
-     * 
+     * @param ac Annotated class to introspect
+     *
+     * @return Array of names of properties to ignore
+     *
      * @since 2.6
      *
-     * @deprecated Since 2.8, use {@link #findPropertyIgnorals} instead
+     * @deprecated Since 2.8, use {@link #findPropertyIgnoralByName} instead
      */
     @Deprecated // since 2.8
     public String[] findPropertiesToIgnore(Annotated ac, boolean forSerialization) {
         return null;
     }
 
-    /**
-     * @deprecated Since 2.6, use variant that takes second argument.
-     */
-    @Deprecated // since 2.6
-    public String[] findPropertiesToIgnore(Annotated ac) {
-        return null;
-    }
-
     /**
      * Method for checking whether an annotation indicates that all unknown properties
+     * should be ignored.
      *
-     * @deprecated Since 2.8, use {@link #findPropertyIgnorals} instead
+     * @param ac Annotated class to introspect
+     *
+     * @return True if class has something indicating "ignore [all] unknown properties"
+     *
+     * @deprecated Since 2.8, use {@link #findPropertyIgnoralByName} instead
      */
     @Deprecated // since 2.8
     public Boolean findIgnoreUnknownProperties(AnnotatedClass ac) { return null; }
-    
+
+    /**
+     * @since 2.8
+     * @deprecated 2.12, use {@link #findPropertyIgnoralByName} instead.
+     */
+    @Deprecated // since 2.12
+    public JsonIgnoreProperties.Value findPropertyIgnorals(Annotated ac) {
+        return JsonIgnoreProperties.Value.empty();
+    }
+
     /*
     /**********************************************************
     /* Property auto-detection
@@ -614,6 +669,42 @@ public abstract class AnnotationIntrospector
         return null;
     }
 
+    /**
+     * Method called on fields that are eligible candidates for properties
+     * (that is, non-static member fields), but not necessarily selected (may
+     * or may not be visible), to let fields affect name linking.
+     * Call will be made after finding implicit name (which by default is just
+     * name of the field, but may be overridden by introspector), but before
+     * discovering other accessors.
+     * If non-null name returned, it is to be used to find other accessors (getters,
+     * setters, creator parameters) and replace their implicit names with that
+     * of field's implicit name (assuming they differ).
+     *<p>
+     * Specific example (and initial use case is for support Kotlin's "is getter"
+     * matching (see
+     * <a href="https://kotlinlang.org/docs/reference/java-to-kotlin-interop.html">Kotling Interop</a>
+     * for details), in which field like '{@code isOpen}' would have implicit name of
+     * "isOpen", match getter {@code getOpen()} and setter {@code setOpen(boolean)},
+     * but use logical external name of "isOpen" (and not implicit name of getter/setter, "open"!).
+     * To achieve this, field implicit name needs to remain "isOpen" but this method needs
+     * to return name {@code PropertyName.construct("open")}: doing so will "pull in" getter
+     * and/or setter, and rename them as "isOpen".
+     *
+     * @param config Effective mapper configuration in use
+     * @param f Field to check
+     * @param implName Implicit name of the field; usually name of field itself but not always,
+     *    used as the target name for accessors to rename.
+     *
+     * @return Name used to find other accessors to rename, if any; {@code null} to indicate
+     *    no renaming
+     *
+     * @since 2.11
+     */
+    public PropertyName findRenameByField(MapperConfig<?> config,
+            AnnotatedField f, PropertyName implName) {
+        return null;
+    }
+
     /**
      * @deprecated Since 2.9 Use {@link #findInjectableValue} instead
      */
@@ -630,21 +721,19 @@ public abstract class AnnotationIntrospector
 
     /**
      * Method for getting a serializer definition on specified method
-     * or field. Type of definition is either instance (of type
-     * {@link JsonSerializer}) or Class (of type
-     * <code>Class&lt;JsonSerializer></code>); if value of different
-     * type is returned, a runtime exception may be thrown by caller.
+     * or field. Type of definition is either instance (of type {@link JsonSerializer})
+     * or Class (of {@code Class<JsonSerializer>} implementation subtype);
+     * if value of different type is returned, a runtime exception may be thrown by caller.
      */
     public Object findSerializer(Annotated am) {
         return null;
     }
 
     /**
-     * Method for getting a serializer definition for keys of associated <code>Map</code> property.
-     * Type of definition is either instance (of type
-     * {@link JsonSerializer}) or Class (of type
-     * <code>Class&lt;JsonSerializer></code>); if value of different
-     * type is returned, a runtime exception may be thrown by caller.
+     * Method for getting a serializer definition for keys of associated {@code java.util.Map} property.
+     * Type of definition is either instance (of type {@link JsonSerializer})
+     * or Class (of type  {@code Class<JsonSerializer>});
+     * if value of different type is returned, a runtime exception may be thrown by caller.
      */
     public Object findKeySerializer(Annotated am) {
         return null;
@@ -652,10 +741,10 @@ public abstract class AnnotationIntrospector
 
     /**
      * Method for getting a serializer definition for content (values) of
-     * associated <code>Collection</code>, <code>array</code> or <code>Map</code> property.
-     * Type of definition is either instance (of type
-     * {@link JsonSerializer}) or Class (of type
-     * <code>Class&lt;JsonSerializer></code>); if value of different
+     * associated <code>Collection</code>, <code>array</code> or {@code Map} property.
+     * Type of definition is either instance (of type {@link JsonSerializer})
+     * or Class (of type  {@code Class<JsonSerializer>});
+     * if value of different
      * type is returned, a runtime exception may be thrown by caller.
      */
     public Object findContentSerializer(Annotated am) {
@@ -878,6 +967,23 @@ public abstract class AnnotationIntrospector
         return null;
     }
 
+    /**
+     * Method for checking whether given method has an annotation
+     * that suggests the return value of annotated field or method
+     * should be used as "the key" of the object instance; usually
+     * serialized as a primitive value such as String or number.
+     *
+     * @return {@link Boolean#TRUE} if such annotation is found and is not disabled;
+     *   {@link Boolean#FALSE} if disabled annotation (block) is found (to indicate
+     *   accessor is definitely NOT to be used "as value"); or `null` if no
+     *   information found.
+     *
+     * @since 2.12
+     */
+    public Boolean hasAsKey(MapperConfig<?> config, Annotated a) {
+        return null;
+    }
+
     /**
      * Method for checking whether given method has an annotation
      * that suggests that the return value of annotated method
@@ -907,16 +1013,18 @@ public abstract class AnnotationIntrospector
      * method to be used for accessing set of miscellaneous "extra"
      * properties, often bound with matching "any setter" method.
      *
+     * @param ann Annotated entity to check
+     * 
      * @return True if such annotation is found (and is not disabled),
      *   false otherwise
      *
      * @since 2.9
      */
-    public Boolean hasAnyGetter(Annotated a) {
+    public Boolean hasAnyGetter(Annotated ann) {
 
         // 21-Nov-2016, tatu: Delegate in 2.9; remove redirect from later versions
-        if (a instanceof AnnotatedMethod) {
-            if (hasAnyGetterAnnotation((AnnotatedMethod) a)) {
+        if (ann instanceof AnnotatedMethod) {
+            if (hasAnyGetterAnnotation((AnnotatedMethod) ann)) {
                 return true;
             }
         }
@@ -927,10 +1035,13 @@ public abstract class AnnotationIntrospector
      * Method for efficiently figuring out which if given set of <code>Enum</code> values
      * have explicitly defined name. Method will overwrite entries in incoming <code>names</code>
      * array with explicit names found, if any, leaving other entries unmodified.
-     *<p>
-     * Default implementation will simply delegate to {@link #findEnumValue}, which is close
-     * enough, although unfortunately NOT 100% equivalent (as it will also consider <code>name()</code>
-     * to give explicit value).
+     *
+     * @param enumType Type of Enumeration
+     * @param enumValues Values of enumeration
+     * @param names Matching declared names of enumeration values (with indexes
+     *     matching {@code enumValues} entries)
+     *
+     * @return Array of names to use (possible {@code names} passed as argument)
      *
      * @since 2.7
      */
@@ -939,11 +1050,30 @@ public abstract class AnnotationIntrospector
         return names;
     }
 
+    /**
+     * Method that is related to {@link #findEnumValues} but is called to check if
+     * there are alternative names (aliased) that can be accepted for entries, in 
+     * addition to primary names introspected earlier.
+     * If so, these aliases should be returned in {@code aliases} {@link List} passed
+     * as argument (and initialized for proper size by caller).
+     *
+     * @param enumType Type of Enumeration
+     * @param enumValues Values of enumeration
+     * @param aliases (in/out) Pre-allocated array where aliases found, if any, may be
+     *     added (in indexes matching those of {@code enumValues})
+     *
+     * @since 2.11
+     */
+    public void findEnumAliases(Class<?> enumType, Enum<?>[] enumValues, String[][] aliases) {
+        ;
+    }
+
     /**
      * Finds the Enum value that should be considered the default value, if possible.
      *
-     * @param enumCls The Enum class to scan for the default value.
-     * @return null if none found or it's not possible to determine one.
+     * @param enumCls The Enum class to scan for the default value
+     *
+     * @return null if none found or it's not possible to determine one
      *
      * @since 2.8
      */
@@ -956,6 +1086,8 @@ public abstract class AnnotationIntrospector
      * given enumeration entry; used when serializing enumerations
      * as Strings (the standard method).
      *
+     * @param value Enum value to introspect
+     *
      * @return Serialized enum value.
      *
      * @deprecated Since 2.8: use {@link #findEnumValues} instead because this method
@@ -969,6 +1101,8 @@ public abstract class AnnotationIntrospector
     }
 
     /**
+     * @param am Annotated method to check
+     * 
      * @deprecated Since 2.9 Use {@link #hasAsValue(Annotated)} instead.
      */
     @Deprecated // since 2.9
@@ -977,6 +1111,8 @@ public abstract class AnnotationIntrospector
     }
 
     /**
+     * @param am Annotated method to check
+     * 
      * @deprecated Since 2.9 Use {@link #hasAnyGetter} instead
      */
     @Deprecated
@@ -993,9 +1129,8 @@ public abstract class AnnotationIntrospector
     /**
      * Method for getting a deserializer definition on specified method
      * or field.
-     * Type of definition is either instance (of type
-     * {@link JsonDeserializer}) or Class (of type
-     * <code>Class&lt;JsonDeserializer></code>); if value of different
+     * Type of definition is either instance (of type {@link JsonDeserializer})
+     * or Class (of type  {@code Class&<JsonDeserializer>});
      * type is returned, a runtime exception may be thrown by caller.
      */
     public Object findDeserializer(Annotated am) {
@@ -1005,9 +1140,9 @@ public abstract class AnnotationIntrospector
     /**
      * Method for getting a deserializer definition for keys of
      * associated <code>Map</code> property.
-     * Type of definition is either instance (of type
-     * {@link JsonDeserializer}) or Class (of type
-     * <code>Class&lt;JsonDeserializer></code>); if value of different
+     * Type of definition is either instance (of type {@link JsonDeserializer})
+     * or Class (of type  {@code Class<JsonDeserializer>});
+     * if value of different
      * type is returned, a runtime exception may be thrown by caller.
      */
     public Object findKeyDeserializer(Annotated am) {
@@ -1018,9 +1153,9 @@ public abstract class AnnotationIntrospector
      * Method for getting a deserializer definition for content (values) of
      * associated <code>Collection</code>, <code>array</code> or
      * <code>Map</code> property.
-     * Type of definition is either instance (of type
-     * {@link JsonDeserializer}) or Class (of type
-     * <code>Class&lt;JsonDeserializer></code>); if value of different
+     * Type of definition is either instance (of type {@link JsonDeserializer})
+     * or Class (of type  {@code Class<JsonDeserializer>});
+     * if value of different
      * type is returned, a runtime exception may be thrown by caller.
      */
     public Object findContentDeserializer(Annotated am) {
@@ -1100,6 +1235,7 @@ public abstract class AnnotationIntrospector
      * Type is usually narrowing conversion (i.e.subtype of declared type).
      * Declared return type of the method is also considered acceptable.
      *
+     * @param ann Annotated entity to introspect
      * @param baseType Assumed type before considering annotations
      *
      * @return Class to use for deserialization instead of declared type
@@ -1107,7 +1243,7 @@ public abstract class AnnotationIntrospector
      * @deprecated Since 2.7 call {@link #refineDeserializationType} instead
      */
     @Deprecated
-    public Class<?> findDeserializationType(Annotated am, JavaType baseType) {
+    public Class<?> findDeserializationType(Annotated ann, JavaType baseType) {
         return null;
     }
     
@@ -1116,6 +1252,7 @@ public abstract class AnnotationIntrospector
      * method can have, to define more specific key type to use.
      * It should be only be used with {@link java.util.Map} types.
      * 
+     * @param ann Annotated entity to introspect
      * @param baseKeyType Assumed key type before considering annotations
      *
      * @return Class specifying more specific type to use instead of
@@ -1124,7 +1261,7 @@ public abstract class AnnotationIntrospector
      * @deprecated Since 2.7 call {@link #refineDeserializationType} instead
      */
     @Deprecated
-    public Class<?> findDeserializationKeyType(Annotated am, JavaType baseKeyType) {
+    public Class<?> findDeserializationKeyType(Annotated ann, JavaType baseKeyType) {
         return null;
     }
 
@@ -1134,6 +1271,7 @@ public abstract class AnnotationIntrospector
      * content refers to Map values and Collection/array elements.
      * It should be only be used with Map, Collection and array types.
      * 
+     * @param ann Annotated entity to introspect
      * @param baseContentType Assumed content (value) type before considering annotations
      *
      * @return Class specifying more specific type to use instead of
@@ -1142,7 +1280,7 @@ public abstract class AnnotationIntrospector
      * @deprecated Since 2.7 call {@link #refineDeserializationType} instead
      */
     @Deprecated
-    public Class<?> findDeserializationContentType(Annotated am, JavaType baseContentType) {
+    public Class<?> findDeserializationContentType(Annotated ann, JavaType baseContentType) {
         return null;
     }
 
@@ -1156,6 +1294,8 @@ public abstract class AnnotationIntrospector
      * Method getting {@link ValueInstantiator} to use for given
      * type (class): return value can either be an instance of
      * instantiator, or class of instantiator to create.
+     *
+     * @param ac Annotated class to introspect
      */
     public Object findValueInstantiator(AnnotatedClass ac) {
         return null;
@@ -1171,6 +1311,8 @@ public abstract class AnnotationIntrospector
      * method does not allow returning instances: the reason is
      * that builders have state, and a separate instance needs
      * to be created for each deserialization call.
+     *
+     * @param ac Annotated class to introspect
      * 
      * @since 2.0
      */
@@ -1179,6 +1321,8 @@ public abstract class AnnotationIntrospector
     }
 
     /**
+     * @param ac Annotated class to introspect
+     *
      * @since 2.0
      */
     public JsonPOJOBuilder.Value findPOJOBuilderConfig(AnnotatedClass ac) {
@@ -1199,13 +1343,13 @@ public abstract class AnnotationIntrospector
      * is found; otherwise a non-null name (possibly
      * {@link PropertyName#USE_DEFAULT}, which means "use default heuristics").
      * 
-     * @param a Property accessor to check
-     * 
+     * @param ann Annotated entity to check
+     *
      * @return Name to use if found; null if not.
-     * 
+     *
      * @since 2.1
      */
-    public PropertyName findNameForDeserialization(Annotated a) {
+    public PropertyName findNameForDeserialization(Annotated ann) {
         return null;
     }
     
@@ -1214,32 +1358,38 @@ public abstract class AnnotationIntrospector
      * that suggests that the method is to serve as "any setter";
      * method to be used for setting values of any properties for
      * which no dedicated setter method is found.
+     * 
+     * @param ann Annotated entity to check
      *
      * @return True if such annotation is found (and is not disabled),
      *   false otherwise
-     *   
+     *
      * @since 2.9
      */
-    public Boolean hasAnySetter(Annotated a) {
+    public Boolean hasAnySetter(Annotated ann) {
         return null;
     }
 
     /**
      * Method for finding possible settings for property, given annotations
      * on an accessor.
+     * 
+     * @param ann Annotated entity to check
      *
      * @since 2.9
      */
-    public JsonSetter.Value findSetterInfo(Annotated a) {
+    public JsonSetter.Value findSetterInfo(Annotated ann) {
         return JsonSetter.Value.empty();
     }
 
     /**
      * Method for finding merge settings for property, if any.
+     * 
+     * @param ann Annotated entity to check
      *
      * @since 2.9
      */
-    public Boolean findMergeInfo(Annotated a) {
+    public Boolean findMergeInfo(Annotated ann) {
         return null;
     }
 
@@ -1254,15 +1404,15 @@ public abstract class AnnotationIntrospector
      * as well as possibly as when using mix-in annotations.
      *
      * @param config Configuration settings in effect (for serialization or deserialization)
-     * @param a Annotated accessor (usually constructor or static method) to check
+     * @param ann Annotated accessor (usually constructor or static method) to check
      *
      * @since 2.9
      */
-    public JsonCreator.Mode findCreatorAnnotation(MapperConfig<?> config, Annotated a) {
+    public JsonCreator.Mode findCreatorAnnotation(MapperConfig<?> config, Annotated ann) {
         // 13-Sep-2016, tatu: for backwards compatibility, implement using delegation
         ///   (remove from version AFTER 2.9)
-        if (hasCreatorAnnotation(a)) {
-            JsonCreator.Mode mode = findCreatorBinding(a);
+        if (hasCreatorAnnotation(ann)) {
+            JsonCreator.Mode mode = findCreatorBinding(ann);
             if (mode == null) {
                 mode = JsonCreator.Mode.DEFAULT;
             }
@@ -1277,6 +1427,8 @@ public abstract class AnnotationIntrospector
      * that suggests that the method is a "creator" (aka factory)
      * method to be used for construct new instances of deserialized
      * values.
+     * 
+     * @param ann Annotated entity to check
      *
      * @return True if such annotation is found (and is not disabled),
      *   false otherwise
@@ -1284,7 +1436,7 @@ public abstract class AnnotationIntrospector
      * @deprecated Since 2.9 use {@link #findCreatorAnnotation} instead.
      */
     @Deprecated
-    public boolean hasCreatorAnnotation(Annotated a) {
+    public boolean hasCreatorAnnotation(Annotated ann) {
         return false;
     }
 
@@ -1294,16 +1446,22 @@ public abstract class AnnotationIntrospector
      * true), for cases where there may be ambiguity (currently: single-argument
      * creator with implicit but no explicit name for the argument).
      * 
+     * @param ann Annotated entity to check
+     * 
      * @since 2.5
      * @deprecated Since 2.9 use {@link #findCreatorAnnotation} instead.
      */
     @Deprecated
-    public JsonCreator.Mode findCreatorBinding(Annotated a) {
+    public JsonCreator.Mode findCreatorBinding(Annotated ann) {
         return null;
     }
 
     /**
+     * @param am Annotated method to check
+     *
      * @deprecated Since 2.9 use {@link #hasAnySetter} instead.
+     *
+     * @return {@code true} if "any-setter" annotation was found; {@code false} otherwise
      */
     @Deprecated // since 2.9
     public boolean hasAnySetterAnnotation(AnnotatedMethod am) {
@@ -1328,12 +1486,18 @@ public abstract class AnnotationIntrospector
      *<code>
      *  return annotated.getAnnotation(annoClass);
      *</code>
-     * 
+     *
+     * @param ann Annotated entity to check for specified annotation
+     * @param annoClass Type of annotation to find
+     *
+     * @return Value of given annotation (as per {@code annoClass}), if entity
+     *    has one; {@code null} otherwise
+     *
      * @since 2.5
      */
-    protected <A extends Annotation> A _findAnnotation(Annotated annotated,
+    protected <A extends Annotation> A _findAnnotation(Annotated ann,
             Class<A> annoClass) {
-        return annotated.getAnnotation(annoClass);
+        return ann.getAnnotation(annoClass);
     }
 
     /**
@@ -1346,20 +1510,31 @@ public abstract class AnnotationIntrospector
      *<code>
      *  return annotated.hasAnnotation(annoClass);
      *</code>
+     *
+     * @param ann Annotated entity to check for specified annotation
+     * @param annoClass Type of annotation to find
+     *
+     * @return {@code true} if specified annotation exists in given entity; {@code false} if not
      * 
      * @since 2.5
      */
-    protected boolean _hasAnnotation(Annotated annotated, Class<? extends Annotation> annoClass) {
-        return annotated.hasAnnotation(annoClass);
+    protected boolean _hasAnnotation(Annotated ann, Class<? extends Annotation> annoClass) {
+        return ann.hasAnnotation(annoClass);
     }
 
     /**
      * Alternative lookup method that is used to see if annotation has at least one of
      * annotations of types listed in second argument.
      *
+     * @param ann Annotated entity to check for specified annotation
+     * @param annoClasses Types of annotation to find
+     *
+     * @return {@code true} if at least one of specified annotation exists in given entity;
+     *    {@code false} otherwise
+     *
      * @since 2.7
      */
-    protected boolean _hasOneOf(Annotated annotated, Class<? extends Annotation>[] annoClasses) {
-        return annotated.hasOneOf(annoClasses);
+    protected boolean _hasOneOf(Annotated ann, Class<? extends Annotation>[] annoClasses) {
+        return ann.hasOneOf(annoClasses);
     }
 }
