@@ -195,15 +195,15 @@ public enum MapperFeature implements ConfigFeature
     INFER_PROPERTY_MUTATORS(true),
 
     /**
-     * Feature that determines handling of <code>java.beans.ConstructorProperties<code>
+     * Feature that determines handling of {@code java.beans.ConstructorProperties}
      * annotation: when enabled, it is considered as alias of
      * {@link com.fasterxml.jackson.annotation.JsonCreator}, to mean that constructor
      * should be considered a property-based Creator; when disabled, only constructor
      * parameter name information is used, but constructor is NOT considered an explicit
      * Creator (although may be discovered as one using other annotations or heuristics).
      *<p>
-     * Feature is mostly used to help interoperability with frameworks like Lombok
-     * that may automatically generate <code>ConstructorProperties</code> annotation
+     * Feature is mostly used to help inter-operability with frameworks like Lombok
+     * that may automatically generate {@code ConstructorProperties} annotation
      * but without necessarily meaning that constructor should be used as Creator
      * for deserialization.
      *<p>
@@ -213,6 +213,19 @@ public enum MapperFeature implements ConfigFeature
      */
     INFER_CREATOR_FROM_CONSTRUCTOR_PROPERTIES(true),
 
+    /**
+     * Feature that determines whether nominal property type of {@link Void} is
+     * allowed for Getter methods to indicate {@code null} valued pseudo-property
+     * or not. If enabled, such properties are recognized (see [databind#2675] for
+     * reasons -- mostly things related to frameworks, code generation); if disabled,
+     * such property accessors (or at least getters) are ignored.
+     *<p>
+     * Feature is disabled by default (in 2.12) for backwards compatibility.
+     *
+     * @since 2.12
+     */
+    ALLOW_VOID_VALUED_PROPERTIES(false),
+
     /*
     /******************************************************
     /* Access modifier handling
@@ -296,10 +309,25 @@ public enum MapperFeature implements ConfigFeature
      *<p>
      * Feature is disabled by default for backwards compatibility.
      *
-     * @since 2.9.6
+     * @since 2.10
      */
     USE_BASE_TYPE_AS_DEFAULT_IMPL(false),
 
+    /**
+     * Feature that enables inferring builder type bindings from the value type
+     * being deserialized. This requires that the generic type declaration on
+     * the value type match that on the builder exactly: mismatched type declarations
+     * are not necessarily detected by databind.
+     *<p>
+     * Feature is enabled by default which means that deserialization does
+     * support deserializing types via builders with type parameters (generic types).
+     *<p>
+     * See: https://github.com/FasterXML/jackson-databind/issues/921
+     *
+     * @since 2.12
+     */
+    INFER_BUILDER_TYPE_BINDINGS(true),
+
     /*
     /******************************************************
     /* View-related features
@@ -332,9 +360,8 @@ public enum MapperFeature implements ConfigFeature
 
     /**
      * Feature that defines default property serialization order used
-     * for POJO fields (note: does <b>not</b> apply to {@link java.util.Map}
-     * serialization!):
-     * if enabled, default ordering is alphabetic (similar to
+     * for POJO properties.
+     * If enabled, default ordering is alphabetic (similar to
      * how {@link com.fasterxml.jackson.annotation.JsonPropertyOrder#alphabetic()}
      * works); if disabled, order is unspecified (based on what JDK gives
      * us, which may be declaration order, but is not guaranteed).
@@ -343,10 +370,30 @@ public enum MapperFeature implements ConfigFeature
      * explicit overrides in classes (for example with
      * {@link com.fasterxml.jackson.annotation.JsonPropertyOrder} annotation)
      *<p>
+     * Note: does <b>not</b> apply to {@link java.util.Map} serialization (since
+     * entries are not considered Bean/POJO properties.
+     *<p>
      * Feature is disabled by default.
      */
     SORT_PROPERTIES_ALPHABETICALLY(false),
 
+    /**
+     * Feature that defines whether Creator properties (ones passed through
+     * constructor or static factory method) should be sorted before other properties
+     * for which no explicit order is specified, in case where alphabetic
+     * ordering is to be used for such properties.
+     * Note that in either case explicit order (whether by name or by index)
+     * will have precedence over this setting.
+     *<p>
+     * Note: does <b>not</b> apply to {@link java.util.Map} serialization (since
+     * entries are not considered Bean/POJO properties.
+     *<p>
+     * Feature is enabled by default.
+     *
+     * @since 2.12
+     */
+    SORT_CREATOR_PROPERTIES_FIRST(true),
+
     /*
     /******************************************************
     /* Name-related features
@@ -361,7 +408,7 @@ public enum MapperFeature implements ConfigFeature
      *<p>
      * Note that there is additional performance overhead since incoming property
      * names need to be lower-cased before comparison, for cases where there are upper-case
-     * letters. Overhead for names that are already lower-case should be negligible however.
+     * letters. Overhead for names that are already lower-case should be negligible.
      *<p>
      * Feature is disabled by default.
      * 
@@ -373,15 +420,32 @@ public enum MapperFeature implements ConfigFeature
     /**
      * Feature that determines if Enum deserialization should be case sensitive or not.
      * If enabled, Enum deserialization will ignore case, that is, case of incoming String
-     * value and enum id (dependant on other settings, either `name()`, `toString()`, or
+     * value and enum id (depending on other settings, either `name()`, `toString()`, or
      * explicit override) do not need to match.
-     * <p>
+     *<p>
+     * This should allow both Enum-as-value deserialization and Enum-as-Map-key, but latter
+     * only works since Jackson 2.12 (due to incomplete implementation).
+     *<p>
      * Feature is disabled by default.
      *
      * @since 2.9
      */
     ACCEPT_CASE_INSENSITIVE_ENUMS(false),
 
+    /**
+     * Feature that permits parsing some enumerated text-based value types but ignoring the case
+     * of the values on deserialization: for example, date/time type deserializers.
+     * Support for this feature depends on deserializer implementations using it.
+     *<p>
+     * Note, however, that regular {@code Enum} types follow {@link #ACCEPT_CASE_INSENSITIVE_ENUMS}
+     * setting instead.
+     *<p>
+     * Feature is disabled by default.
+     * 
+     * @since 2.10
+     */
+    ACCEPT_CASE_INSENSITIVE_VALUES(false),
+
     /**
      * Feature that can be enabled to make property names be
      * overridden by wrapper name (usually detected with annotations
@@ -482,14 +546,31 @@ public enum MapperFeature implements ConfigFeature
      * merging is skipped and new value is created (<code>true</code>) or
      * an exception is thrown (false).
      *<p>
-     * Feature is disabled by default since non-mergeable property types are ignored
-     * even if defaults call for merging, and usually explicit per-type or per-property
-     * settings for such types should result in an exception.
+     * Feature is enabled by default, to allow use of merge defaults even in presence
+     * of some unmergeable properties.
      *
      * @since 2.9
      */
-    IGNORE_MERGE_FOR_UNMERGEABLE(true)
+    IGNORE_MERGE_FOR_UNMERGEABLE(true),
 
+    /**
+     * Setting that may be enabled to reconfigure <b>default</b>
+     * {@link com.fasterxml.jackson.databind.jsontype.PolymorphicTypeValidator}
+     * used by legacy {@code ObjectMapper.enableDefaultTyping()} methods as well as default used
+     * for annotation-based polymorphic handling so that it uses
+     * {@link com.fasterxml.jackson.databind.jsontype.DefaultBaseTypeLimitingValidator}.
+     * This will block use of a set of "unsafe" base types such as {@link java.lang.Object}
+     * through methods that do not require passing of explicit {@link com.fasterxml.jackson.databind.jsontype.PolymorphicTypeValidator}.
+     * It is still possible to override one used; this only affects default settings.
+     *<p>
+     * Enabling of this setting is <b>strongly recommended</b>.
+     *<p>
+     * Feature is disabled by default in 2.x for backwards compatibility reasons: it will become
+     * default setting (and feature likely removed) in 3.0.
+     * 
+     * @since 2.11
+     */
+    BLOCK_UNSAFE_POLYMORPHIC_BASE_TYPES(false)
     ;
 
     private final boolean _defaultState;
