@@ -115,6 +115,19 @@ public enum SerializationFeature implements ConfigFeature
      */
     FAIL_ON_UNWRAPPED_TYPE_IDENTIFIERS(true),
 
+    /**
+     * Feature that determines what happens when a direct self-reference is detected
+     * by a POJO (and no Object Id handling is enabled for it):
+     * if enabled write that reference as null; if disabled, default behavior is
+     * used (which will try to serialize usually resulting in exception).
+     * But if {@link SerializationFeature#FAIL_ON_SELF_REFERENCES} is enabled. this property is ignored.
+     * <p>
+     * Feature is disabled by default.
+     *
+     * @since 2.11
+     */
+    WRITE_SELF_REFERENCES_AS_NULL(false),
+
     /*
     /******************************************************
     /* Output life cycle features
@@ -162,32 +175,33 @@ public enum SerializationFeature implements ConfigFeature
     /**
      * Feature that determines whether Date (and date/time) values
      * (and Date-based things like {@link java.util.Calendar}s) are to be
-     * serialized as numeric timestamps (true; the default),
+     * serialized as numeric time stamps (true; the default),
      * or as something else (usually textual representation).
-     * If textual representation is used, the actual format is
-     * one returned by a call to
-     * {@link com.fasterxml.jackson.databind.SerializationConfig#getDateFormat}:
-     * the default setting being {@link com.fasterxml.jackson.databind.util.StdDateFormat},
-     * which corresponds to format String of "yyyy-MM-dd'T'HH:mm:ss.SSSZ"
-     * (see {@link java.text.DateFormat} for details of format Strings).
+     * If textual representation is used, the actual format depends on configuration
+     * settings including possible per-property use of {@code @JsonFormat} annotation,
+     * globally configured {@link java.text.DateFormat}.
      *<p>
-     * Note: whether this feature affects handling of other date-related
+     * For "classic" JDK date types ({@link java.util.Date}, {@link java.util.Calendar})
+     * the default formatting is provided by {@link com.fasterxml.jackson.databind.util.StdDateFormat},
+     * and corresponds to format String of "yyyy-MM-dd'T'HH:mm:ss.SSSX"
+     * (see {@link java.text.DateFormat} for details of format Strings).
+     * Whether this feature affects handling of other date-related
      * types depend on handlers of those types, although ideally they
      * should use this feature
      *<p>
      * Note: whether {@link java.util.Map} keys are serialized as Strings
-     * or not is controlled using {@link #WRITE_DATE_KEYS_AS_TIMESTAMPS}.
+     * or not is controlled using {@link #WRITE_DATE_KEYS_AS_TIMESTAMPS} instead of
+     * this feature.
      *<p>
      * Feature is enabled by default, so that date/time are by default
-     * serialized as timestamps.
+     * serialized as time stamps.
      */
     WRITE_DATES_AS_TIMESTAMPS(true),
 
     /**
      * Feature that determines whether {@link java.util.Date}s
      * (and sub-types) used as {@link java.util.Map} keys are serialized
-     * as timestamps or not (if not, will be serialized as textual
-     * values).
+     * as time stamps or not (if not, will be serialized as textual values).
      *<p>
      * Default value is 'false', meaning that Date-valued Map keys are serialized
      * as textual (ISO-8601) values.
@@ -257,7 +271,7 @@ public enum SerializationFeature implements ConfigFeature
     WRITE_ENUMS_USING_TO_STRING(false),
 
     /**
-     * Feature that determines whethere Java Enum values are serialized
+     * Feature that determines whether Java Enum values are serialized
      * as numbers (true), or textual values (false). If textual values are
      * used, other settings are also considered.
      * If this feature is enabled,
@@ -267,16 +281,42 @@ public enum SerializationFeature implements ConfigFeature
      * Note that this feature has precedence over {@link #WRITE_ENUMS_USING_TO_STRING},
      * which is only considered if this feature is set to false.
      *<p>
+     * Note that since 2.10, this does NOT apply to {@link Enum}s written as
+     * keys of {@link java.util.Map} values, which has separate setting,
+     * {@link #WRITE_ENUM_KEYS_USING_INDEX}.
+     *<p>
      * Feature is disabled by default.
      */
     WRITE_ENUMS_USING_INDEX(false),
 
+    /**
+     * Feature that determines whether {link Enum}s
+     * used as {@link java.util.Map} keys are serialized
+     * as using {@link Enum#ordinal()} or not.
+     * Similar to {@link #WRITE_ENUMS_USING_INDEX} used when writing
+     * {@link Enum}s as regular values.
+     *<p>
+     * Feature is disabled by default.
+     * 
+     * @since 2.10
+     */
+    WRITE_ENUM_KEYS_USING_INDEX(false),
+    
     /**
      * Feature that determines whether Map entries with null values are
      * to be serialized (true) or not (false).
      *<p>
+     * NOTE: unlike other {@link SerializationFeature}s, this feature <b>cannot</b> be
+     * dynamically changed on per-call basis, because its effect is considered during
+     * construction of serializers and property handlers.
+     *<p>
      * Feature is enabled by default.
+     *
+     * @deprecated Since 2.9 there are better mechanism for specifying filtering; specifically
+     *   using {@link com.fasterxml.jackson.annotation.JsonInclude} or configuration overrides
+     *   (see {@link ObjectMapper#configOverride(Class)}}).
      */
+    @Deprecated // since 2.9
     WRITE_NULL_MAP_VALUES(true),
 
     /**
@@ -288,7 +328,7 @@ public enum SerializationFeature implements ConfigFeature
      * Note that this does not change behavior of {@link java.util.Map}s, or
      * "Collection-like" types.
      *<p>
-     * NOTE: unlike other {@link SerializationFeature}s, this feature <b>can not</b> be
+     * NOTE: unlike other {@link SerializationFeature}s, this feature <b>cannot</b> be
      * dynamically changed on per-call basis, because its effect is considered during
      * construction of serializers and property handlers.
      *<p>
@@ -351,7 +391,7 @@ public enum SerializationFeature implements ConfigFeature
      * and this setting <b>has no effect</b> on such types.
      *<p>
      * If disabled, standard millisecond timestamps are assumed.
-     * This is the counterpart to {@link SerializationFeature#WRITE_DATE_TIMESTAMPS_AS_NANOSECONDS}.
+     * This is the counterpart to {@link DeserializationFeature#READ_DATE_TIMESTAMPS_AS_NANOSECONDS}.
      *<p>
      * Feature is enabled by default, to support most accurate time values possible.
      *
