@@ -10,6 +10,7 @@ import com.fasterxml.jackson.core.Base64Variant;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.introspect.ClassIntrospector;
 import com.fasterxml.jackson.databind.introspect.ClassIntrospector.MixInResolver;
+import com.fasterxml.jackson.databind.introspect.AccessorNamingStrategy;
 import com.fasterxml.jackson.databind.introspect.AnnotatedClass;
 import com.fasterxml.jackson.databind.introspect.SimpleMixInResolver;
 import com.fasterxml.jackson.databind.introspect.VisibilityChecker;
@@ -137,17 +138,17 @@ public abstract class MapperConfigBase<CFG extends ConfigFeature,
      * Copy constructor usually called to make a copy for use by
      * ObjectMapper that is copied.
      *
-     * @since 2.8
+     * @since 2.11.2
      */
     protected MapperConfigBase(MapperConfigBase<CFG,T> src,
-            SimpleMixInResolver mixins, RootNameLookup rootNames,
+            SubtypeResolver str, SimpleMixInResolver mixins, RootNameLookup rootNames,
             ConfigOverrides configOverrides)
     {
         // 18-Apr-2018, tatu: [databind#1898] need to force copying of `ClassIntrospector`
         //    (to clear its cache) to avoid leakage
         super(src, src._base.copy());
         _mixIns = mixins;
-        _subtypeResolver = src._subtypeResolver;
+        _subtypeResolver = str;
         _rootNames = rootNames;
         _rootName = src._rootName;
         _view = src._view;
@@ -457,6 +458,17 @@ public abstract class MapperConfigBase<CFG extends ConfigFeature,
         return _withBase(_base.withPropertyNamingStrategy(pns));
     }
 
+    /**
+     * Method for constructing and returning a new instance with different
+     * {@link PropertyNamingStrategy}
+     * to use.
+     *
+     * @since 2.12
+     */
+    public final T with(AccessorNamingStrategy.Provider p) {
+        return _withBase(_base.withAccessorNaming(p));
+    }
+
     /**
      * Method for constructing and returning a new instance with different
      * {@link HandlerInstantiator}
@@ -637,14 +649,7 @@ public abstract class MapperConfigBase<CFG extends ConfigFeature,
 
     @Override
     public final JsonFormat.Value getDefaultPropertyFormat(Class<?> type) {
-        ConfigOverride overrides = _configOverrides.findOverride(type);
-        if (overrides != null) {
-            JsonFormat.Value v = overrides.getFormat();
-            if (v != null) {
-                return v;
-            }
-        }
-        return EMPTY_FORMAT;
+        return _configOverrides.findFormatDefaults(type);
     }
 
     @Override
@@ -667,11 +672,19 @@ public abstract class MapperConfigBase<CFG extends ConfigFeature,
     {
         AnnotationIntrospector intr = getAnnotationIntrospector();
         JsonIgnoreProperties.Value base = (intr == null) ? null
-                : intr.findPropertyIgnorals(actualClass);
+                : intr.findPropertyIgnoralByName(this, actualClass);
         JsonIgnoreProperties.Value overrides = getDefaultPropertyIgnorals(baseType);
         return JsonIgnoreProperties.Value.merge(base, overrides);
     }
 
+    @Override
+    public final JsonIncludeProperties.Value getDefaultPropertyInclusions(Class<?> baseType,
+            AnnotatedClass actualClass)
+    {
+        AnnotationIntrospector intr = getAnnotationIntrospector();
+        return (intr == null) ? null : intr.findPropertyInclusionByName(this, actualClass);
+    }
+
     @Override
     public final VisibilityChecker<?> getDefaultVisibilityChecker()
     {
