@@ -5,8 +5,10 @@ import java.util.*;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.cfg.CoercionAction;
 import com.fasterxml.jackson.databind.deser.impl.*;
 import com.fasterxml.jackson.databind.deser.impl.ReadableObjectId.Referring;
+import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 import com.fasterxml.jackson.databind.util.TokenBuffer;
 
@@ -55,14 +57,31 @@ public class BeanDeserializer
 
     /**
      * Constructor used by {@link BeanDeserializerBuilder}.
+     *
+     * @deprecated in 2.12, remove from 3.0
      */
+    @Deprecated
     public BeanDeserializer(BeanDeserializerBuilder builder, BeanDescription beanDesc,
             BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
             HashSet<String> ignorableProps, boolean ignoreAllUnknown,
             boolean hasViews)
     {
         super(builder, beanDesc, properties, backRefs,
-                ignorableProps, ignoreAllUnknown, hasViews);
+                ignorableProps, ignoreAllUnknown, null, hasViews);
+    }
+
+    /**
+     * Constructor used by {@link BeanDeserializerBuilder}.
+     *
+     * @since 2.12
+     */
+    public BeanDeserializer(BeanDeserializerBuilder builder, BeanDescription beanDesc,
+                            BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
+                            HashSet<String> ignorableProps, boolean ignoreAllUnknown, Set<String> includableProps,
+                            boolean hasViews)
+    {
+        super(builder, beanDesc, properties, backRefs,
+                ignorableProps, ignoreAllUnknown, includableProps, hasViews);
     }
 
     /**
@@ -85,10 +104,21 @@ public class BeanDeserializer
         super(src, oir);
     }
 
+    /**
+     * @deprecated in 2.12, remove from 3.0
+     */
+    @Deprecated
     public BeanDeserializer(BeanDeserializerBase src, Set<String> ignorableProps) {
         super(src, ignorableProps);
     }
 
+    /**
+     * @since 2.12
+     */
+    public BeanDeserializer(BeanDeserializerBase src, Set<String> ignorableProps, Set<String> includableProps) {
+        super(src, ignorableProps, includableProps);
+    }
+
     public BeanDeserializer(BeanDeserializerBase src, BeanPropertyMap props) {
         super(src, props);
     }
@@ -118,8 +148,14 @@ public class BeanDeserializer
     }
 
     @Override
-    public BeanDeserializer withIgnorableProperties(Set<String> ignorableProps) {
-        return new BeanDeserializer(this, ignorableProps);
+    public BeanDeserializer withByNameInclusion(Set<String> ignorableProps,
+            Set<String> includableProps) {
+        return new BeanDeserializer(this, ignorableProps, includableProps);
+    }
+
+    @Override
+    public BeanDeserializerBase withIgnoreAllUnknown(boolean ignoreUnknown) {
+        return new BeanDeserializer(this, ignoreUnknown);
     }
 
     @Override
@@ -158,7 +194,7 @@ public class BeanDeserializer
             }
             return deserializeFromObject(p, ctxt);
         }
-        return _deserializeOther(p, ctxt, p.getCurrentToken());
+        return _deserializeOther(p, ctxt, p.currentToken());
     }
 
     protected final Object _deserializeOther(JsonParser p, DeserializationContext ctxt,
@@ -181,8 +217,8 @@ public class BeanDeserializer
             case VALUE_NULL:
                 return deserializeFromNull(p, ctxt);
             case START_ARRAY:
-                // these only work if there's a (delegating) creator...
-                return deserializeFromArray(p, ctxt);
+                // these only work if there's a (delegating) creator, or UNWRAP_SINGLE_ARRAY
+                return _deserializeFromArray(p, ctxt);
             case FIELD_NAME:
             case END_OBJECT: // added to resolve [JACKSON-319], possible related issues
                 if (_vanillaProcessing) {
@@ -195,7 +231,7 @@ public class BeanDeserializer
             default:
             }
         }
-        return ctxt.handleUnexpectedToken(handledType(), p);
+        return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
     }
 
     @Deprecated // since 2.8; remove unless getting used
@@ -232,7 +268,7 @@ public class BeanDeserializer
             }
         } else {
             if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-                propName = p.getCurrentName();
+                propName = p.currentName();
             } else {
                 return bean;
             }
@@ -278,7 +314,7 @@ public class BeanDeserializer
         // [databind#631]: Assign current value, to be accessible by custom serializers
         p.setCurrentValue(bean);
         if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             do {
                 p.nextToken();
                 SettableBeanProperty prop = _beanProperties.find(propName);
@@ -312,7 +348,7 @@ public class BeanDeserializer
          */
         if ((_objectIdReader != null) && _objectIdReader.maySerializeAsObject()) {
             if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)
-                    && _objectIdReader.isValidReferencePropertyName(p.getCurrentName(), p)) {
+                    && _objectIdReader.isValidReferencePropertyName(p.currentName(), p)) {
                 return deserializeFromObjectId(p, ctxt);
             }
         }
@@ -324,9 +360,6 @@ public class BeanDeserializer
                 return deserializeWithExternalTypeId(p, ctxt);
             }
             Object bean = deserializeFromObjectUsingNonDefault(p, ctxt);
-            if (_injectables != null) {
-                injectValues(ctxt, bean);
-            }
             /* 27-May-2014, tatu: I don't think view processing would work
              *   at this point, so commenting it out; but leaving in place
              *   just in case I forgot something fundamental...
@@ -360,7 +393,7 @@ public class BeanDeserializer
             }
         }
         if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             do {
                 p.nextToken();
                 SettableBeanProperty prop = _beanProperties.find(propName);
@@ -387,7 +420,6 @@ public class BeanDeserializer
      * as well.
      */
     @Override
-    @SuppressWarnings("resource")
     protected Object _deserializeUsingPropertyBased(final JsonParser p, final DeserializationContext ctxt)
         throws IOException
     {
@@ -396,17 +428,17 @@ public class BeanDeserializer
         TokenBuffer unknown = null;
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
 
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         List<BeanReferring> referrings = null;
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             p.nextToken(); // to point to value
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             // Object Id property?
-            if (buffer.readIdProperty(propName)) {
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
                 continue;
             }
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
             if (creatorProp != null) {
                 // Last creator property to set?
                 Object value;
@@ -461,7 +493,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -485,11 +517,16 @@ public class BeanDeserializer
         // We hit END_OBJECT, so:
         Object bean;
         try {
-            bean =  creator.build(ctxt, buffer);
+            bean = creator.build(ctxt, buffer);
         } catch (Exception e) {
             wrapInstantiationProblem(e, ctxt);
             bean = null; // never gets here
         }
+        // 13-Apr-2020, tatu: [databind#2678] need to handle injection here
+        if (_injectables != null) {
+            injectValues(ctxt, bean);
+        }
+
         if (referrings != null) {
             for (BeanReferring referring : referrings) {
                referring.setBean(bean);
@@ -559,7 +596,48 @@ public class BeanDeserializer
             p2.close();
             return ob;
         }
-        return ctxt.handleUnexpectedToken(handledType(), p);
+        return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
+    }
+
+    @Override
+    protected Object _deserializeFromArray(JsonParser p, DeserializationContext ctxt) throws IOException
+    {
+        // note: cannot call `_delegateDeserializer()` since order reversed here:
+        JsonDeserializer<Object> delegateDeser = _arrayDelegateDeserializer;
+        // fallback to non-array delegate
+        if ((delegateDeser != null) || ((delegateDeser = _delegateDeserializer) != null)) {
+            Object bean = _valueInstantiator.createUsingArrayDelegate(ctxt,
+                    delegateDeser.deserialize(p, ctxt));
+            if (_injectables != null) {
+                injectValues(ctxt, bean);
+            }
+            return bean;
+        }
+        final CoercionAction act = _findCoercionFromEmptyArray(ctxt);
+        final boolean unwrap = ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
+
+        if (unwrap || (act != CoercionAction.Fail)) {
+            JsonToken t = p.nextToken();
+            if (t == JsonToken.END_ARRAY) {
+                switch (act) {
+                case AsEmpty:
+                    return getEmptyValue(ctxt);
+                case AsNull:
+                case TryConvert:
+                    return getNullValue(ctxt);
+                default:
+                }
+                return ctxt.handleUnexpectedToken(getValueType(ctxt), JsonToken.START_ARRAY, p, null);
+            }
+            if (unwrap) {
+                final Object value = deserialize(p, ctxt);
+                if (p.nextToken() != JsonToken.END_ARRAY) {
+                    handleMissingEndArrayForSingle(p, ctxt);
+                }
+                return value;
+            }
+        }
+        return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
     }
 
     /*
@@ -573,7 +651,7 @@ public class BeanDeserializer
         throws IOException
     {
         if (p.hasTokenId(JsonTokenId.ID_FIELD_NAME)) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             do {
                 p.nextToken();
                 // TODO: 06-Jan-2015, tatu: try streamlining call sequences here as well
@@ -627,7 +705,7 @@ public class BeanDeserializer
             injectValues(ctxt, bean);
         }
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        String propName = p.hasTokenId(JsonTokenId.ID_FIELD_NAME) ? p.getCurrentName() : null;
+        String propName = p.hasTokenId(JsonTokenId.ID_FIELD_NAME) ? p.currentName() : null;
 
         for (; propName != null; propName = p.nextFieldName()) {
             p.nextToken();
@@ -645,7 +723,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
@@ -679,7 +757,7 @@ public class BeanDeserializer
             Object bean)
         throws IOException
     {
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         if (t == JsonToken.START_OBJECT) {
             t = p.nextToken();
         }
@@ -687,7 +765,7 @@ public class BeanDeserializer
         tokens.writeStartObject();
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             SettableBeanProperty prop = _beanProperties.find(propName);
             p.nextToken();
             if (prop != null) { // normal case
@@ -702,7 +780,7 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
@@ -746,12 +824,16 @@ public class BeanDeserializer
         TokenBuffer tokens = new TokenBuffer(p, ctxt);
         tokens.writeStartObject();
 
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             p.nextToken(); // to point to value
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            // Object Id property?
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
+                continue;
+            }
             if (creatorProp != null) {
                 // Last creator property to set?
                 if (buffer.assignParameter(creatorProp,
@@ -790,10 +872,6 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            // Object Id property?
-            if (buffer.readIdProperty(propName)) {
-                continue;
-            }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
@@ -801,7 +879,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -872,8 +950,8 @@ public class BeanDeserializer
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         final ExternalTypeHandler ext = _externalTypeIdHandler.start();
 
-        for (JsonToken t = p.getCurrentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+        for (JsonToken t = p.currentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
+            String propName = p.currentName();
             t = p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) { // normal case
@@ -893,7 +971,7 @@ public class BeanDeserializer
                 continue;
             }
             // ignorable things should be ignored
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
@@ -928,12 +1006,16 @@ public class BeanDeserializer
         TokenBuffer tokens = new TokenBuffer(p, ctxt);
         tokens.writeStartObject();
 
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             p.nextToken(); // to point to value
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            // Object Id property?
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
+                continue;
+            }
             if (creatorProp != null) {
                 // first: let's check to see if this might be part of value with external type id:
                 // 11-Sep-2015, tatu: Important; do NOT pass buffer as last arg, but null,
@@ -969,10 +1051,6 @@ public class BeanDeserializer
                 }
                 continue;
             }
-            // Object Id property?
-            if (buffer.readIdProperty(propName)) {
-                continue;
-            }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
@@ -984,7 +1062,7 @@ public class BeanDeserializer
                 continue;
             }
             // Things marked as ignorable should not be passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -992,7 +1070,10 @@ public class BeanDeserializer
             if (_anySetter != null) {
                 buffer.bufferAnyProperty(_anySetter, propName,
                         _anySetter.deserialize(p, ctxt));
+                continue;
             }
+            // Unknown: let's call handler method
+            handleUnknownProperty(p, ctxt, _valueClass, propName);
         }
         tokens.writeEndObject();
 
