@@ -18,6 +18,7 @@ import com.fasterxml.jackson.databind.exc.IgnoredPropertyException;
 import com.fasterxml.jackson.databind.introspect.*;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
 import com.fasterxml.jackson.databind.type.ClassKey;
+import com.fasterxml.jackson.databind.type.LogicalType;
 import com.fasterxml.jackson.databind.util.*;
 
 /**
@@ -84,9 +85,13 @@ public abstract class BeanDeserializerBase
     protected PropertyBasedCreator _propertyBasedCreator;
 
     /**
-     * Flag that is set to mark "non-standard" cases; where either
-     * we use one of non-default creators, or there are unwrapped
-     * values to consider.
+     * Flag that is set to mark cases where deserialization from Object value
+     * using otherwise "standard" property binding will need to use non-default
+     * creation method: namely, either "full" delegation (array-delegation does
+     * not apply), or properties-based Creator method is used.
+     *<p>
+     * Note that flag is somewhat mis-named as it is not affected by scalar-delegating
+     * creators; it only has effect on Object Value binding.
      */
     protected boolean _nonStandardCreation;
 
@@ -130,6 +135,11 @@ public abstract class BeanDeserializerBase
      */
     final protected Set<String> _ignorableProps;
 
+    /**
+     * Keep track of the the properties that needs to be specifically included.
+     */
+    final protected Set<String> _includableProps;
+
     /**
      * Flag that can be set to ignore and skip unknown properties.
      * If set, will not throw an exception for unknown properties.
@@ -196,6 +206,7 @@ public abstract class BeanDeserializerBase
             BeanDescription beanDesc,
             BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
             Set<String> ignorableProps, boolean ignoreAllUnknown,
+            Set<String> includableProps,
             boolean hasViews)
     {
         super(beanDesc.getType());
@@ -206,22 +217,31 @@ public abstract class BeanDeserializerBase
         _backRefs = backRefs;
         _ignorableProps = ignorableProps;
         _ignoreAllUnknown = ignoreAllUnknown;
+        _includableProps = includableProps;
 
         _anySetter = builder.getAnySetter();
         List<ValueInjector> injectables = builder.getInjectables();
         _injectables = (injectables == null || injectables.isEmpty()) ? null
                 : injectables.toArray(new ValueInjector[injectables.size()]);
         _objectIdReader = builder.getObjectIdReader();
+
+        // 02-May-2020, tatu: This boolean setting is only used when binding from
+        //    Object value, and hence does not consider "array-delegating" or various
+        //    scalar-delegation cases. It is set when default (0-argument) constructor
+        //    is NOT to be used when binding an Object value (or in case of
+        //    POJO-as-array, Array value).
         _nonStandardCreation = (_unwrappedPropertyHandler != null)
             || _valueInstantiator.canCreateUsingDelegate()
-            || _valueInstantiator.canCreateUsingArrayDelegate() // new in 2.7
+        // [databind#2486]: as per above, array-delegating creator should not be considered
+        //   as doing so will prevent use of Array-or-standard-Object deserialization
+        //            || _valueInstantiator.canCreateUsingArrayDelegate()
             || _valueInstantiator.canCreateFromObjectWith()
             || !_valueInstantiator.canCreateUsingDefault()
             ;
 
         // Any transformation we may need to apply?
-        JsonFormat.Value format = beanDesc.findExpectedFormat(null);
-        _serializationShape = (format == null) ? null : format.getShape();
+        final JsonFormat.Value format = beanDesc.findExpectedFormat(null);
+        _serializationShape = format.getShape();
 
         _needViewProcesing = hasViews;
         _vanillaProcessing = !_nonStandardCreation
@@ -250,6 +270,7 @@ public abstract class BeanDeserializerBase
         _backRefs = src._backRefs;
         _ignorableProps = src._ignorableProps;
         _ignoreAllUnknown = ignoreAllUnknown;
+        _includableProps = src._includableProps;
         _anySetter = src._anySetter;
         _injectables = src._injectables;
         _objectIdReader = src._objectIdReader;
@@ -275,6 +296,7 @@ public abstract class BeanDeserializerBase
         _backRefs = src._backRefs;
         _ignorableProps = src._ignorableProps;
         _ignoreAllUnknown = (unwrapper != null) || src._ignoreAllUnknown;
+        _includableProps = src._includableProps;
         _anySetter = src._anySetter;
         _injectables = src._injectables;
         _objectIdReader = src._objectIdReader;
@@ -312,6 +334,7 @@ public abstract class BeanDeserializerBase
         _backRefs = src._backRefs;
         _ignorableProps = src._ignorableProps;
         _ignoreAllUnknown = src._ignoreAllUnknown;
+        _includableProps = src._includableProps;
         _anySetter = src._anySetter;
         _injectables = src._injectables;
         
@@ -337,7 +360,11 @@ public abstract class BeanDeserializerBase
         }
     }
 
-    public BeanDeserializerBase(BeanDeserializerBase src, Set<String> ignorableProps)
+    /**
+     * @since 2.12
+     */
+    public BeanDeserializerBase(BeanDeserializerBase src,
+            Set<String> ignorableProps, Set<String> includableProps)
     {
         super(src._beanType);
         _beanType = src._beanType;
@@ -349,6 +376,7 @@ public abstract class BeanDeserializerBase
         _backRefs = src._backRefs;
         _ignorableProps = ignorableProps;
         _ignoreAllUnknown = src._ignoreAllUnknown;
+        _includableProps = includableProps;
         _anySetter = src._anySetter;
         _injectables = src._injectables;
 
@@ -362,7 +390,7 @@ public abstract class BeanDeserializerBase
 
         // 01-May-2016, tatu: [databind#1217]: Remove properties from mapping,
         //    to avoid them being deserialized
-        _beanProperties = src._beanProperties.withoutProperties(ignorableProps);
+        _beanProperties = src._beanProperties.withoutProperties(ignorableProps, includableProps);
     }
 
     /**
@@ -381,6 +409,7 @@ public abstract class BeanDeserializerBase
         _backRefs = src._backRefs;
         _ignorableProps = src._ignorableProps;
         _ignoreAllUnknown = src._ignoreAllUnknown;
+        _includableProps = src._includableProps;
         _anySetter = src._anySetter;
         _injectables = src._injectables;
         _objectIdReader = src._objectIdReader;
@@ -392,13 +421,27 @@ public abstract class BeanDeserializerBase
 
         _vanillaProcessing = src._vanillaProcessing;
     }
+
+    @Deprecated // since 2.12
+    protected BeanDeserializerBase(BeanDeserializerBase src, Set<String> ignorableProps)
+    {
+        this(src, ignorableProps, src._includableProps);
+    }
     
     @Override
     public abstract JsonDeserializer<Object> unwrappingDeserializer(NameTransformer unwrapper);
 
     public abstract BeanDeserializerBase withObjectIdReader(ObjectIdReader oir);
 
-    public abstract BeanDeserializerBase withIgnorableProperties(Set<String> ignorableProps);
+    /**
+     * @since 2.12
+     */
+    public abstract BeanDeserializerBase withByNameInclusion(Set<String> ignorableProps, Set<String> includableProps);
+
+    /**
+     * @since 2.11
+     */
+    public abstract BeanDeserializerBase withIgnoreAllUnknown(boolean ignoreUnknown);
 
     /**
      * Mutant factory method that custom sub-classes must override; not left as
@@ -420,6 +463,14 @@ public abstract class BeanDeserializerBase
      */
     protected abstract BeanDeserializerBase asArrayDeserializer();
 
+    /**
+     * @deprecated Since 2.12 use {@link #withByNameInclusion} instead
+     */
+    @Deprecated
+    public BeanDeserializerBase withIgnorableProperties(Set<String> ignorableProps) {
+        return withByNameInclusion(ignorableProps, _includableProps);
+    }
+
     /*
     /**********************************************************
     /* Validation, post-processing
@@ -444,10 +495,10 @@ public abstract class BeanDeserializerBase
             // 22-Jan-2018, tatu: May need to propagate "ignorable" status (from `Access.READ_ONLY`
             //     or perhaps class-ignorables) into Creator properties too. Can not just delete,
             //     at this point, but is needed for further processing down the line
-            if (_ignorableProps != null) {
+            if (_ignorableProps != null || _includableProps != null) {
                 for (int i = 0, end = creatorProps.length; i < end; ++i) {
                     SettableBeanProperty prop  = creatorProps[i];
-                    if (_ignorableProps.contains(prop.getName())) {
+                    if (IgnorePropertiesUtil.shouldIgnore(prop.getName(), _ignorableProps, _includableProps)) {
                         creatorProps[i].markAsIgnorable();
                     }
                 }
@@ -547,7 +598,7 @@ public abstract class BeanDeserializerBase
             if (delegateType == null) {
                 ctxt.reportBadDefinition(_beanType, String.format(
 "Invalid delegate-creator definition for %s: value instantiator (%s) returned true for 'canCreateUsingDelegate()', but null for 'getDelegateType()'",
-                    _beanType, _valueInstantiator.getClass().getName()));
+ClassUtil.getTypeDescription(_beanType), ClassUtil.classNameOf(_valueInstantiator)));
             }
             _delegateDeserializer = _findDelegateDeserializer(ctxt, delegateType,
                     _valueInstantiator.getDelegateCreator());
@@ -559,7 +610,7 @@ public abstract class BeanDeserializerBase
             if (delegateType == null) {
                 ctxt.reportBadDefinition(_beanType, String.format(
 "Invalid delegate-creator definition for %s: value instantiator (%s) returned true for 'canCreateUsingArrayDelegate()', but null for 'getArrayDelegateType()'",
-                        _beanType, _valueInstantiator.getClass().getName()));
+ClassUtil.getTypeDescription(_beanType), ClassUtil.classNameOf(_valueInstantiator)));
             }
             _arrayDelegateDeserializer = _findDelegateDeserializer(ctxt, delegateType,
                     _valueInstantiator.getArrayDelegateCreator());
@@ -706,8 +757,8 @@ public abstract class BeanDeserializerBase
                     idProp = findProperty(propName);
                     if (idProp == null) {
                         ctxt.reportBadDefinition(_beanType, String.format(
-                                "Invalid Object Id definition for %s: cannot find property with name '%s'",
-                                handledType().getName(), propName));
+"Invalid Object Id definition for %s: cannot find property with name %s",
+ClassUtil.nameOf(handledType()), ClassUtil.name(propName)));
                     }
                     idType = idProp.getType();
                     idGen = new PropertyBasedObjectIdGenerator(objectIdInfo.getScope());
@@ -729,18 +780,7 @@ public abstract class BeanDeserializerBase
         }
         // And possibly add more properties to ignore
         if (accessor != null) {
-            JsonIgnoreProperties.Value ignorals = intr.findPropertyIgnorals(accessor);
-            if (ignorals != null) {
-                Set<String> ignored = ignorals.findIgnoredForDeserialization();
-                if (!ignored.isEmpty()) {
-                    Set<String> prev = contextual._ignorableProps;
-                    if ((prev != null) && !prev.isEmpty()) {
-                        ignored = new HashSet<String>(ignored);
-                        ignored.addAll(prev);
-                    }
-                    contextual = contextual.withIgnorableProperties(ignored);
-                }
-            }
+            contextual = _handleByNameInclusion(ctxt, intr, contextual, accessor);
         }
 
         // One more thing: are we asked to serialize POJO as array?
@@ -770,6 +810,47 @@ public abstract class BeanDeserializerBase
         return contextual;
     }
 
+    // @since 2.12
+    protected BeanDeserializerBase _handleByNameInclusion(DeserializationContext ctxt,
+            AnnotationIntrospector intr,
+            BeanDeserializerBase contextual,
+            AnnotatedMember accessor) throws JsonMappingException
+    {
+        final DeserializationConfig config = ctxt.getConfig();
+        JsonIgnoreProperties.Value ignorals = intr.findPropertyIgnoralByName(config, accessor);
+
+        // 30-Mar-2020, tatu: As per [databind#2627], need to also allow
+        //    per-property override to "ignore all unknown".
+        //  NOTE: there is no way to override with `false` because annotation
+        //  defaults to `false` (i.e. can not know if `false` is explicit value)
+        if (ignorals.getIgnoreUnknown() && !_ignoreAllUnknown) {
+            contextual = contextual.withIgnoreAllUnknown(true);
+        }
+
+        final Set<String> namesToIgnore = ignorals.findIgnoredForDeserialization();
+        final Set<String> prevNamesToIgnore = contextual._ignorableProps;
+        final Set<String> newNamesToIgnore;
+
+        if (namesToIgnore.isEmpty()) {
+            newNamesToIgnore = prevNamesToIgnore;
+        } else if ((prevNamesToIgnore == null) || prevNamesToIgnore.isEmpty()) {
+            newNamesToIgnore = namesToIgnore;
+        } else {
+            newNamesToIgnore = new HashSet<String>(prevNamesToIgnore);
+            newNamesToIgnore.addAll(namesToIgnore);
+        }
+
+        final Set<String> prevNamesToInclude = contextual._includableProps;
+        final Set<String> newNamesToInclude = IgnorePropertiesUtil.combineNamesToInclude(prevNamesToInclude,
+                intr.findPropertyInclusionByName(config, accessor).getIncluded());
+
+        if ((newNamesToIgnore != prevNamesToIgnore)
+                || (newNamesToInclude != prevNamesToInclude)) {
+            contextual = contextual.withByNameInclusion(newNamesToIgnore, newNamesToInclude);
+        }
+        return contextual;
+    }
+
     /**
      * Helper method called to see if given property is part of 'managed' property
      * pair (managed + back reference), and if so, handle resolution details.
@@ -786,8 +867,8 @@ public abstract class BeanDeserializerBase
         SettableBeanProperty backProp = valueDeser.findBackReference(refName);
         if (backProp == null) {
             ctxt.reportBadDefinition(_beanType, String.format(
-"Cannot handle managed/back reference '%s': no back reference property found from type %s",
-                    refName, prop.getType()));
+"Cannot handle managed/back reference %s: no back reference property found from type %s",
+ClassUtil.name(refName), ClassUtil.getTypeDescription(prop.getType())));
         }
         // also: verify that type is compatible
         JavaType referredType = _beanType;
@@ -795,8 +876,8 @@ public abstract class BeanDeserializerBase
         boolean isContainer = prop.getType().isContainerType();
         if (!backRefType.getRawClass().isAssignableFrom(referredType.getRawClass())) {
             ctxt.reportBadDefinition(_beanType, String.format(
-"Cannot handle managed/back reference '%s': back reference type (%s) not compatible with managed type (%s)",
-                    refName, backRefType.getRawClass().getName(),
+"Cannot handle managed/back reference %s: back reference type (%s) not compatible with managed type (%s)",
+ClassUtil.name(refName), ClassUtil.getTypeDescription(backRefType),
                     referredType.getRawClass().getName()));
         }
         return new ManagedReferenceProperty(prop, refName, backProp, isContainer);
@@ -899,9 +980,9 @@ public abstract class BeanDeserializerBase
                 }
             } else if (!mayMerge.booleanValue()) { // prevented
                 if (!merge.fromDefaults) {
-                    // If attempts was made via explicit annotation/per-type config override,
+                    // If attempt was made via explicit annotation/per-type config override,
                     // should be reported; may or may not result in exception
-                    ctxt.reportBadMerge(valueDeser);
+                    ctxt.handleBadMerge(valueDeser);
                 }
                 return prop;
             }
@@ -958,6 +1039,19 @@ public abstract class BeanDeserializerBase
     @Override
     public boolean isCachable() { return true; }
 
+    /**
+     * Accessor for checking whether this deserializer is operating
+     * in case-insensitive manner.
+     * 
+     * @return True if this deserializer should match property names without
+     *    considering casing; false if case has to match exactly.
+     *
+     * @since 2.12
+     */
+    public boolean isCaseInsensitive() {
+        return _beanProperties.isCaseInsensitive();
+    }
+
     @Override // since 2.9
     public Boolean supportsUpdate(DeserializationConfig config) {
         // although with possible caveats, yes, values can be updated
@@ -1014,6 +1108,11 @@ public abstract class BeanDeserializerBase
     @Override
     public JavaType getValueType() { return _beanType; }
 
+    @Override // since 2.12
+    public LogicalType logicalType() {
+        return LogicalType.POJO;
+    }
+
     /**
      * Accessor for iterating over properties this deserializer uses; with
      * the exception that properties passed via Creator methods
@@ -1158,7 +1257,7 @@ public abstract class BeanDeserializerBase
                 }
             }
             // or, Object Ids Jackson explicitly sets
-            JsonToken t = p.getCurrentToken();
+            JsonToken t = p.currentToken();
             if (t != null) {
                 // Most commonly, a scalar (int id, uuid String, ...)
                 if (t.isScalarValue()) {
@@ -1169,7 +1268,7 @@ public abstract class BeanDeserializerBase
                     t = p.nextToken();
                 }
                 if ((t == JsonToken.FIELD_NAME) && _objectIdReader.maySerializeAsObject()
-                        && _objectIdReader.isValidReferencePropertyName(p.getCurrentName(), p)) {
+                        && _objectIdReader.isValidReferencePropertyName(p.currentName(), p)) {
                     return deserializeFromObjectId(p, ctxt);
                 }
             }
@@ -1280,8 +1379,12 @@ public abstract class BeanDeserializerBase
     {
         final JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
         if (delegateDeser != null) {
-            return _valueInstantiator.createUsingDelegate(ctxt,
+            final Object bean = _valueInstantiator.createUsingDelegate(ctxt,
                     delegateDeser.deserialize(p, ctxt));
+            if (_injectables != null) {
+                injectValues(ctxt, bean);
+            }
+            return bean;
         }
         if (_propertyBasedCreator != null) {
             return _deserializeUsingPropertyBased(p, ctxt);
@@ -1292,10 +1395,10 @@ public abstract class BeanDeserializerBase
         Class<?> raw = _beanType.getRawClass();
         if (ClassUtil.isNonStaticInnerClass(raw)) {
             return ctxt.handleMissingInstantiator(raw, null, p,
-"can only instantiate non-static inner class by using default, no-argument constructor");
+"non-static inner classes like this can only by instantiated using default, no-argument constructor");
         }
         return ctxt.handleMissingInstantiator(raw, getValueInstantiator(), p,
-                "cannot deserialize from Object value (no delegate- or property-based Creator)");
+"cannot deserialize from Object value (no delegate- or property-based Creator)");
     }
 
     protected abstract Object _deserializeUsingPropertyBased(final JsonParser p,
@@ -1336,15 +1439,19 @@ public abstract class BeanDeserializerBase
             }
             return _valueInstantiator.createFromLong(ctxt, p.getLongValue());
         }
-        // actually, could also be BigInteger, so:
-        if (delegateDeser != null) {
-            Object bean = _valueInstantiator.createUsingDelegate(ctxt,
-                    delegateDeser.deserialize(p, ctxt));
-            if (_injectables != null) {
-                injectValues(ctxt, bean);
+        if (nt == NumberType.BIG_INTEGER) {
+            if (delegateDeser != null) {
+                if (!_valueInstantiator.canCreateFromBigInteger()) {
+                    Object bean = _valueInstantiator.createUsingDelegate(ctxt, delegateDeser.deserialize(p, ctxt));
+                    if (_injectables != null) {
+                        injectValues(ctxt, bean);
+                    }
+                    return bean;
+                }
             }
-            return bean;
+            return _valueInstantiator.createFromBigInteger(ctxt, p.getBigIntegerValue());
         }
+
         return ctxt.handleMissingInstantiator(handledType(), getValueInstantiator(), p,
                 "no suitable creator method found to deserialize from Number value (%s)",
                 p.getNumberValue());
@@ -1370,7 +1477,7 @@ public abstract class BeanDeserializerBase
                 return bean;
             }
         }
-        return _valueInstantiator.createFromString(ctxt, p.getText());
+        return _deserializeFromString(p, ctxt);
     }
 
     /**
@@ -1395,12 +1502,22 @@ public abstract class BeanDeserializerBase
             }
             return _valueInstantiator.createFromDouble(ctxt, p.getDoubleValue());
         }
-        // actually, could also be BigDecimal, so:
-        JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
-        if (delegateDeser != null) {
-            return _valueInstantiator.createUsingDelegate(ctxt,
-                    delegateDeser.deserialize(p, ctxt));
+
+        if (t == NumberType.BIG_DECIMAL) {
+            JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
+            if (delegateDeser != null) {
+                if (!_valueInstantiator.canCreateFromBigDecimal()) {
+                    Object bean = _valueInstantiator.createUsingDelegate(ctxt, delegateDeser.deserialize(p, ctxt));
+                    if (_injectables != null) {
+                        injectValues(ctxt, bean);
+                    }
+                    return bean;
+                }
+            }
+
+            return _valueInstantiator.createFromBigDecimal(ctxt, p.getDecimalValue());
         }
+
         return ctxt.handleMissingInstantiator(handledType(), getValueInstantiator(), p,
                 "no suitable creator method found to deserialize from Number value (%s)",
                 p.getNumberValue());
@@ -1422,43 +1539,19 @@ public abstract class BeanDeserializerBase
                 return bean;
             }
         }
-        boolean value = (p.getCurrentToken() == JsonToken.VALUE_TRUE);
+        boolean value = (p.currentToken() == JsonToken.VALUE_TRUE);
         return _valueInstantiator.createFromBoolean(ctxt, value);
     }
 
-    public Object deserializeFromArray(JsonParser p, DeserializationContext ctxt) throws IOException
-    {
-        // note: cannot call `_delegateDeserializer()` since order reversed here:
-        JsonDeserializer<Object> delegateDeser = _arrayDelegateDeserializer;
-        // fallback to non-array delegate
-        if ((delegateDeser != null) || ((delegateDeser = _delegateDeserializer) != null)) {
-            Object bean = _valueInstantiator.createUsingArrayDelegate(ctxt,
-                    delegateDeser.deserialize(p, ctxt));
-            if (_injectables != null) {
-                injectValues(ctxt, bean);
-            }
-            return bean;
-        }
-        if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
-            JsonToken t = p.nextToken();
-            if (t == JsonToken.END_ARRAY && ctxt.isEnabled(DeserializationFeature.ACCEPT_EMPTY_ARRAY_AS_NULL_OBJECT)) {
-                return null;
-            }
-            final Object value = deserialize(p, ctxt);
-            if (p.nextToken() != JsonToken.END_ARRAY) {
-                handleMissingEndArrayForSingle(p, ctxt);
-            }
-            return value;
-        }
-        if (ctxt.isEnabled(DeserializationFeature.ACCEPT_EMPTY_ARRAY_AS_NULL_OBJECT)) {
-            JsonToken t = p.nextToken();
-            if (t == JsonToken.END_ARRAY) {
-                return null;
-            }
-            return ctxt.handleUnexpectedToken(handledType(),
-                    JsonToken.START_ARRAY, p, null);
-        }
-        return ctxt.handleUnexpectedToken(handledType(), p);
+    /**
+     * @deprecated Since 2.11 Should not be used: was never meant to be called by
+     *    code other than sub-classes (implementations), and implementations details
+     *    differ
+     */
+    @Deprecated
+    public Object deserializeFromArray(JsonParser p, DeserializationContext ctxt) throws IOException {
+        // should work as subtypes ought to override this method:
+        return _deserializeFromArray(p, ctxt);
     }
 
     public Object deserializeFromEmbedded(JsonParser p, DeserializationContext ctxt)
@@ -1499,7 +1592,7 @@ public abstract class BeanDeserializerBase
     /**
      * @since 2.9
      */
-    private final JsonDeserializer<Object> _delegateDeserializer() {
+    protected final JsonDeserializer<Object> _delegateDeserializer() {
         JsonDeserializer<Object> deser = _delegateDeserializer;
         if (deser == null) {
             deser = _arrayDelegateDeserializer;
@@ -1526,7 +1619,6 @@ public abstract class BeanDeserializerBase
      * stored in their entirety in given {@link TokenBuffer}
      * (as field entries, name and value).
      */
-    @SuppressWarnings("resource")
     protected Object handleUnknownProperties(DeserializationContext ctxt,
             Object bean, TokenBuffer unknownTokens)
         throws IOException
@@ -1537,7 +1629,7 @@ public abstract class BeanDeserializerBase
         // note: buffer does NOT have starting START_OBJECT
         JsonParser bufferParser = unknownTokens.asParser();
         while (bufferParser.nextToken() != JsonToken.END_OBJECT) {
-            String propName = bufferParser.getCurrentName();
+            String propName = bufferParser.currentName();
             // Unknown: let's call handler method
             bufferParser.nextToken();
             handleUnknownProperty(bufferParser, ctxt, bean, propName);
@@ -1548,23 +1640,28 @@ public abstract class BeanDeserializerBase
     /**
      * Helper method called for an unknown property, when using "vanilla"
      * processing.
+     *
+     * @param beanOrBuilder Either POJO instance (if constructed), or builder
+     *   (in case of builder-based approach), that has property we haven't been
+     *   able to handle yet.
      */
     protected void handleUnknownVanilla(JsonParser p, DeserializationContext ctxt,
-            Object bean, String propName)
+            Object beanOrBuilder, String propName)
         throws IOException
     {
-        if (_ignorableProps != null && _ignorableProps.contains(propName)) {
-            handleIgnoredProperty(p, ctxt, bean, propName);
+
+        if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
+            handleIgnoredProperty(p, ctxt, beanOrBuilder, propName);
         } else if (_anySetter != null) {
             try {
                // should we consider return type of any setter?
-                _anySetter.deserializeAndSet(p, ctxt, bean, propName);
+                _anySetter.deserializeAndSet(p, ctxt, beanOrBuilder, propName);
             } catch (Exception e) {
-                wrapAndThrow(e, bean, propName, ctxt);
+                wrapAndThrow(e, beanOrBuilder, propName, ctxt);
             }
         } else {
             // Unknown: let's call handler method
-            handleUnknownProperty(p, ctxt, bean, propName);         
+            handleUnknownProperty(p, ctxt, beanOrBuilder, propName);         
         }
     }
 
@@ -1581,7 +1678,7 @@ public abstract class BeanDeserializerBase
             p.skipChildren();
             return;
         }
-        if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+        if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
             handleIgnoredProperty(p, ctxt, beanOrClass, propName);
         }
         // Otherwise use default handling (call handler(s); if not
@@ -1616,7 +1713,6 @@ public abstract class BeanDeserializerBase
      *   (in addition to buffered properties); if null, all properties are passed
      *   in buffer
      */
-    @SuppressWarnings("resource")
     protected Object handlePolymorphic(JsonParser p, DeserializationContext ctxt,                                          
             Object bean, TokenBuffer unknownTokens)
         throws IOException
