@@ -2,6 +2,7 @@ package com.fasterxml.jackson.databind.deser;
 
 import java.util.*;
 
+import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
 import com.fasterxml.jackson.databind.deser.impl.BeanPropertyMap;
@@ -10,6 +11,8 @@ import com.fasterxml.jackson.databind.deser.impl.ObjectIdReader;
 import com.fasterxml.jackson.databind.deser.impl.ValueInjector;
 import com.fasterxml.jackson.databind.introspect.*;
 import com.fasterxml.jackson.databind.util.Annotations;
+import com.fasterxml.jackson.databind.util.ClassUtil;
+import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
 
 /**
  * Builder class used for aggregating deserialization information about
@@ -26,6 +29,11 @@ public class BeanDeserializerBuilder
 
     final protected DeserializationConfig _config;
 
+    /**
+     * @since 2.9
+     */
+    final protected DeserializationContext _context;
+
     /*
     /**********************************************************
     /* General information about POJO
@@ -42,18 +50,18 @@ public class BeanDeserializerBuilder
     /* Accumulated information about properties
     /**********************************************************
      */
-    
+
     /**
      * Properties to deserialize collected so far.
      */
     final protected Map<String, SettableBeanProperty> _properties
         = new LinkedHashMap<String, SettableBeanProperty>();
-    
+
     /**
      * Value injectors for deserialization
      */
     protected List<ValueInjector> _injectables;
-    
+
     /**
      * Back-reference properties this bean contains (if any)
      */
@@ -64,7 +72,13 @@ public class BeanDeserializerBuilder
      * purposes (meaning no exception is thrown, value is just skipped).
      */
     protected HashSet<String> _ignorableProps;
-    
+
+    /**
+     * Set of names of properties that are recognized and are set to be included for deserialization
+     * purposes (null deactivate this, empty includes nothing).
+     */
+    protected HashSet<String> _includableProps;
+
     /**
      * Object that will handle value instantiation for the bean type.
      */
@@ -75,7 +89,7 @@ public class BeanDeserializerBuilder
      * bean type.
      */
     protected ObjectIdReader _objectIdReader;
-    
+
     /**
      * Fallback setter used for handling any properties that are not
      * mapped to regular setters. If setter is not null, it will be
@@ -105,12 +119,13 @@ public class BeanDeserializerBuilder
     /* Life-cycle: construction
     /**********************************************************
      */
-    
+
     public BeanDeserializerBuilder(BeanDescription beanDesc,
-            DeserializationConfig config)
+            DeserializationContext ctxt)
     { 
         _beanDesc = beanDesc;
-        _config = config;
+        _context = ctxt;
+        _config = ctxt.getConfig();
     }
 
     /**
@@ -120,6 +135,7 @@ public class BeanDeserializerBuilder
     protected BeanDeserializerBuilder(BeanDeserializerBuilder src)
     {
         _beanDesc = src._beanDesc;
+        _context = src._context;
         _config = src._config;
 
         // let's make copy of properties
@@ -127,13 +143,14 @@ public class BeanDeserializerBuilder
         _injectables = _copy(src._injectables);
         _backRefProperties = _copy(src._backRefProperties);
         // Hmmh. Should we create defensive copies here? For now, not yet
-        _ignorableProps = src._ignorableProps;        
+        _ignorableProps = src._ignorableProps;
+        _includableProps = src._includableProps;
         _valueInstantiator = src._valueInstantiator;
         _objectIdReader = src._objectIdReader;
-        
+
         _anySetter = src._anySetter;
         _ignoreAllUnknown = src._ignoreAllUnknown;
-        
+
         _buildMethod = src._buildMethod;
         _builderConfig = src._builderConfig;
     }
@@ -146,7 +163,7 @@ public class BeanDeserializerBuilder
     private static <T> List<T> _copy(List<T> src) {
         return (src == null) ? null : new ArrayList<T>(src);
     }
-    
+
     /*
     /**********************************************************
     /* Life-cycle: state modification (adders, setters)
@@ -186,14 +203,19 @@ public class BeanDeserializerBuilder
         // 15-Sep-2016, tatu: For some reason fixing access at point of `build()` does
         //    NOT work (2 failing unit tests). Not 100% clear why, but for now force
         //    access set early; unfortunate, but since it works....
-        prop.fixAccess(_config);
+        if (_config.canOverrideAccessModifiers()) {
+            prop.fixAccess(_config);
+        }
         _backRefProperties.put(referenceName, prop);
-        // also: if we had property with same name, actually remove it
+        // 16-Jan-2018, tatu: As per [databind#1878] we may want to leave it as is, to allow
+        //    population for cases of "wrong direction", traversing parent first
+        //   If this causes problems should probably instead include in "ignored properties" list
+        //   Alternatively could also extend annotation to allow/disallow explicit value from input
+        /*
         if (_properties != null) {
             _properties.remove(prop.getName());
         }
-        // ??? 23-Jul-2012, tatu: Should it be included in list of all properties?
-        //   For now, won't add, since it is inferred, not explicit...
+        */
     }
 
     public void addInjectable(PropertyName propName, JavaType propType,
@@ -203,13 +225,10 @@ public class BeanDeserializerBuilder
         if (_injectables == null) {
             _injectables = new ArrayList<ValueInjector>();
         }
-        boolean fixAccess = _config.canOverrideAccessModifiers();
-        boolean forceAccess = fixAccess && _config.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS);
-        if (fixAccess) {
-            member.fixAccess(forceAccess);
+        if ( _config.canOverrideAccessModifiers()) {
+            member.fixAccess(_config.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
         }
-        _injectables.add(new ValueInjector(propName, propType,
-                contextAnnotations, member, valueId));
+        _injectables.add(new ValueInjector(propName, propType, member, valueId));
     }
 
     /**
@@ -224,6 +243,19 @@ public class BeanDeserializerBuilder
         _ignorableProps.add(propName);
     }
 
+    /**
+     * Method that will add property name as one of the properties that will be included.
+     *
+     * @since 2.12
+     */
+    public void addIncludable(String propName)
+    {
+        if (_includableProps == null) {
+            _includableProps = new HashSet<>();
+        }
+        _includableProps.add(propName);
+    }
+
     /**
      * Method called by deserializer factory, when a "creator property"
      * (something that is passed via constructor- or factory method argument;
@@ -318,6 +350,13 @@ public class BeanDeserializerBuilder
         return _builderConfig;
     }
 
+    /**
+     * @since 2.9.4
+     */
+    public boolean hasIgnorable(String name) {
+        return IgnorePropertiesUtil.shouldIgnore(name, _ignorableProps, _includableProps);
+    }
+
     /*
     /**********************************************************
     /* Build method(s)
@@ -332,9 +371,9 @@ public class BeanDeserializerBuilder
     {
         Collection<SettableBeanProperty> props = _properties.values();
         _fixAccess(props);
-
-        BeanPropertyMap propertyMap = BeanPropertyMap.construct(props,
-                _config.isEnabled(MapperFeature.ACCEPT_CASE_INSENSITIVE_PROPERTIES));
+        BeanPropertyMap propertyMap = BeanPropertyMap.construct(_config, props,
+                _collectAliases(props),
+                _findCaseInsensitivity());
         propertyMap.assignIndexes();
 
         // view processing must be enabled if:
@@ -361,7 +400,7 @@ public class BeanDeserializerBuilder
         }
 
         return new BeanDeserializer(this,
-                _beanDesc, propertyMap, _backRefProperties, _ignorableProps, _ignoreAllUnknown,
+                _beanDesc, propertyMap, _backRefProperties, _ignorableProps, _ignoreAllUnknown, _includableProps,
                 anyViews);
     }
 
@@ -373,23 +412,23 @@ public class BeanDeserializerBuilder
      * @since 2.0
      */
     public AbstractDeserializer buildAbstract() {
-        return new AbstractDeserializer(this, _beanDesc, _backRefProperties);
+        return new AbstractDeserializer(this, _beanDesc, _backRefProperties, _properties);
     }
 
     /**
      * Method for constructing a specialized deserializer that uses
      * additional external Builder object during data binding.
      */
-    public JsonDeserializer<?> buildBuilderBased(JavaType valueType,
-    		String expBuildMethodName)
+    public JsonDeserializer<?> buildBuilderBased(JavaType valueType, String expBuildMethodName)
+        throws JsonMappingException
     {
         // First: validation; must have build method that returns compatible type
         if (_buildMethod == null) {
             // as per [databind#777], allow empty name
             if (!expBuildMethodName.isEmpty()) {
-                throw new IllegalArgumentException(String.format(
-                        "Builder class %s does not have build method (name: '%s')",
-                        _beanDesc.getBeanClass().getName(),
+                _context.reportBadDefinition(_beanDesc.getType(),
+                        String.format("Builder class %s does not have build method (name: '%s')",
+                        ClassUtil.getTypeDescription(_beanDesc.getType()),
                         expBuildMethodName));
             }
         } else {
@@ -399,16 +438,19 @@ public class BeanDeserializerBuilder
             if ((rawBuildType != rawValueType)
                     && !rawBuildType.isAssignableFrom(rawValueType)
                     && !rawValueType.isAssignableFrom(rawBuildType)) {
-                throw new IllegalArgumentException("Build method '"+_buildMethod.getFullName()
-                        +" has bad return type ("+rawBuildType.getName()
-                        +"), not compatible with POJO type ("+valueType.getRawClass().getName()+")");
+                _context.reportBadDefinition(_beanDesc.getType(),
+                        String.format("Build method `%s` has wrong return type (%s), not compatible with POJO type (%s)",
+                        _buildMethod.getFullName(),
+                        ClassUtil.getClassDescription(rawBuildType),
+                        ClassUtil.getTypeDescription(valueType)));
             }
         }
         // And if so, we can try building the deserializer
         Collection<SettableBeanProperty> props = _properties.values();
         _fixAccess(props);
-        BeanPropertyMap propertyMap = BeanPropertyMap.construct(props,
-                _config.isEnabled(MapperFeature.ACCEPT_CASE_INSENSITIVE_PROPERTIES));
+        BeanPropertyMap propertyMap = BeanPropertyMap.construct(_config, props,
+                _collectAliases(props),
+                _findCaseInsensitivity());
         propertyMap.assignIndexes();
 
         boolean anyViews = !_config.isEnabled(MapperFeature.DEFAULT_VIEW_INCLUSION);
@@ -423,18 +465,26 @@ public class BeanDeserializerBuilder
         }
 
         if (_objectIdReader != null) {
-            /* 18-Nov-2012, tatu: May or may not have annotations for id property;
-             *   but no easy access. But hard to see id property being optional,
-             *   so let's consider required at this point.
-             */
+            // May or may not have annotations for id property; but no easy access.
+            // But hard to see id property being optional, so let's consider required at this point.
             ObjectIdValueProperty prop = new ObjectIdValueProperty(_objectIdReader,
                     PropertyMetadata.STD_REQUIRED);
             propertyMap = propertyMap.withProperty(prop);
         }
 
+        return createBuilderBasedDeserializer(valueType, propertyMap, anyViews);
+    }
+
+    /**
+     * Extension point for overriding the actual creation of the builder deserializer.
+     *
+     * @since 2.11
+     */
+    protected JsonDeserializer<?> createBuilderBasedDeserializer(JavaType valueType,
+            BeanPropertyMap propertyMap, boolean anyViews) {
         return new BuilderBasedDeserializer(this,
-                _beanDesc, propertyMap, _backRefProperties, _ignorableProps, _ignoreAllUnknown,
-                anyViews);
+                _beanDesc, valueType, propertyMap, _backRefProperties, _ignorableProps, _ignoreAllUnknown,
+                _includableProps, anyViews);
     }
 
     /*
@@ -443,7 +493,7 @@ public class BeanDeserializerBuilder
     /**********************************************************
      */
 
-    private void _fixAccess(Collection<SettableBeanProperty> mainProps)
+    protected void _fixAccess(Collection<SettableBeanProperty> mainProps)
     {
         /* 07-Sep-2016, tatu: Ideally we should be able to avoid forcing
          *   access to properties that are likely ignored, but due to
@@ -457,15 +507,22 @@ public class BeanDeserializerBuilder
             ignorable = Collections.emptySet();
         }
         */
-        for (SettableBeanProperty prop : mainProps) {
-            /*
-            // first: no point forcing access on to-be-ignored properties
-            if (ignorable.contains(prop.getName())) {
-                continue;
+
+        // 17-Jun-2020, tatu: [databind#2760] means we should not force access
+        //   if we are not configured to... at least not "regular" properties
+
+        if (_config.canOverrideAccessModifiers()) {
+            for (SettableBeanProperty prop : mainProps) {
+                /*
+                // first: no point forcing access on to-be-ignored properties
+                if (ignorable.contains(prop.getName())) {
+                    continue;
+                }
+                */
+                prop.fixAccess(_config);
             }
-            */
-            prop.fixAccess(_config);
         }
+
         // 15-Sep-2016, tatu: Access via back-ref properties has been done earlier
         //   as it has to, for some reason, so not repeated here.
 /*        
@@ -475,6 +532,11 @@ public class BeanDeserializerBuilder
             }
         }
         */
+
+        // 17-Jun-2020, tatu: Despite [databind#2760], it seems that methods that
+        //    are explicitly defined (any setter via annotation, builder too) can not
+        //    be left as-is? May reconsider based on feedback
+        
         if (_anySetter != null) {
             _anySetter.fixAccess(_config);
         }
@@ -482,4 +544,37 @@ public class BeanDeserializerBuilder
             _buildMethod.fixAccess(_config.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
         }
     }
+
+    protected Map<String,List<PropertyName>> _collectAliases(Collection<SettableBeanProperty> props)
+    {
+        Map<String,List<PropertyName>> mapping = null;
+        AnnotationIntrospector intr = _config.getAnnotationIntrospector();
+        if (intr != null) {
+            for (SettableBeanProperty prop : props) {
+                List<PropertyName> aliases = intr.findPropertyAliases(prop.getMember());
+                if ((aliases == null) || aliases.isEmpty()) {
+                    continue;
+                }
+                if (mapping == null) {
+                    mapping = new HashMap<>();
+                }
+                mapping.put(prop.getName(), aliases);
+            }
+        }
+        if (mapping == null) {
+            return Collections.emptyMap();
+        }
+        return mapping;
+    }
+
+    // @since 2.12
+    protected boolean _findCaseInsensitivity() {
+        // 07-May-2020, tatu: First find combination of per-type config overrides (higher
+        //   precedence) and per-type annotations (lower):
+        JsonFormat.Value format = _beanDesc.findExpectedFormat(null);
+        // and see if any of those has explicit definition; if not, use global baseline default
+        Boolean B = format.getFeature(JsonFormat.Feature.ACCEPT_CASE_INSENSITIVE_PROPERTIES);
+        return (B == null) ? _config.isEnabled(MapperFeature.ACCEPT_CASE_INSENSITIVE_PROPERTIES)
+                : B.booleanValue();
+    }
 }
