@@ -5,8 +5,10 @@ import java.util.*;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.cfg.CoercionAction;
 import com.fasterxml.jackson.databind.deser.impl.*;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMethod;
+import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 import com.fasterxml.jackson.databind.util.TokenBuffer;
 
@@ -49,9 +51,21 @@ public class BuilderBasedDeserializer
             BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
             Set<String> ignorableProps, boolean ignoreAllUnknown,
             boolean hasViews)
+    {
+        this(builder, beanDesc, targetType, properties, backRefs, ignorableProps, ignoreAllUnknown, null, hasViews);
+    }
+
+    /**
+     * @since 2.12
+     */
+    public BuilderBasedDeserializer(BeanDeserializerBuilder builder,
+                                    BeanDescription beanDesc, JavaType targetType,
+                                    BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
+                                    Set<String> ignorableProps, boolean ignoreAllUnknown, Set<String> includableProps,
+                                    boolean hasViews)
     {
         super(builder, beanDesc, properties, backRefs,
-                ignorableProps, ignoreAllUnknown, hasViews);
+                ignorableProps, ignoreAllUnknown, includableProps, hasViews);
         _targetType = targetType;
         _buildMethod = builder.getBuildMethod();
         // 05-Mar-2012, tatu: Cannot really make Object Ids work with builders, not yet anyway
@@ -105,7 +119,11 @@ public class BuilderBasedDeserializer
     }
 
     public BuilderBasedDeserializer(BuilderBasedDeserializer src, Set<String> ignorableProps) {
-        super(src, ignorableProps);
+        this(src, ignorableProps, src._includableProps);
+    }
+
+    public BuilderBasedDeserializer(BuilderBasedDeserializer src, Set<String> ignorableProps, Set<String> includableProps) {
+        super(src, ignorableProps, includableProps);
         _buildMethod = src._buildMethod;
         _targetType = src._targetType;
     }
@@ -132,8 +150,14 @@ public class BuilderBasedDeserializer
     }
 
     @Override
-    public BeanDeserializerBase withIgnorableProperties(Set<String> ignorableProps) {
-        return new BuilderBasedDeserializer(this, ignorableProps);
+    public BeanDeserializerBase withByNameInclusion(Set<String> ignorableProps,
+            Set<String> includableProps) {
+        return new BuilderBasedDeserializer(this, ignorableProps, includableProps);
+    }
+
+    @Override
+    public BeanDeserializerBase withIgnoreAllUnknown(boolean ignoreUnknown) {
+        return new BuilderBasedDeserializer(this, ignoreUnknown);
     }
 
     @Override
@@ -192,11 +216,10 @@ public class BuilderBasedDeserializer
             if (_vanillaProcessing) {
                 return finishBuild(ctxt, vanillaDeserialize(p, ctxt, t));
             }
-            Object builder = deserializeFromObject(p, ctxt);
-            return finishBuild(ctxt, builder);
+            return finishBuild(ctxt, deserializeFromObject(p, ctxt));
         }
         // and then others, generally requiring use of @JsonCreator
-        switch (p.getCurrentTokenId()) {
+        switch (p.currentTokenId()) {
         case JsonTokenId.ID_STRING:
             return finishBuild(ctxt, deserializeFromString(p, ctxt));
         case JsonTokenId.ID_NUMBER_INT:
@@ -209,14 +232,15 @@ public class BuilderBasedDeserializer
         case JsonTokenId.ID_FALSE:
             return finishBuild(ctxt, deserializeFromBoolean(p, ctxt));
         case JsonTokenId.ID_START_ARRAY:
-            // these only work if there's a (delegating) creator...
-            return finishBuild(ctxt, deserializeFromArray(p, ctxt));
+            // these only work if there's a (delegating) creator, or UNWRAP_SINGLE_ARRAY
+            // [databind#2608]: Do NOT call `finishBuild()` as method implements it
+            return _deserializeFromArray(p, ctxt);
         case JsonTokenId.ID_FIELD_NAME:
         case JsonTokenId.ID_END_OBJECT:
             return finishBuild(ctxt, deserializeFromObject(p, ctxt));
         default:
         }
-        return ctxt.handleUnexpectedToken(handledType(), p);
+        return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
     }
 
     /**
@@ -259,8 +283,8 @@ public class BuilderBasedDeserializer
         throws IOException
     {
         Object bean = _valueInstantiator.createUsingDefault(ctxt);
-        for (; p.getCurrentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
-            String propName = p.getCurrentName();
+        for (; p.currentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
+            String propName = p.currentName();
             // Skip field name:
             p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
@@ -304,8 +328,8 @@ public class BuilderBasedDeserializer
                 return deserializeWithView(p, ctxt, bean, view);
             }
         }
-        for (; p.getCurrentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
-            String propName = p.getCurrentName();
+        for (; p.currentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
+            String propName = p.currentName();
             // Skip field name:
             p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
@@ -333,7 +357,6 @@ public class BuilderBasedDeserializer
      * @return Builder instance constructed
      */
     @Override
-    @SuppressWarnings("resource")
     protected Object _deserializeUsingPropertyBased(final JsonParser p,
             final DeserializationContext ctxt)
         throws IOException
@@ -345,12 +368,16 @@ public class BuilderBasedDeserializer
         // 04-Jan-2010, tatu: May need to collect unknown properties for polymorphic cases
         TokenBuffer unknown = null;
 
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             p.nextToken(); // to point to value
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            // Object Id property?
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
+                continue;
+            }
             if (creatorProp != null) {
                 if ((activeView != null) && !creatorProp.visibleInView(activeView)) {
                     p.skipChildren();
@@ -378,10 +405,6 @@ public class BuilderBasedDeserializer
                 }
                 continue;
             }
-            // Object Id property?
-            if (buffer.readIdProperty(propName)) {
-                continue;
-            }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
@@ -390,7 +413,7 @@ public class BuilderBasedDeserializer
             }
             // As per [JACKSON-313], things marked as ignorable should not be
             // passed to any setter
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -425,7 +448,6 @@ public class BuilderBasedDeserializer
         return builder;
     }
 
-    @SuppressWarnings("resource")
     protected final Object _deserialize(JsonParser p,
             DeserializationContext ctxt, Object builder) throws IOException
     {        
@@ -449,13 +471,13 @@ public class BuilderBasedDeserializer
                 return deserializeWithView(p, ctxt, builder, view);
             }
         }
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         // 23-Mar-2010, tatu: In some cases, we start with full JSON object too...
         if (t == JsonToken.START_OBJECT) {
             t = p.nextToken();
         }
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             // Skip field name:
             p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
@@ -468,11 +490,52 @@ public class BuilderBasedDeserializer
                 }
                 continue;
             }
-            handleUnknownVanilla(p, ctxt, handledType(), propName);
+            handleUnknownVanilla(p, ctxt, builder, propName);
         }
         return builder;
     }
 
+    @Override // since 2.11, custom implementation
+    protected Object _deserializeFromArray(JsonParser p, DeserializationContext ctxt) throws IOException
+    {
+        // note: cannot call `_delegateDeserializer()` since order reversed here:
+        JsonDeserializer<Object> delegateDeser = _arrayDelegateDeserializer;
+        // fallback to non-array delegate
+        if ((delegateDeser != null) || ((delegateDeser = _delegateDeserializer) != null)) {
+            Object builder = _valueInstantiator.createUsingArrayDelegate(ctxt,
+                    delegateDeser.deserialize(p, ctxt));
+            if (_injectables != null) {
+                injectValues(ctxt, builder);
+            }
+            return finishBuild(ctxt, builder);
+        }
+        final CoercionAction act = _findCoercionFromEmptyArray(ctxt);
+        final boolean unwrap = ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
+
+        if (unwrap || (act != CoercionAction.Fail)) {
+            JsonToken t = p.nextToken();
+            if (t == JsonToken.END_ARRAY) {
+                switch (act) {
+                case AsEmpty:
+                    return getEmptyValue(ctxt);
+                case AsNull:
+                case TryConvert:
+                    return getNullValue(ctxt);
+                default:
+                }
+                return ctxt.handleUnexpectedToken(getValueType(ctxt), JsonToken.START_ARRAY, p, null);
+            }
+            if (unwrap) {
+                final Object value = deserialize(p, ctxt);
+                if (p.nextToken() != JsonToken.END_ARRAY) {
+                    handleMissingEndArrayForSingle(p, ctxt);
+                }
+                return value;
+            }
+        }
+        return ctxt.handleUnexpectedToken(getValueType(ctxt), p);
+    }
+
     /*
     /**********************************************************
     /* Deserializing when we have to consider an active View
@@ -483,9 +546,9 @@ public class BuilderBasedDeserializer
             Object bean, Class<?> activeView)
         throws IOException
     {
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             // Skip field name:
             p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
@@ -535,8 +598,8 @@ public class BuilderBasedDeserializer
         }
 
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        for (; p.getCurrentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
-            String propName = p.getCurrentName();
+        for (; p.currentToken() == JsonToken.FIELD_NAME; p.nextToken()) {
+            String propName = p.currentName();
             p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) { // normal case
@@ -552,7 +615,7 @@ public class BuilderBasedDeserializer
                 continue;
             }
             // ignorable things should be ignored
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
@@ -585,12 +648,16 @@ public class BuilderBasedDeserializer
         tokens.writeStartObject();
         Object builder = null;
 
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
         for (; t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+            String propName = p.currentName();
             p.nextToken(); // to point to value
             // creator property?
-            SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            final SettableBeanProperty creatorProp = creator.findCreatorProperty(propName);
+            // Object Id property?
+            if (buffer.readIdProperty(propName) && creatorProp == null) {
+                continue;
+            }
             if (creatorProp != null) {
                 // Last creator property to set?
                 if (buffer.assignParameter(creatorProp, creatorProp.deserialize(p, ctxt))) {
@@ -608,17 +675,13 @@ public class BuilderBasedDeserializer
                 }
                 continue;
             }
-            // Object Id property?
-            if (buffer.readIdProperty(propName)) {
-                continue;
-            }
             // regular property? needs buffering
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) {
                 buffer.bufferProperty(prop, prop.deserialize(p, ctxt));
                 continue;
             }
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, handledType(), propName);
                 continue;
             }
@@ -647,8 +710,8 @@ public class BuilderBasedDeserializer
         throws IOException
     {
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
-        for (JsonToken t = p.getCurrentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+        for (JsonToken t = p.currentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
+            String propName = p.currentName();
             SettableBeanProperty prop = _beanProperties.find(propName);
             p.nextToken();
             if (prop != null) { // normal case
@@ -663,7 +726,7 @@ public class BuilderBasedDeserializer
                 }
                 continue;
             }
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, builder, propName);
                 continue;
             }
@@ -702,8 +765,8 @@ public class BuilderBasedDeserializer
         final Class<?> activeView = _needViewProcesing ? ctxt.getActiveView() : null;
         final ExternalTypeHandler ext = _externalTypeIdHandler.start();
 
-        for (JsonToken t = p.getCurrentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
-            String propName = p.getCurrentName();
+        for (JsonToken t = p.currentToken(); t == JsonToken.FIELD_NAME; t = p.nextToken()) {
+            String propName = p.currentName();
             t = p.nextToken();
             SettableBeanProperty prop = _beanProperties.find(propName);
             if (prop != null) { // normal case
@@ -723,7 +786,7 @@ public class BuilderBasedDeserializer
                 continue;
             }
             // ignorable things should be ignored
-            if (_ignorableProps != null && _ignorableProps.contains(propName)) {
+            if (IgnorePropertiesUtil.shouldIgnore(propName, _ignorableProps, _includableProps)) {
                 handleIgnoredProperty(p, ctxt, bean, propName);
                 continue;
             }
