@@ -2,17 +2,18 @@ package com.fasterxml.jackson.databind.deser.std;
 
 import java.io.IOException;
 import java.util.*;
+import java.util.Objects;
 
 import com.fasterxml.jackson.annotation.JsonFormat;
-
 import com.fasterxml.jackson.core.*;
-
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
+import com.fasterxml.jackson.databind.cfg.CoercionAction;
+import com.fasterxml.jackson.databind.cfg.CoercionInputShape;
 import com.fasterxml.jackson.databind.deser.*;
 import com.fasterxml.jackson.databind.deser.impl.ReadableObjectId.Referring;
-import com.fasterxml.jackson.databind.deser.std.ContainerDeserializerBase;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
+import com.fasterxml.jackson.databind.type.LogicalType;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 
 /**
@@ -113,7 +114,6 @@ public class CollectionDeserializer
             JsonDeserializer<?> vd, TypeDeserializer vtd,
             NullValueProvider nuller, Boolean unwrapSingle)
     {
-//if (true) throw new Error();
         return new CollectionDeserializer(_containerType,
                 (JsonDeserializer<Object>) vd, vtd,
                 _valueInstantiator, (JsonDeserializer<Object>) dd,
@@ -130,6 +130,11 @@ public class CollectionDeserializer
                 ;
     }
 
+    @Override // since 2.12
+    public LogicalType logicalType() {
+        return LogicalType.Collection;
+    }
+
     /*
     /**********************************************************
     /* Validation, post-processing (ResolvableDeserializer)
@@ -190,7 +195,7 @@ _containerType,
             valueTypeDeser = valueTypeDeser.forProperty(property);
         }
         NullValueProvider nuller = findContentNullProvider(ctxt, property, valueDeser);
-        if ( (unwrapSingle != _unwrapSingle)
+        if ((!Objects.equals(unwrapSingle, _unwrapSingle))
                 || (nuller != _nullProvider)
                 || (delegateDeser != _delegateDeserializer)
                 || (valueDeser != _valueDeserializer)
@@ -233,16 +238,18 @@ _containerType,
             return (Collection<Object>) _valueInstantiator.createUsingDelegate(ctxt,
                     _delegateDeserializer.deserialize(p, ctxt));
         }
+        // 16-May-2020, tatu: As per [dataformats-text#199] need to first check for
+        //   possible Array-coercion and only after that String coercion
+        if (p.isExpectedStartArrayToken()) {
+            return _deserializeFromArray(p, ctxt, createDefaultInstance(ctxt));
+        }
         // Empty String may be ok; bit tricky to check, however, since
         // there is also possibility of "auto-wrapping" of single-element arrays.
         // Hence we only accept empty String here.
         if (p.hasToken(JsonToken.VALUE_STRING)) {
-            String str = p.getText();
-            if (str.length() == 0) {
-                return (Collection<Object>) _valueInstantiator.createFromString(ctxt, str);
-            }
+            return _deserializeFromString(p, ctxt, p.getText());
         }
-        return deserialize(p, ctxt, createDefaultInstance(ctxt));
+        return handleNonArray(p, ctxt, createDefaultInstance(ctxt));
     }
 
     /**
@@ -254,16 +261,70 @@ _containerType,
     {
         return (Collection<Object>) _valueInstantiator.createUsingDefault(ctxt);
     }
-    
+
     @Override
     public Collection<Object> deserialize(JsonParser p, DeserializationContext ctxt,
             Collection<Object> result)
         throws IOException
     {
         // Ok: must point to START_ARRAY (or equivalent)
-        if (!p.isExpectedStartArrayToken()) {
-            return handleNonArray(p, ctxt, result);
+        if (p.isExpectedStartArrayToken()) {
+            return _deserializeFromArray(p, ctxt, result);
         }
+        return handleNonArray(p, ctxt, result);
+    }
+
+    @Override
+    public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
+            TypeDeserializer typeDeserializer)
+        throws IOException
+    {
+        // In future could check current token... for now this should be enough:
+        return typeDeserializer.deserializeTypedFromArray(p, ctxt);
+    }
+
+    /**
+     * Logic extracted to deal with incoming String value.
+     *
+     * @since 2.12
+     */
+    @SuppressWarnings("unchecked")
+    protected Collection<Object> _deserializeFromString(JsonParser p, DeserializationContext ctxt,
+            String value)
+        throws IOException
+    {
+        final Class<?> rawTargetType = handledType();
+
+        // 05-Nov-2020, ckozak: As per [jackson-databind#2922] string values may be handled
+        // using handleNonArray, however empty strings may result in a null or empty collection
+        // depending on configuration.
+
+        // Start by verifying if we got empty/blank string since accessing
+        // CoercionAction may be costlier than String value we'll almost certainly
+        // need anyway
+        if (value.isEmpty()) { // ... in future may want to allow blank, too?
+            CoercionAction act = ctxt.findCoercionAction(logicalType(), rawTargetType,
+                    CoercionInputShape.EmptyString);
+            act = _checkCoercionFail(ctxt, act, rawTargetType, value,
+                    "empty String (\"\")");
+            if (act != null) {
+                    // handleNonArray may successfully deserialize the result (if
+                    // ACCEPT_SINGLE_VALUE_AS_ARRAY is enabled, for example) otherwise it
+                    // is capable of failing just as well as _deserializeFromEmptyString.
+                return (Collection<Object>) _deserializeFromEmptyString(
+                        p, ctxt, act, rawTargetType, "empty String (\"\")");
+            }
+        }
+        return handleNonArray(p, ctxt, createDefaultInstance(ctxt));
+    }
+
+    /**
+     * @since 2.12
+     */
+    protected Collection<Object> _deserializeFromArray(JsonParser p, DeserializationContext ctxt,
+            Collection<Object> result)
+        throws IOException
+    {
         // [databind#631]: Assign current value, to be accessible by custom serializers
         p.setCurrentValue(result);
 
@@ -305,15 +366,6 @@ _containerType,
         return result;
     }
 
-    @Override
-    public Object deserializeWithType(JsonParser p, DeserializationContext ctxt,
-            TypeDeserializer typeDeserializer)
-        throws IOException
-    {
-        // In future could check current token... for now this should be enough:
-        return typeDeserializer.deserializeTypedFromArray(p, ctxt);
-    }
-
     /**
      * Helper method called when current token is no START_ARRAY. Will either
      * throw an exception, or try to handle value as if member of implicit
@@ -329,16 +381,15 @@ _containerType,
                 ((_unwrapSingle == null) &&
                         ctxt.isEnabled(DeserializationFeature.ACCEPT_SINGLE_VALUE_AS_ARRAY));
         if (!canWrap) {
-            return (Collection<Object>) ctxt.handleUnexpectedToken(_containerType.getRawClass(), p);
+            return (Collection<Object>) ctxt.handleUnexpectedToken(_containerType, p);
         }
         JsonDeserializer<Object> valueDes = _valueDeserializer;
         final TypeDeserializer typeDeser = _valueTypeDeserializer;
-        JsonToken t = p.getCurrentToken();
 
         Object value;
 
         try {
-            if (t == JsonToken.VALUE_NULL) {
+            if (p.hasToken(JsonToken.VALUE_NULL)) {
                 // 03-Feb-2017, tatu: Hmmh. I wonder... let's try skipping here, too
                 if (_skipNullValues) {
                     return result;
@@ -350,6 +401,10 @@ _containerType,
                 value = valueDes.deserializeWithType(p, ctxt, typeDeser);
             }
         } catch (Exception e) {
+            boolean wrap = ctxt.isEnabled(DeserializationFeature.WRAP_EXCEPTIONS);
+            if (!wrap) {
+                ClassUtil.throwIfRTE(e);
+            }
             // note: pass Object.class, not Object[].class, as we need element type for error info
             throw JsonMappingException.wrapWithPath(e, Object.class, result.size());
         }
