@@ -15,7 +15,10 @@ import java.util.regex.Pattern;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.util.VersionUtil;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.cfg.CoercionAction;
+import com.fasterxml.jackson.databind.cfg.CoercionInputShape;
 import com.fasterxml.jackson.databind.exc.InvalidFormatException;
+import com.fasterxml.jackson.databind.type.LogicalType;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 
 /**
@@ -24,7 +27,7 @@ import com.fasterxml.jackson.databind.util.ClassUtil;
  * can be coerced into text, like Numbers and Booleans).
  * Simple JSON String values are trimmed using {@link java.lang.String#trim}.
  * Partial deserializer implementation will try to first access current token as
- * a String, calls {@link #_deserialize(String,DeserializationContext)} and
+ * a String, calls {@code _deserialize(String,DeserializationContext)} and
  * returns return value.
  * If this does not work (current token not a simple scalar type), attempts
  * are made so that:
@@ -67,13 +70,15 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
             TimeZone.class,
             InetAddress.class,
             InetSocketAddress.class,
+
+            // Special impl:
             StringBuilder.class,
         };
     }
-    
+
     /*
     /**********************************************************
-    /* Deserializer implementations
+    /* Life-cycle
     /**********************************************************
      */
     
@@ -85,7 +90,7 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
      * Factory method for trying to find a deserializer for one of supported
      * types that have simple from-String serialization.
      */
-    public static Std findDeserializer(Class<?> rawType)
+    public static FromStringDeserializer<?> findDeserializer(Class<?> rawType)
     {
         int kind = 0;
         if (rawType == File.class) {
@@ -113,12 +118,17 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
         } else if (rawType == InetSocketAddress.class) {
             kind = Std.STD_INET_SOCKET_ADDRESS;
         } else if (rawType == StringBuilder.class) {
-            kind = Std.STD_STRING_BUILDER;
+            return new StringBuilderDeserializer();
         } else {
             return null;
         }
         return new Std(rawType, kind);
     }
+
+    @Override // since 2.12
+    public LogicalType logicalType() {
+        return LogicalType.OtherScalar;
+    }
     
     /*
     /**********************************************************
@@ -132,33 +142,49 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
     {
         // Let's get textual value, possibly via coercion from other scalar types
         String text = p.getValueAsString();
-        if (text != null) { // has String representation
-            if (text.length() == 0 || (text = text.trim()).length() == 0) {
-                // Usually should become null; but not always
-                return _deserializeFromEmptyString();
+        if (text == null) {
+            JsonToken t = p.currentToken();
+            if (t != JsonToken.START_OBJECT) {
+                return (T) _deserializeFromOther(p, ctxt, t);
             }
-            Exception cause = null;
-            try {
-                // 19-May-2017, tatu: Used to require non-null result (assuming `null`
-                //    indicated error; but that seems wrong. Should be able to return
-                //    `null` as value.
-                return _deserialize(text, ctxt);
-            } catch (IllegalArgumentException | MalformedURLException e) {
-                cause = e;
-            }
-            // note: `cause` can't be null
-            String msg = "not a valid textual representation";
-            String m2 = cause.getMessage();
-            if (m2 != null) {
-                msg = msg + ", problem: "+m2;
-            }
-            // 05-May-2016, tatu: Unlike most usage, this seems legit, so...
-            JsonMappingException e = ctxt.weirdStringException(text, _valueClass, msg);
-            e.initCause(cause);
-            throw e;
-            // nothing to do here, yet? We'll fail anyway
+            // 29-Jun-2020, tatu: New! "Scalar from Object" (mostly for XML)
+            text = ctxt.extractScalarFromObject(p, this, _valueClass);
         }
-        JsonToken t = p.getCurrentToken();
+        if (text.isEmpty() || (text = text.trim()).isEmpty()) {
+            // 09-Jun-2020, tatu: Commonly `null` but may coerce to "empty" as well
+            return (T) _deserializeFromEmptyString(ctxt);
+        }
+        Exception cause = null;
+        try {
+            // 19-May-2017, tatu: Used to require non-null result (assuming `null`
+            //    indicated error; but that seems wrong. Should be able to return
+            //    `null` as value.
+            return _deserialize(text, ctxt);
+        } catch (IllegalArgumentException | MalformedURLException e) {
+            cause = e;
+        }
+        // note: `cause` can't be null
+        String msg = "not a valid textual representation";
+        String m2 = cause.getMessage();
+        if (m2 != null) {
+            msg = msg + ", problem: "+m2;
+        }
+        // 05-May-2016, tatu: Unlike most usage, this seems legit, so...
+        JsonMappingException e = ctxt.weirdStringException(text, _valueClass, msg);
+        e.initCause(cause);
+        throw e;
+    }
+
+    /**
+     * Main method from trying to deserialize actual value from non-empty
+     * String.
+     */
+    protected abstract T _deserialize(String value, DeserializationContext ctxt) throws IOException;
+
+    // @since 2.12
+    protected Object _deserializeFromOther(JsonParser p, DeserializationContext ctxt,
+            JsonToken t) throws IOException
+    {
         // [databind#381]
         if (t == JsonToken.START_ARRAY) {
             return _deserializeFromArray(p, ctxt);
@@ -170,15 +196,18 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
                 return null;
             }
             if (_valueClass.isAssignableFrom(ob.getClass())) {
-                return (T) ob;
+                return ob;
             }
             return _deserializeEmbedded(ob, ctxt);
         }
-        return (T) ctxt.handleUnexpectedToken(_valueClass, p);
+        return ctxt.handleUnexpectedToken(_valueClass, p);
     }
-        
-    protected abstract T _deserialize(String value, DeserializationContext ctxt) throws IOException;
 
+    /**
+     * Overridable method to allow coercion from embedded value that is neither
+     * {@code null} nor directly assignable to target type.
+     * Used, for example, by {@link UUIDDeserializer} to coerce from {@code byte[]}.
+     */
     protected T _deserializeEmbedded(Object ob, DeserializationContext ctxt) throws IOException {
         // default impl: error out
         ctxt.reportInputMismatch(this,
@@ -187,10 +216,41 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
         return null;
     }
 
-    protected T _deserializeFromEmptyString() throws IOException {
+    @Deprecated // since 2.12 -- override variant that takes context
+    protected final T _deserializeFromEmptyString() throws IOException {
         return null;
     }
 
+    /**
+     * @since 2.12
+     */
+    protected Object _deserializeFromEmptyString(DeserializationContext ctxt) throws IOException {
+        CoercionAction act = ctxt.findCoercionAction(logicalType(), _valueClass,
+                CoercionInputShape.EmptyString);
+        if (act == CoercionAction.Fail) {
+            ctxt.reportInputMismatch(this,
+"Cannot coerce empty String (\"\") to %s (but could if enabling coercion using `CoercionConfig`)",
+_coercedTypeDesc());
+        }
+        if (act == CoercionAction.AsNull) {
+            return getNullValue(ctxt);
+        }
+        if (act == CoercionAction.AsEmpty) {
+            return getEmptyValue(ctxt);
+        }
+        // 09-Jun-2020, tatu: semantics for `TryConvert` are bit interesting due to
+        //    historical reasons
+        return _deserializeFromEmptyStringDefault(ctxt);
+    }
+
+    /**
+     * @since 2.12
+     */
+    protected Object _deserializeFromEmptyStringDefault(DeserializationContext ctxt) throws IOException {
+        // by default, "as-null", but overridable by sub-classes
+        return getNullValue(ctxt);
+    }
+
     /*
     /**********************************************************
     /* A general-purpose implementation
@@ -219,10 +279,11 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
         public final static int STD_TIME_ZONE = 10;
         public final static int STD_INET_ADDRESS = 11;
         public final static int STD_INET_SOCKET_ADDRESS = 12;
-        public final static int STD_STRING_BUILDER = 13;
+        // No longer implemented here since 2.12
+        //        public final static int STD_STRING_BUILDER = 13;
 
         protected final int _kind;
-        
+
         protected Std(Class<?> valueType, int kind) {
             super(valueType);
             _kind = kind;
@@ -297,27 +358,32 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
                 }
                 // host or unbracketed IPv6, without port number
                 return new InetSocketAddress(value, 0);
-            case STD_STRING_BUILDER:
-                return new StringBuilder(value);
             }
             VersionUtil.throwInternal();
             return null;
         }
 
-        @Override
-        protected Object _deserializeFromEmptyString() throws IOException {
-            // As per [databind#398], URI requires special handling
-            if (_kind == STD_URI) {
+        @Override // since 2.12
+        public Object getEmptyValue(DeserializationContext ctxt)
+                throws JsonMappingException
+        {
+            switch (_kind) {
+            case STD_URI:
+                // As per [databind#398], URI requires special handling
                 return URI.create("");
-            }
-            // As per [databind#1123], Locale too
-            if (_kind == STD_LOCALE) {
+            case STD_LOCALE:
+                // As per [databind#1123], Locale too
                 return Locale.ROOT;
             }
-            if (_kind == STD_STRING_BUILDER) {
-                return new StringBuilder();
-            }
-            return super._deserializeFromEmptyString();
+            return super.getEmptyValue(ctxt);
+        }
+
+        @Override
+        protected Object _deserializeFromEmptyStringDefault(DeserializationContext ctxt) throws IOException {
+            // 09-Jun-2020, tatu: For backwards compatibility deserialize "as-empty"
+            //    as URI and Locale did that in 2.11 (and StringBuilder probably ought to).
+            //   But doing this here instead of super-class lets UUID return "as-null" instead
+            return getEmptyValue(ctxt);
         }
 
         protected int _firstHyphenOrUnderscore(String str)
@@ -331,4 +397,42 @@ public abstract class FromStringDeserializer<T> extends StdScalarDeserializer<T>
             return -1;
         }
     }
+
+    // @since 2.12 to simplify logic a bit: should not use coercions when reading
+    //   String Values
+    static class StringBuilderDeserializer extends FromStringDeserializer<Object>
+    {
+        public StringBuilderDeserializer() {
+            super(StringBuilder.class);
+        }
+
+        @Override
+        public LogicalType logicalType() {
+            return LogicalType.Textual;
+        }
+
+        @Override
+        public Object getEmptyValue(DeserializationContext ctxt)
+            throws JsonMappingException
+        {
+            return new StringBuilder();
+        }
+
+        @Override
+        public Object deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
+        {
+            String text = p.getValueAsString();
+            if (text != null) {
+                return _deserialize(text, ctxt);
+            }
+            return super.deserialize(p, ctxt);
+        }
+
+        @Override
+        protected Object _deserialize(String value, DeserializationContext ctxt)
+            throws IOException
+        {
+            return new StringBuilder(value);
+        }
+    }
 }
