@@ -6,6 +6,7 @@ import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
 import com.fasterxml.jackson.databind.node.*;
+import com.fasterxml.jackson.databind.type.LogicalType;
 import com.fasterxml.jackson.databind.util.RawValue;
 
 /**
@@ -22,7 +23,12 @@ public class JsonNodeDeserializer
      */
     private final static JsonNodeDeserializer instance = new JsonNodeDeserializer();
 
-    protected JsonNodeDeserializer() { super(JsonNode.class); }
+    protected JsonNodeDeserializer() {
+        // `null` means that explicit "merge" is honored and may or may not work, but
+        // that per-type and global defaults do not enable merging. This because
+        // some node types (Object, Array) do support, others don't.
+        super(JsonNode.class, null);
+    }
 
     /**
      * Factory method for accessing deserializer for specific node type
@@ -47,13 +53,7 @@ public class JsonNodeDeserializer
 
     @Override
     public JsonNode getNullValue(DeserializationContext ctxt) {
-        return NullNode.getInstance();
-    }
-
-    @Override
-    @Deprecated // since 2.6, remove from 2.7
-    public JsonNode getNullValue() {
-        return NullNode.getInstance();
+        return ctxt.getNodeFactory().nullNode();
     }
 
     /**
@@ -64,14 +64,14 @@ public class JsonNodeDeserializer
     @Override
     public JsonNode deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        switch (p.getCurrentTokenId()) {
+        switch (p.currentTokenId()) {
         case JsonTokenId.ID_START_OBJECT:
             return deserializeObject(p, ctxt, ctxt.getNodeFactory());
         case JsonTokenId.ID_START_ARRAY:
             return deserializeArray(p, ctxt, ctxt.getNodeFactory());
         default:
-            return deserializeAny(p, ctxt, ctxt.getNodeFactory());
         }
+        return deserializeAny(p, ctxt, ctxt.getNodeFactory());
     }
 
     /*
@@ -87,16 +87,19 @@ public class JsonNodeDeserializer
 
         protected final static ObjectDeserializer _instance = new ObjectDeserializer();
 
-        protected ObjectDeserializer() { super(ObjectNode.class); }
+        protected ObjectDeserializer() { super(ObjectNode.class, true); }
 
         public static ObjectDeserializer getInstance() { return _instance; }
-        
+
         @Override
         public ObjectNode deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
         {
-            if (p.isExpectedStartObjectToken() || p.hasToken(JsonToken.FIELD_NAME)) {
+            if (p.isExpectedStartObjectToken()) {
                 return deserializeObject(p, ctxt, ctxt.getNodeFactory());
             }
+            if (p.hasToken(JsonToken.FIELD_NAME)) {
+                return deserializeObjectAtName(p, ctxt, ctxt.getNodeFactory());
+            }
             // 23-Sep-2015, tatu: Ugh. We may also be given END_OBJECT (similar to FIELD_NAME),
             //    if caller has advanced to the first token of Object, but for empty Object
             if (p.hasToken(JsonToken.END_OBJECT)) {
@@ -104,8 +107,23 @@ public class JsonNodeDeserializer
             }
             return (ObjectNode) ctxt.handleUnexpectedToken(ObjectNode.class, p);
          }
+
+        /**
+         * Variant needed to support both root-level `updateValue()` and merging.
+         *
+         * @since 2.9
+         */
+        @Override
+        public ObjectNode deserialize(JsonParser p, DeserializationContext ctxt,
+                ObjectNode node) throws IOException
+        {
+            if (p.isExpectedStartObjectToken() || p.hasToken(JsonToken.FIELD_NAME)) {
+                return (ObjectNode) updateObject(p, ctxt, (ObjectNode) node);
+            }
+            return (ObjectNode) ctxt.handleUnexpectedToken(ObjectNode.class, p);
+        }
     }
-        
+
     final static class ArrayDeserializer
         extends BaseNodeDeserializer<ArrayNode>
     {
@@ -113,10 +131,10 @@ public class JsonNodeDeserializer
 
         protected final static ArrayDeserializer _instance = new ArrayDeserializer();
 
-        protected ArrayDeserializer() { super(ArrayNode.class); }
+        protected ArrayDeserializer() { super(ArrayNode.class, true); }
 
         public static ArrayDeserializer getInstance() { return _instance; }
-        
+
         @Override
         public ArrayNode deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
         {
@@ -125,6 +143,21 @@ public class JsonNodeDeserializer
             }
             return (ArrayNode) ctxt.handleUnexpectedToken(ArrayNode.class, p);
         }
+
+        /**
+         * Variant needed to support both root-level `updateValue()` and merging.
+         *
+         * @since 2.9
+         */
+        @Override
+        public ArrayNode deserialize(JsonParser p, DeserializationContext ctxt,
+                ArrayNode node) throws IOException
+        {
+            if (p.isExpectedStartArrayToken()) {
+                return (ArrayNode) updateArray(p, ctxt, (ArrayNode) node);
+            }
+            return (ArrayNode) ctxt.handleUnexpectedToken(ArrayNode.class, p);
+        }
     }
 }
 
@@ -136,8 +169,11 @@ public class JsonNodeDeserializer
 abstract class BaseNodeDeserializer<T extends JsonNode>
     extends StdDeserializer<T>
 {
-    public BaseNodeDeserializer(Class<T> vc) {
+    protected final Boolean _supportsUpdates;
+
+    public BaseNodeDeserializer(Class<T> vc, Boolean supportsUpdates) {
         super(vc);
+        _supportsUpdates = supportsUpdates;
     }
 
     @Override
@@ -145,30 +181,32 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
             TypeDeserializer typeDeserializer)
         throws IOException
     {
-        /* Output can be as JSON Object, Array or scalar: no way to know
-         * a priori. So:
-         */
+        // Output can be as JSON Object, Array or scalar: no way to know a priori:
         return typeDeserializer.deserializeTypedFromAny(p, ctxt);
     }
 
-    /* 07-Nov-2014, tatu: When investigating [databind#604], realized that it makes
-     *   sense to also mark this is cachable, since lookup not exactly free, and
-     *   since it's not uncommon to "read anything"
-     */
+    @Override // since 2.12
+    public LogicalType logicalType() {
+        return LogicalType.Untyped;
+    }
+
+    // 07-Nov-2014, tatu: When investigating [databind#604], realized that it makes
+    //   sense to also mark this is cachable, since lookup not exactly free, and
+    //   since it's not uncommon to "read anything"
     @Override
     public boolean isCachable() { return true; }
 
+    @Override // since 2.9
+    public Boolean supportsUpdate(DeserializationConfig config) {
+        return _supportsUpdates;
+    }
+
     /*
     /**********************************************************
     /* Overridable methods
     /**********************************************************
      */
 
-    @Deprecated // since 2.8
-    protected void _reportProblem(JsonParser p, String msg) throws JsonMappingException {
-        throw JsonMappingException.from(p, msg);
-    }
-
     /**
      * Method called when there is a duplicate value for a field.
      * By default we don't care, and the last value is used.
@@ -190,9 +228,27 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
     {
         // [databind#237]: Report an error if asked to do so:
         if (ctxt.isEnabled(DeserializationFeature.FAIL_ON_READING_DUP_TREE_KEY)) {
-            ctxt.reportMappingException("Duplicate field '%s' for ObjectNode: not allowed when FAIL_ON_READING_DUP_TREE_KEY enabled",
+            // 11-Sep-2019, tatu: Can not pass "property name" because we may be
+            //    missing enclosing JSON content context...
+// ctxt.reportPropertyInputMismatch(JsonNode.class, fieldName,
+            ctxt.reportInputMismatch(JsonNode.class,
+"Duplicate field '%s' for `ObjectNode`: not allowed when `DeserializationFeature.FAIL_ON_READING_DUP_TREE_KEY` enabled",
                     fieldName);
         }
+        // [databind#2732]: Special case for XML; automatically coerce into `ArrayNode`
+        if (ctxt.isEnabled(StreamReadCapability.DUPLICATE_PROPERTIES)) {
+            // Note that ideally we wouldn't have to shuffle things but... Map.putIfAbsent()
+            // only added in JDK 8, to efficiently check for add. So...
+            if (oldValue.isArray()) { // already was array, to append
+                ((ArrayNode) oldValue).add(newValue);
+                objectNode.replace(fieldName, oldValue);
+            } else { // was not array, convert
+                ArrayNode arr = nodeFactory.arrayNode();
+                arr.add(oldValue);
+                arr.add(newValue);
+                objectNode.replace(fieldName, arr);
+            }
+        }
     }
 
     /*
@@ -201,28 +257,20 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
     /**********************************************************
      */
 
+    /**
+     * Method called to deserialize Object node instance when there is no existing
+     * node to modify.
+     */
     protected final ObjectNode deserializeObject(JsonParser p, DeserializationContext ctxt,
             final JsonNodeFactory nodeFactory) throws IOException
     {
-        ObjectNode node = nodeFactory.objectNode();
-        String key;
-        if (p.isExpectedStartObjectToken()) {
-            key = p.nextFieldName();
-        } else {
-            JsonToken t = p.getCurrentToken();
-            if (t == JsonToken.END_OBJECT) {
-                return node;
-            }
-            if (t != JsonToken.FIELD_NAME) {
-                return (ObjectNode) ctxt.handleUnexpectedToken(handledType(), p);
-            }
-            key = p.getCurrentName();
-        }
+        final ObjectNode node = nodeFactory.objectNode();
+        String key = p.nextFieldName();
         for (; key != null; key = p.nextFieldName()) {
             JsonNode value;
             JsonToken t = p.nextToken();
-            if (t == null) {
-                throw ctxt.mappingException("Unexpected end-of-input when binding data into ObjectNode");
+            if (t == null) { // can this ever occur?
+                t = JsonToken.NOT_AVAILABLE; // can this ever occur?
             }
             switch (t.id()) {
             case JsonTokenId.ID_START_OBJECT:
@@ -261,6 +309,142 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
         return node;
     }
 
+    /**
+     * Alternate deserialization method used when parser already points to first
+     * FIELD_NAME and not START_OBJECT.
+     *
+     * @since 2.9
+     */
+    protected final ObjectNode deserializeObjectAtName(JsonParser p, DeserializationContext ctxt,
+            final JsonNodeFactory nodeFactory) throws IOException
+    {
+        final ObjectNode node = nodeFactory.objectNode();
+        String key = p.currentName();
+        for (; key != null; key = p.nextFieldName()) {
+            JsonNode value;
+            JsonToken t = p.nextToken();
+            if (t == null) { // can this ever occur?
+                t = JsonToken.NOT_AVAILABLE; // can this ever occur?
+            }
+            switch (t.id()) {
+            case JsonTokenId.ID_START_OBJECT:
+                value = deserializeObject(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_START_ARRAY:
+                value = deserializeArray(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_EMBEDDED_OBJECT:
+                value = _fromEmbedded(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_STRING:
+                value = nodeFactory.textNode(p.getText());
+                break;
+            case JsonTokenId.ID_NUMBER_INT:
+                value = _fromInt(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_TRUE:
+                value = nodeFactory.booleanNode(true);
+                break;
+            case JsonTokenId.ID_FALSE:
+                value = nodeFactory.booleanNode(false);
+                break;
+            case JsonTokenId.ID_NULL:
+                value = nodeFactory.nullNode();
+                break;
+            default:
+                value = deserializeAny(p, ctxt, nodeFactory);
+            }
+            JsonNode old = node.replace(key, value);
+            if (old != null) {
+                _handleDuplicateField(p, ctxt, nodeFactory,
+                        key, node, old, value);
+            }
+        }
+        return node;
+    }
+    
+    /**
+     * Alternate deserialization method that is to update existing {@link ObjectNode}
+     * if possible.
+     *
+     * @since 2.9
+     */
+    protected final JsonNode updateObject(JsonParser p, DeserializationContext ctxt,
+        final ObjectNode node) throws IOException
+    {
+        String key;
+        if (p.isExpectedStartObjectToken()) {
+            key = p.nextFieldName();
+        } else {
+            if (!p.hasToken(JsonToken.FIELD_NAME)) {
+                return deserialize(p, ctxt);
+            }
+            key = p.currentName();
+        }
+        for (; key != null; key = p.nextFieldName()) {
+            // If not, fall through to regular handling
+            JsonToken t = p.nextToken();
+
+            // First: see if we can merge things:
+            JsonNode old = node.get(key);
+            if (old != null) {
+                if (old instanceof ObjectNode) {
+                    JsonNode newValue = updateObject(p, ctxt, (ObjectNode) old);
+                    if (newValue != old) {
+                        node.set(key, newValue);
+                    }
+                    continue;
+                }
+                if (old instanceof ArrayNode) {
+                    JsonNode newValue = updateArray(p, ctxt, (ArrayNode) old);
+                    if (newValue != old) {
+                        node.set(key, newValue);
+                    }
+                    continue;
+                }
+            }
+            if (t == null) { // can this ever occur?
+                t = JsonToken.NOT_AVAILABLE;
+            }
+            JsonNode value;
+            JsonNodeFactory nodeFactory = ctxt.getNodeFactory();
+            switch (t.id()) {
+            case JsonTokenId.ID_START_OBJECT:
+                value = deserializeObject(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_START_ARRAY:
+                value = deserializeArray(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_EMBEDDED_OBJECT:
+                value = _fromEmbedded(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_STRING:
+                value = nodeFactory.textNode(p.getText());
+                break;
+            case JsonTokenId.ID_NUMBER_INT:
+                value = _fromInt(p, ctxt, nodeFactory);
+                break;
+            case JsonTokenId.ID_TRUE:
+                value = nodeFactory.booleanNode(true);
+                break;
+            case JsonTokenId.ID_FALSE:
+                value = nodeFactory.booleanNode(false);
+                break;
+            case JsonTokenId.ID_NULL:
+                value = nodeFactory.nullNode();
+                break;
+            default:
+                value = deserializeAny(p, ctxt, nodeFactory);
+            }
+            if (old != null) {
+                _handleDuplicateField(p, ctxt, nodeFactory,
+                        key, node, old, value);
+            }
+            node.set(key, value);
+        }
+        return node;
+    }
+
     protected final ArrayNode deserializeArray(JsonParser p, DeserializationContext ctxt,
             final JsonNodeFactory nodeFactory) throws IOException
     {
@@ -301,16 +485,60 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
         }
     }
 
+    /**
+     * Alternate deserialization method that is to update existing {@link ObjectNode}
+     * if possible.
+     *
+     * @since 2.9
+     */
+    protected final JsonNode updateArray(JsonParser p, DeserializationContext ctxt,
+        final ArrayNode node) throws IOException
+    {
+        final JsonNodeFactory nodeFactory = ctxt.getNodeFactory();
+        while (true) {
+            JsonToken t = p.nextToken();
+            switch (t.id()) {
+            case JsonTokenId.ID_START_OBJECT:
+                node.add(deserializeObject(p, ctxt, nodeFactory));
+                break;
+            case JsonTokenId.ID_START_ARRAY:
+                node.add(deserializeArray(p, ctxt, nodeFactory));
+                break;
+            case JsonTokenId.ID_END_ARRAY:
+                return node;
+            case JsonTokenId.ID_EMBEDDED_OBJECT:
+                node.add(_fromEmbedded(p, ctxt, nodeFactory));
+                break;
+            case JsonTokenId.ID_STRING:
+                node.add(nodeFactory.textNode(p.getText()));
+                break;
+            case JsonTokenId.ID_NUMBER_INT:
+                node.add(_fromInt(p, ctxt, nodeFactory));
+                break;
+            case JsonTokenId.ID_TRUE:
+                node.add(nodeFactory.booleanNode(true));
+                break;
+            case JsonTokenId.ID_FALSE:
+                node.add(nodeFactory.booleanNode(false));
+                break;
+            case JsonTokenId.ID_NULL:
+                node.add(nodeFactory.nullNode());
+                break;
+            default:
+                node.add(deserializeAny(p, ctxt, nodeFactory));
+                break;
+            }
+        }
+    }
+
     protected final JsonNode deserializeAny(JsonParser p, DeserializationContext ctxt,
             final JsonNodeFactory nodeFactory) throws IOException
     {
-        switch (p.getCurrentTokenId()) {
-        case JsonTokenId.ID_START_OBJECT:
-        case JsonTokenId.ID_END_OBJECT: // for empty JSON Objects we may point to this
+        switch (p.currentTokenId()) {
+        case JsonTokenId.ID_END_OBJECT: // for empty JSON Objects we may point to this?
+            return nodeFactory.objectNode();
         case JsonTokenId.ID_FIELD_NAME:
-            return deserializeObject(p, ctxt, nodeFactory);
-        case JsonTokenId.ID_START_ARRAY:
-            return deserializeArray(p, ctxt, nodeFactory);
+            return deserializeObjectAtName(p, ctxt, nodeFactory);
         case JsonTokenId.ID_EMBEDDED_OBJECT:
             return _fromEmbedded(p, ctxt, nodeFactory);
         case JsonTokenId.ID_STRING:
@@ -325,8 +553,16 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
             return nodeFactory.booleanNode(false);
         case JsonTokenId.ID_NULL:
             return nodeFactory.nullNode();
+
+            /* Caller checks for these, should not get here ever
+        case JsonTokenId.ID_START_OBJECT:
+            return deserializeObject(p, ctxt, nodeFactory);
+        case JsonTokenId.ID_START_ARRAY:
+            return deserializeArray(p, ctxt, nodeFactory);
+            */
+
             
-            // These states can not be mapped; input stream is
+            // These states cannot be mapped; input stream is
             // off by an event or two
 
         //case END_OBJECT:
@@ -371,12 +607,8 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
         if (ctxt.isEnabled(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS)) {
             // 20-May-2016, tatu: As per [databind#1028], need to be careful
             //   (note: JDK 1.8 would have `Double.isFinite()`)
-            // 21-Aug-2016, tatu: Not optimal, really, because this may result in
-            //   value getting parsed twice. But has to do for now, to resolve
-            //  [databind#1315]
-            double d = p.getDoubleValue();
-            if (Double.isInfinite(d) || Double.isNaN(d)) {
-                return nodeFactory.numberNode(d);
+            if (p.isNaN()) {
+                return nodeFactory.numberNode(p.getDoubleValue());
             }
             return nodeFactory.numberNode(p.getDecimalValue());
         }
@@ -402,7 +634,7 @@ abstract class BaseNodeDeserializer<T extends JsonNode>
             return nodeFactory.rawValueNode((RawValue) ob);
         }
         if (ob instanceof JsonNode) {
-            // [Issue#433]: but could also be a JsonNode hiding in there!
+            // [databind#433]: but could also be a JsonNode hiding in there!
             return (JsonNode) ob;
         }
         // any other special handling needed?
