@@ -8,6 +8,7 @@ import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.deser.*;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
+import com.fasterxml.jackson.databind.type.LogicalType;
 
 /**
  * Basic serializer that can take JSON "Object" structure and
@@ -103,6 +104,12 @@ public class MapEntryDeserializer
                 keyDeser, (JsonDeserializer<Object>) valueDeser, valueTypeDeser);
     }
 
+    @Override // since 2.12
+    public LogicalType logicalType() {
+        // Slightly tricky, could consider POJO too?
+        return LogicalType.Map;
+    }
+
     /*
     /**********************************************************
     /* Validation, post-processing (ResolvableDeserializer)
@@ -155,7 +162,10 @@ public class MapEntryDeserializer
     public JsonDeserializer<Object> getContentDeserializer() {
         return _valueDeserializer;
     }
-    
+
+    // 31-May-2020, tatu: Should probably define but we don't have it yet
+//    public ValueInstantiator getValueInstantiator() { }
+
     /*
     /**********************************************************
     /* JsonDeserializer API
@@ -167,14 +177,15 @@ public class MapEntryDeserializer
     public Map.Entry<Object,Object> deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
     {
         // Ok: must point to START_OBJECT, FIELD_NAME or END_OBJECT
-        JsonToken t = p.getCurrentToken();
-        if (t != JsonToken.START_OBJECT && t != JsonToken.FIELD_NAME && t != JsonToken.END_OBJECT) {
-            // String may be ok however:
-            // slightly redundant (since String was passed above), but
-            return _deserializeFromEmpty(p, ctxt);
-        }
+        JsonToken t = p.currentToken();
         if (t == JsonToken.START_OBJECT) {
             t = p.nextToken();
+        } else if (t != JsonToken.FIELD_NAME && t != JsonToken.END_OBJECT) {
+            // Empty array, or single-value wrapped in array?
+            if (t == JsonToken.START_ARRAY) {
+                return _deserializeFromArray(p, ctxt);
+            }
+            return (Map.Entry<Object,Object>) ctxt.handleUnexpectedToken(getValueType(ctxt), p);
         }
         if (t != JsonToken.FIELD_NAME) {
             if (t == JsonToken.END_OBJECT) {
@@ -188,7 +199,7 @@ public class MapEntryDeserializer
         final JsonDeserializer<Object> valueDes = _valueDeserializer;
         final TypeDeserializer typeDeser = _valueTypeDeserializer;
 
-        final String keyStr = p.getCurrentName();
+        final String keyStr = p.currentName();
         Object key = keyDes.deserializeKey(keyStr, ctxt);
         Object value = null;
         // And then the value...
@@ -212,7 +223,7 @@ public class MapEntryDeserializer
             if (t == JsonToken.FIELD_NAME) { // most likely
                 ctxt.reportInputMismatch(this,
                         "Problem binding JSON into Map.Entry: more than one entry in JSON (second field: '%s')",
-                        p.getCurrentName());
+                        p.currentName());
             } else {
                 // how would this occur?
                 ctxt.reportInputMismatch(this,
