@@ -6,9 +6,10 @@ import java.lang.reflect.InvocationTargetException;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.deser.*;
-import com.fasterxml.jackson.databind.introspect.AnnotatedParameter;
 import com.fasterxml.jackson.databind.introspect.AnnotatedWithParams;
 import com.fasterxml.jackson.databind.util.ClassUtil;
+import java.math.BigDecimal;
+import java.math.BigInteger;
 
 /**
  * Default {@link ValueInstantiator} implementation, which supports
@@ -47,7 +48,7 @@ public class StdValueInstantiator
     protected SettableBeanProperty[] _constructorArguments;
 
     // // // Delegate construction
-    
+
     protected JavaType _delegateType;
     protected AnnotatedWithParams _delegateCreator;
     protected SettableBeanProperty[] _delegateArguments;
@@ -57,18 +58,17 @@ public class StdValueInstantiator
     protected JavaType _arrayDelegateType;
     protected AnnotatedWithParams _arrayDelegateCreator;
     protected SettableBeanProperty[] _arrayDelegateArguments;
-    
+
     // // // Scalar construction
 
     protected AnnotatedWithParams _fromStringCreator;
     protected AnnotatedWithParams _fromIntCreator;
     protected AnnotatedWithParams _fromLongCreator;
+    protected AnnotatedWithParams _fromBigIntegerCreator;
     protected AnnotatedWithParams _fromDoubleCreator;
+    protected AnnotatedWithParams _fromBigDecimalCreator;
     protected AnnotatedWithParams _fromBooleanCreator;
 
-    // // // Incomplete creator
-    protected AnnotatedParameter  _incompleteParameter;
-    
     /*
     /**********************************************************
     /* Life-cycle
@@ -110,11 +110,13 @@ public class StdValueInstantiator
         _arrayDelegateType = src._arrayDelegateType;
         _arrayDelegateCreator = src._arrayDelegateCreator;
         _arrayDelegateArguments = src._arrayDelegateArguments;
-        
+
         _fromStringCreator = src._fromStringCreator;
         _fromIntCreator = src._fromIntCreator;
         _fromLongCreator = src._fromLongCreator;
+        _fromBigIntegerCreator = src._fromBigIntegerCreator;
         _fromDoubleCreator = src._fromDoubleCreator;
+        _fromBigDecimalCreator = src._fromBigDecimalCreator;
         _fromBooleanCreator = src._fromBooleanCreator;
     }
 
@@ -157,18 +159,18 @@ public class StdValueInstantiator
         _fromLongCreator = creator;
     }
 
+    public void configureFromBigIntegerCreator(AnnotatedWithParams creator) { _fromBigIntegerCreator = creator; }
+
     public void configureFromDoubleCreator(AnnotatedWithParams creator) {
         _fromDoubleCreator = creator;
     }
 
+    public void configureFromBigDecimalCreator(AnnotatedWithParams creator) { _fromBigDecimalCreator = creator; }
+
     public void configureFromBooleanCreator(AnnotatedWithParams creator) {
         _fromBooleanCreator = creator;
     }
 
-    public void configureIncompleteParameter(AnnotatedParameter parameter) {
-        _incompleteParameter = parameter;
-    }
-    
     /*
     /**********************************************************
     /* Public API implementation; metadata
@@ -200,11 +202,17 @@ public class StdValueInstantiator
         return (_fromLongCreator != null);
     }
 
+    @Override
+    public boolean canCreateFromBigInteger() { return _fromBigIntegerCreator != null; }
+
     @Override
     public boolean canCreateFromDouble() {
         return (_fromDoubleCreator != null);
     }
 
+    @Override
+    public boolean canCreateFromBigDecimal() { return _fromBigDecimalCreator != null; }
+
     @Override
     public boolean canCreateFromBoolean() {
         return (_fromBooleanCreator != null);
@@ -253,13 +261,13 @@ public class StdValueInstantiator
     public SettableBeanProperty[] getFromObjectArguments(DeserializationConfig config) {
         return _constructorArguments;
     }
-    
+
     /*
     /**********************************************************
     /* Public API implementation; instantiation from JSON Object
     /**********************************************************
      */
-    
+
     @Override
     public Object createUsingDefault(DeserializationContext ctxt) throws IOException
     {
@@ -319,17 +327,17 @@ public class StdValueInstantiator
     @Override
     public Object createFromString(DeserializationContext ctxt, String value) throws IOException
     {
-        if (_fromStringCreator == null) {
-            return _createFromStringFallbacks(ctxt, value);
-        }
-        try {
-            return _fromStringCreator.call1(value);
-        } catch (Throwable t) {
-            return ctxt.handleInstantiationProblem(_fromStringCreator.getDeclaringClass(),
-                    value, rewrapCtorProblem(ctxt, t));
+        if (_fromStringCreator != null) {
+            try {
+                return _fromStringCreator.call1(value);
+            } catch (Throwable t) {
+                return ctxt.handleInstantiationProblem(_fromStringCreator.getDeclaringClass(),
+                        value, rewrapCtorProblem(ctxt, t));
+            }
         }
+        return super.createFromString(ctxt, value);
     }
-    
+
     @Override
     public Object createFromInt(DeserializationContext ctxt, int value) throws IOException
     {
@@ -353,37 +361,130 @@ public class StdValueInstantiator
                         arg, rewrapCtorProblem(ctxt, t0));
             }
         }
+
+        if (_fromBigIntegerCreator != null) {
+            Object arg = BigInteger.valueOf(value);
+            try {
+                return _fromBigIntegerCreator.call1(arg);
+            } catch (Throwable t0) {
+                return ctxt.handleInstantiationProblem(_fromBigIntegerCreator.getDeclaringClass(),
+                        arg, rewrapCtorProblem(ctxt, t0)
+                );
+            }
+        }
+
         return super.createFromInt(ctxt, value);
     }
 
     @Override
     public Object createFromLong(DeserializationContext ctxt, long value) throws IOException
     {
-        if (_fromLongCreator == null) {
-            return super.createFromLong(ctxt, value);
+        if (_fromLongCreator != null) {
+            Long arg = Long.valueOf(value);
+            try {
+                return _fromLongCreator.call1(arg);
+            } catch (Throwable t0) {
+                return ctxt.handleInstantiationProblem(_fromLongCreator.getDeclaringClass(),
+                        arg, rewrapCtorProblem(ctxt, t0)
+                );
+            }
         }
-        Object arg = Long.valueOf(value);
-        try {
-            return _fromLongCreator.call1(arg);
-        } catch (Throwable t0) {
-            return ctxt.handleInstantiationProblem(_fromLongCreator.getDeclaringClass(),
-                    arg, rewrapCtorProblem(ctxt, t0));
+
+        if (_fromBigIntegerCreator != null) {
+            BigInteger arg = BigInteger.valueOf(value);
+            try {
+                return _fromBigIntegerCreator.call1(arg);
+            } catch (Throwable t0) {
+                return ctxt.handleInstantiationProblem(_fromBigIntegerCreator.getDeclaringClass(),
+                        arg, rewrapCtorProblem(ctxt, t0)
+                );
+            }
         }
+
+        return super.createFromLong(ctxt, value);
+    }
+
+    @Override
+    public Object createFromBigInteger(DeserializationContext ctxt, BigInteger value) throws IOException
+    {
+        if (_fromBigIntegerCreator != null) {
+            try {
+                return _fromBigIntegerCreator.call1(value);
+            } catch (Throwable t) {
+                return ctxt.handleInstantiationProblem(_fromBigIntegerCreator.getDeclaringClass(),
+                        value, rewrapCtorProblem(ctxt, t)
+                );
+            }
+        }
+
+        return super.createFromBigInteger(ctxt, value);
     }
 
     @Override
     public Object createFromDouble(DeserializationContext ctxt, double value) throws IOException
     {
-        if (_fromDoubleCreator == null) {
-            return super.createFromDouble(ctxt, value);
+        if(_fromDoubleCreator != null) {
+            Double arg = Double.valueOf(value);
+            try {
+                return _fromDoubleCreator.call1(arg);
+            } catch (Throwable t0) {
+                return ctxt.handleInstantiationProblem(_fromDoubleCreator.getDeclaringClass(),
+                        arg, rewrapCtorProblem(ctxt, t0));
+            }
         }
-        Object arg = Double.valueOf(value);
-        try {
-            return _fromDoubleCreator.call1(arg);
-        } catch (Throwable t0) {
-            return ctxt.handleInstantiationProblem(_fromDoubleCreator.getDeclaringClass(),
-                    arg, rewrapCtorProblem(ctxt, t0));
+
+        if (_fromBigDecimalCreator != null) {
+            BigDecimal arg = BigDecimal.valueOf(value);
+            try {
+                return _fromBigDecimalCreator.call1(arg);
+            } catch (Throwable t0) {
+                return ctxt.handleInstantiationProblem(_fromBigDecimalCreator.getDeclaringClass(),
+                        arg, rewrapCtorProblem(ctxt, t0));
+            }
         }
+
+        return super.createFromDouble(ctxt, value);
+    }
+
+    @Override
+    public Object createFromBigDecimal(DeserializationContext ctxt, BigDecimal value) throws IOException
+    {
+        if (_fromBigDecimalCreator != null) {
+            try {
+                return _fromBigDecimalCreator.call1(value);
+            } catch (Throwable t) {
+                return ctxt.handleInstantiationProblem(_fromBigDecimalCreator.getDeclaringClass(),
+                        value, rewrapCtorProblem(ctxt, t)
+                );
+            }
+        }
+
+        // 13-Dec-2020, ckozak: Unlike other types, BigDecimal values may be represented
+        // with less precision as doubles. When written to a TokenBuffer for polymorphic
+        // deserialization the most specific type is recorded, though a less precise
+        // floating point value may be needed.
+        if (_fromDoubleCreator != null) {
+            Double dbl = tryConvertToDouble(value);
+            if (dbl != null) {
+                try {
+                    return _fromDoubleCreator.call1(dbl);
+                } catch (Throwable t0) {
+                    return ctxt.handleInstantiationProblem(_fromDoubleCreator.getDeclaringClass(),
+                            dbl, rewrapCtorProblem(ctxt, t0));
+                }
+            }
+        }
+
+        return super.createFromBigDecimal(ctxt, value);
+    }
+
+    // BigDecimal cannot represent special values NaN, positive infinity, or negative infinity.
+    // When the value cannot be represented as a double, positive or negative infinity is returned.
+    //
+    // @since 2.12.1
+    static Double tryConvertToDouble(BigDecimal value) {
+        double doubleValue = value.doubleValue();
+        return Double.isInfinite(doubleValue) ? null : doubleValue;
     }
 
     @Override
@@ -400,7 +501,7 @@ public class StdValueInstantiator
                     arg, rewrapCtorProblem(ctxt, t0));
         }
     }
-    
+
     /*
     /**********************************************************
     /* Extended API: configuration mutators, accessors
@@ -427,11 +528,6 @@ public class StdValueInstantiator
         return _withArgsCreator;
     }
 
-    @Override
-    public AnnotatedParameter getIncompleteParameter() {
-        return _incompleteParameter;
-    }
-
     /*
     /**********************************************************
     /* Internal methods
@@ -439,7 +535,7 @@ public class StdValueInstantiator
      */
 
     /**
-     * @deprecated Since 2.7 call either {@link #unwrapAndWrapException} or
+     * @deprecated Since 2.7 call either {@link #rewrapCtorProblem} or
      *  {@link #wrapAsJsonMappingException}
      */
     @Deprecated // since 2.7
@@ -457,8 +553,10 @@ public class StdValueInstantiator
     }
 
     /**
-     * @since 2.7
+     * @deprecated Since 2.7 call either {@link #rewrapCtorProblem} or
+     *  {@link #wrapAsJsonMappingException}
      */
+    @Deprecated // since 2.10
     protected JsonMappingException unwrapAndWrapException(DeserializationContext ctxt, Throwable t)
     {
         // 05-Nov-2015, tatu: This used to always unwrap the whole exception, but now only
@@ -472,6 +570,11 @@ public class StdValueInstantiator
     }
 
     /**
+     * Helper method that will return given {@link Throwable} case as
+     * a {@link JsonMappingException} (if it is of that type), or call
+     * {@link DeserializationContext#instantiationException(Class, Throwable)} to
+     * produce and return suitable {@link JsonMappingException}.
+     *
      * @since 2.7
      */
     protected JsonMappingException wrapAsJsonMappingException(DeserializationContext ctxt,
@@ -485,6 +588,10 @@ public class StdValueInstantiator
     }
 
     /**
+     * Method that subclasses may call for standard handling of an exception thrown when
+     * calling constructor or factory method. Will unwrap {@link ExceptionInInitializerError}
+     * and {@link InvocationTargetException}s, then call {@link #wrapAsJsonMappingException}.
+     *
      * @since 2.7
      */
     protected JsonMappingException rewrapCtorProblem(DeserializationContext ctxt,
