@@ -4,6 +4,7 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.util.*;
 
+import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.annotation.JsonInclude;
 
@@ -26,6 +27,9 @@ import com.fasterxml.jackson.databind.util.Converter;
  */
 public class BasicBeanDescription extends BeanDescription
 {
+    // since 2.9
+    private final static Class<?>[] NO_VIEWS = new Class<?>[0];
+
     /*
     /**********************************************************
     /* General configuration
@@ -42,12 +46,28 @@ public class BasicBeanDescription extends BeanDescription
     final protected MapperConfig<?> _config;
 
     final protected AnnotationIntrospector _annotationIntrospector;
+
+    /*
+    /**********************************************************
+    /* Information about type itself
+    /**********************************************************
+     */
     
     /**
      * Information collected about the class introspected.
      */
     final protected AnnotatedClass _classInfo;
 
+    /**
+     * @since 2.9
+     */
+    protected Class<?>[] _defaultViews;
+
+    /**
+     * @since 2.9
+     */
+    protected boolean _defaultViewsResolved;
+
     /*
     /**********************************************************
     /* Member information
@@ -219,12 +239,25 @@ public class BasicBeanDescription extends BeanDescription
         return _properties();
     }
 
+    @Override // since 2.12
+    public AnnotatedMember findJsonKeyAccessor() {
+        return (_propCollector == null) ? null
+                : _propCollector.getJsonKeyAccessor();
+    }
+
     @Override
+    @Deprecated // since 2.9
     public AnnotatedMethod findJsonValueMethod() {
         return (_propCollector == null) ? null
                 : _propCollector.getJsonValueMethod();
     }
 
+    @Override // since 2.9
+    public AnnotatedMember findJsonValueAccessor() {
+        return (_propCollector == null) ? null
+                : _propCollector.getJsonValueAccessor();
+    }
+ 
     @Override
     public Set<String> getIgnoredPropertyNames() {
         Set<String> ign = (_propCollector == null) ? null
@@ -254,10 +287,9 @@ public class BasicBeanDescription extends BeanDescription
     @Override
     @Deprecated // since 2.8
     public JavaType resolveType(java.lang.reflect.Type jdkType) {
-        if (jdkType == null) {
-            return null;
-        }
-        return _config.getTypeFactory().constructType(jdkType, _type.getBindings());
+        // 06-Sep-2020, tatu: Careful wrt [databind#2846][databind#2821],
+        //     call new method added in 2.12
+        return _config.getTypeFactory().resolveMemberType(jdkType, _type.getBindings());
     }
 
     @Override
@@ -266,25 +298,39 @@ public class BasicBeanDescription extends BeanDescription
     }
 
     @Override
-    public AnnotatedMethod findAnySetter() throws IllegalArgumentException
+    public AnnotatedMember findAnySetterAccessor() throws IllegalArgumentException
     {
-        AnnotatedMethod anySetter = (_propCollector == null) ? null
-                : _propCollector.getAnySetterMethod();
-        if (anySetter != null) {
-            /* Also, let's be somewhat strict on how field name is to be
-             * passed; String, Object make sense, others not
-             * so much.
-             */
-            /* !!! 18-May-2009, tatu: how about enums? Can add support if
-             *  requested; easy enough for devs to add support within
-             *  method.
-             */
-            Class<?> type = anySetter.getRawParameterType(0);
-            if (type != String.class && type != Object.class) {
-                throw new IllegalArgumentException("Invalid 'any-setter' annotation on method "+anySetter.getName()+"(): first argument not of type String or Object, but "+type.getName());
+        if (_propCollector != null) {
+            AnnotatedMethod anyMethod = _propCollector.getAnySetterMethod();
+            if (anyMethod != null) {
+                // Also, let's be somewhat strict on how field name is to be
+                // passed; String, Object make sense, others not so much.
+    
+                /* !!! 18-May-2009, tatu: how about enums? Can add support if
+                 *  requested; easy enough for devs to add support within method.
+                 */
+                Class<?> type = anyMethod.getRawParameterType(0);
+                if ((type != String.class) && (type != Object.class)) {
+                    throw new IllegalArgumentException(String.format(
+"Invalid 'any-setter' annotation on method '%s()': first argument not of type String or Object, but %s",
+anyMethod.getName(), type.getName()));
+                }
+                return anyMethod;
+            }
+            AnnotatedMember anyField = _propCollector.getAnySetterField();
+            if (anyField != null) {
+                // For now let's require a Map; in future can add support for other
+                // types like perhaps Iterable<Map.Entry>?
+                Class<?> type = anyField.getRawType();
+                if (!Map.class.isAssignableFrom(type)) {
+                    throw new IllegalArgumentException(String.format(
+"Invalid 'any-setter' annotation on field '%s': type is not instance of java.util.Map",
+anyField.getName()));
+                }
+                return anyField;
             }
         }
-        return anySetter;
+        return null;
     }
 
     @Override
@@ -316,9 +362,11 @@ public class BasicBeanDescription extends BeanDescription
             while (t.getCause() != null) {
                 t = t.getCause();
             }
-            if (t instanceof Error) throw (Error) t;
-            if (t instanceof RuntimeException) throw (RuntimeException) t;
-            throw new IllegalArgumentException("Failed to instantiate bean of type "+_classInfo.getAnnotated().getName()+": ("+t.getClass().getName()+") "+t.getMessage(), t);
+            ClassUtil.throwIfError(t);
+            ClassUtil.throwIfRTE(t);
+            throw new IllegalArgumentException("Failed to instantiate bean of type "
+                    +_classInfo.getAnnotated().getName()+": ("+t.getClass().getName()+") "
+                    +ClassUtil.exceptionMessage(t), t);
         }
     }
 
@@ -364,7 +412,25 @@ public class BasicBeanDescription extends BeanDescription
         }
         return defValue;
     }
-    
+
+    @Override // since 2.9
+    public Class<?>[] findDefaultViews()
+    {
+        if (!_defaultViewsResolved) {
+            _defaultViewsResolved = true;
+            Class<?>[] def = (_annotationIntrospector == null) ? null
+                    : _annotationIntrospector.findViews(_classInfo);
+            // one more twist: if default inclusion disabled, need to force empty set of views
+            if (def == null) {
+                if (!_config.isEnabled(MapperFeature.DEFAULT_VIEW_INCLUSION)) {
+                    def = NO_VIEWS;
+                }
+            }
+            _defaultViews = def;
+        }
+        return _defaultViews;
+    }
+
     /*
     /**********************************************************
     /* Introspection for serialization
@@ -406,69 +472,72 @@ public class BasicBeanDescription extends BeanDescription
     @Override
     public AnnotatedMember findAnyGetter() throws IllegalArgumentException
     {
-        AnnotatedMember anyGetter = (_propCollector == null) ? null
-                : _propCollector.getAnyGetter();
-        if (anyGetter != null) {
-            /* For now let's require a Map; in future can add support for other
-             * types like perhaps Iterable<Map.Entry>?
-             */
-            Class<?> type = anyGetter.getRawType();
-            if (!Map.class.isAssignableFrom(type)) {
-                throw new IllegalArgumentException("Invalid 'any-getter' annotation on method "+anyGetter.getName()+"(): return type is not instance of java.util.Map");
+        if (_propCollector != null) {
+            AnnotatedMember anyGetter = _propCollector.getAnyGetterMethod();
+            if (anyGetter != null) {
+                // For now let's require a Map; in future can add support for other
+                // types like perhaps Iterable<Map.Entry>?
+                Class<?> type = anyGetter.getRawType();
+                if (!Map.class.isAssignableFrom(type)) {
+                    throw new IllegalArgumentException(String.format(
+                            "Invalid 'any-getter' annotation on method %s(): return type is not instance of java.util.Map",
+                            anyGetter.getName()));
+                }
+                return anyGetter;
+            }
+
+            AnnotatedMember anyField = _propCollector.getAnyGetterField();
+            if (anyField != null) {
+                // For now let's require a Map; in future can add support for other
+                // types like perhaps Iterable<Map.Entry>?
+                Class<?> type = anyField.getRawType();
+                if (!Map.class.isAssignableFrom(type)) {
+                    throw new IllegalArgumentException(String.format(
+                            "Invalid 'any-getter' annotation on field '%s': type is not instance of java.util.Map",
+                            anyField.getName()));
+                }
+                return anyField;
             }
         }
-        return anyGetter;
+        return null;
     }
 
     @Override
-    public AnnotatedMember findAnySetterField() throws IllegalArgumentException {
-        AnnotatedMember anySetter = (_propCollector == null) ? null : _propCollector.getAnySetterField();
-		if (anySetter != null) {
-			/*
-			 * For now let's require a Map; in future can add support for other
-			 * types like perhaps Iterable<Map.Entry>?
-			 */
-			Class<?> type = anySetter.getRawType();
-			if (!Map.class.isAssignableFrom(type)) {
-				throw new IllegalArgumentException("Invalid 'any-setter' annotation on field " + anySetter.getName()
-				        + "(): type is not instance of java.util.Map");
-			}
-		}
-		return anySetter;
-	}
+    public List<BeanPropertyDefinition> findBackReferences()
+    {
+        List<BeanPropertyDefinition> result = null;
+        HashSet<String> names = null;
+        for (BeanPropertyDefinition property : _properties()) {
+            AnnotationIntrospector.ReferenceProperty refDef = property.findReferenceType();
+            if ((refDef == null) || !refDef.isBackReference()) {
+                continue;
+            }
+            final String refName = refDef.getName();
+            if (result == null) {
+                result = new ArrayList<BeanPropertyDefinition>();
+                names = new HashSet<>();
+                names.add(refName);
+            } else {
+                if (!names.add(refName)) {
+                    throw new IllegalArgumentException("Multiple back-reference properties with name "+ClassUtil.name(refName));
+                }
+            }
+            result.add(property);
+        }
+        return result;
+    }
 
+    @Deprecated // since 2.9
     @Override
     public Map<String,AnnotatedMember> findBackReferenceProperties()
     {
-        HashMap<String,AnnotatedMember> result = null;
-//        boolean hasIgnored = (_ignoredPropertyNames != null);
-
-        for (BeanPropertyDefinition property : _properties()) {
-            /* 23-Sep-2014, tatu: As per [Databind#426], we _should_ try to avoid
-             *   calling accessor, as it triggers exception from seeming conflict.
-             *   But the problem is that _ignoredPropertyNames here only contains
-             *   ones ignored on per-property annotations, but NOT class annotations...
-             *   so commented out part does not work, alas
-             */
-            /*
-            if (hasIgnored && _ignoredPropertyNames.contains(property.getName())) {
-                continue;
-            }
-            */
-            AnnotatedMember am = property.getMutator();
-            if (am == null) {
-                continue;
-            }
-            AnnotationIntrospector.ReferenceProperty refDef = _annotationIntrospector.findReferenceType(am);
-            if (refDef != null && refDef.isBackReference()) {
-                if (result == null) {
-                    result = new HashMap<String,AnnotatedMember>();
-                }
-                String refName = refDef.getName();
-                if (result.put(refName, am) != null) {
-                    throw new IllegalArgumentException("Multiple back-reference properties with name '"+refName+"'");
-                }
-            }
+        List<BeanPropertyDefinition> props = findBackReferences();
+        if (props == null) {
+            return null;
+        }
+        Map<String,AnnotatedMember> result = new HashMap<>();
+        for (BeanPropertyDefinition prop : props) {
+            result.put(prop.getName(), prop.getMutator());
         }
         return result;
     }
@@ -483,16 +552,22 @@ public class BasicBeanDescription extends BeanDescription
     public List<AnnotatedMethod> getFactoryMethods()
     {
         // must filter out anything that clearly is not a factory method
-        List<AnnotatedMethod> candidates = _classInfo.getStaticMethods();
+        List<AnnotatedMethod> candidates = _classInfo.getFactoryMethods();
         if (candidates.isEmpty()) {
             return candidates;
         }
-        ArrayList<AnnotatedMethod> result = new ArrayList<AnnotatedMethod>();
+        List<AnnotatedMethod> result = null;
         for (AnnotatedMethod am : candidates) {
             if (isFactoryMethod(am)) {
+                if (result == null) {
+                    result = new ArrayList<AnnotatedMethod>();
+                }
                 result.add(am);
             }
         }
+        if (result == null) {
+            return Collections.emptyList();
+        }
         return result;
     }
 
@@ -520,7 +595,7 @@ public class BasicBeanDescription extends BeanDescription
     public Method findFactoryMethod(Class<?>... expArgTypes)
     {
         // So, of all single-arg static methods:
-        for (AnnotatedMethod am : _classInfo.getStaticMethods()) {
+        for (AnnotatedMethod am : _classInfo.getFactoryMethods()) {
             // 24-Oct-2016, tatu: Better ensure it only takes 1 arg, no matter what
             if (isFactoryMethod(am) && am.getParameterCount() == 1) {
                 // And must take one of expected arg types (or supertype)
@@ -538,9 +613,8 @@ public class BasicBeanDescription extends BeanDescription
 
     protected boolean isFactoryMethod(AnnotatedMethod am)
     {
-        /* First: return type must be compatible with the introspected class
-         * (i.e. allowed to be sub-class, although usually is the same class)
-         */
+        // First: return type must be compatible with the introspected class
+        // (i.e. allowed to be sub-class, although usually is the same class)
         Class<?> rt = am.getRawReturnType();
         if (!getBeanClass().isAssignableFrom(rt)) {
             return false;
@@ -549,7 +623,8 @@ public class BasicBeanDescription extends BeanDescription
          * (a) marked with @JsonCreator annotation, or
          * (b) "valueOf" (at this point, need not be public)
          */
-        if (_annotationIntrospector.hasCreatorAnnotation(am)) {
+        JsonCreator.Mode mode = _annotationIntrospector.findCreatorAnnotation(_config, am);
+        if ((mode != null) && (mode != JsonCreator.Mode.DISABLED)) {
             return true;
         }
         final String name = am.getName();
@@ -666,7 +741,7 @@ public class BasicBeanDescription extends BeanDescription
      */
     
     @SuppressWarnings("unchecked")
-    public Converter<Object,Object> _createConverter(Object converterDef)
+    protected Converter<Object,Object> _createConverter(Object converterDef)
     {
         if (converterDef == null) {
             return null;
