@@ -1,5 +1,6 @@
 package com.fasterxml.jackson.databind.ser;
 
+import java.io.IOException;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
@@ -21,6 +22,7 @@ import com.fasterxml.jackson.databind.ser.impl.PropertySerializerMap;
 import com.fasterxml.jackson.databind.ser.impl.UnwrappingBeanPropertyWriter;
 import com.fasterxml.jackson.databind.ser.std.BeanSerializerBase;
 import com.fasterxml.jackson.databind.util.Annotations;
+import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 
 /**
@@ -134,7 +136,7 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
      */
 
     /**
-     * Serializer to use for writing out the value: null if it can not be known
+     * Serializer to use for writing out the value: null if it cannot be known
      * statically; non-null if it can.
      */
     protected JsonSerializer<Object> _serializer;
@@ -201,19 +203,23 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
     /***********************************************************
      */
 
+    /**
+     * @since 2.9 (added `includeInViews` since 2.8)
+     */
     @SuppressWarnings("unchecked")
     public BeanPropertyWriter(BeanPropertyDefinition propDef,
             AnnotatedMember member, Annotations contextAnnotations,
-            JavaType declaredType, JsonSerializer<?> ser,
-            TypeSerializer typeSer, JavaType serType, boolean suppressNulls,
-            Object suppressableValue) {
+            JavaType declaredType,
+            JsonSerializer<?> ser, TypeSerializer typeSer, JavaType serType,
+            boolean suppressNulls, Object suppressableValue,
+            Class<?>[] includeInViews)
+    {
         super(propDef);
         _member = member;
         _contextAnnotations = contextAnnotations;
 
         _name = new SerializedString(propDef.getName());
         _wrapperName = propDef.getWrapperName();
-        _includeInViews = propDef.findViews();
 
         _declaredType = declaredType;
         _serializer = (JsonSerializer<Object>) ser;
@@ -239,6 +245,19 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
 
         // this will be resolved later on, unless nulls are to be suppressed
         _nullSerializer = null;
+        _includeInViews = includeInViews;
+    }
+
+    @Deprecated // Since 2.9
+    public BeanPropertyWriter(BeanPropertyDefinition propDef,
+            AnnotatedMember member, Annotations contextAnnotations,
+            JavaType declaredType,
+            JsonSerializer<?> ser, TypeSerializer typeSer, JavaType serType,
+            boolean suppressNulls, Object suppressableValue)
+    {
+        this(propDef, member, contextAnnotations, declaredType,
+                ser, typeSer, serType, suppressNulls, suppressableValue,
+                null);
     }
 
     /**
@@ -372,8 +391,10 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
      */
     public void assignSerializer(JsonSerializer<Object> ser) {
         // may need to disable check in future?
-        if (_serializer != null && _serializer != ser) {
-            throw new IllegalStateException("Can not override serializer");
+        if ((_serializer != null) && (_serializer != ser)) {
+            throw new IllegalStateException(String.format(
+                    "Cannot override _serializer: had a %s, trying to set to %s",
+                    ClassUtil.classNameOf(_serializer), ClassUtil.classNameOf(ser)));
         }
         _serializer = ser;
     }
@@ -384,7 +405,9 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
     public void assignNullSerializer(JsonSerializer<Object> nullSer) {
         // may need to disable check in future?
         if ((_nullSerializer != null) && (_nullSerializer != nullSer)) {
-            throw new IllegalStateException("Can not override null serializer");
+            throw new IllegalStateException(String.format(
+                    "Cannot override _nullSerializer: had a %s, trying to set to %s",
+                    ClassUtil.classNameOf(_nullSerializer), ClassUtil.classNameOf(nullSer)));
         }
         _nullSerializer = nullSer;
     }
@@ -605,6 +628,7 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
         return _cfgSerializationType;
     }
 
+    @Deprecated // since 2.9
     public Class<?> getRawSerializationType() {
         return (_cfgSerializationType == null) ? null : _cfgSerializationType
                 .getRawClass();
@@ -662,7 +686,7 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
             SerializerProvider prov) throws Exception {
         // inlined 'get()'
         final Object value = (_accessorMethod == null) ? _field.get(bean)
-                : _accessorMethod.invoke(bean);
+                : _accessorMethod.invoke(bean, (Object[]) null);
 
         // Null handling is bit different, check that first
         if (value == null) {
@@ -694,7 +718,7 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
         }
         // For non-nulls: simple check for direct cycles
         if (value == bean) {
-            // three choices: exception; handled by call; or pass-through
+            // four choices: exception; handled by call; pass-through or write null
             if (_handleSelfReference(bean, gen, prov, ser)) {
                 return;
             }
@@ -734,7 +758,7 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
             SerializerProvider prov) throws Exception {
         // inlined 'get()'
         final Object value = (_accessorMethod == null) ? _field.get(bean)
-                : _accessorMethod.invoke(bean);
+                : _accessorMethod.invoke(bean, (Object[]) null);
         if (value == null) { // nulls need specialized handling
             if (_nullSerializer != null) {
                 _nullSerializer.serialize(null, gen, prov);
@@ -761,9 +785,8 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
                     serializeAsPlaceholder(bean, gen, prov);
                     return;
                 }
-            } else if (_suppressableValue.equals(value)) { // can NOT suppress
-                                                           // entries in tabular
-                                                           // output
+            } else if (_suppressableValue.equals(value)) {
+                // can NOT suppress entries in tabular output
                 serializeAsPlaceholder(bean, gen, prov);
                 return;
             }
@@ -890,7 +913,7 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
      */
     public final Object get(Object bean) throws Exception {
         return (_accessorMethod == null) ? _field.get(bean) : _accessorMethod
-                .invoke(bean);
+                .invoke(bean, (Object[]) null);
     }
 
     /**
@@ -909,16 +932,30 @@ public class BeanPropertyWriter extends PropertyWriter // which extends
      */
     protected boolean _handleSelfReference(Object bean, JsonGenerator gen,
             SerializerProvider prov, JsonSerializer<?> ser)
-            throws JsonMappingException {
-        if (prov.isEnabled(SerializationFeature.FAIL_ON_SELF_REFERENCES)
-                && !ser.usesObjectId()) {
-            // 05-Feb-2013, tatu: Usually a problem, but NOT if we are handling
-            // object id; this may be the case for BeanSerializers at least.
-            // 13-Feb-2014, tatu: another possible ok case: custom serializer
-            // (something
-            // OTHER than {@link BeanSerializerBase}
-            if (ser instanceof BeanSerializerBase) {
-                prov.reportMappingProblem("Direct self-reference leading to cycle");
+            throws IOException
+    {
+        if (!ser.usesObjectId()) {
+            if (prov.isEnabled(SerializationFeature.FAIL_ON_SELF_REFERENCES)) {
+                // 05-Feb-2013, tatu: Usually a problem, but NOT if we are handling
+                // object id; this may be the case for BeanSerializers at least.
+                // 13-Feb-2014, tatu: another possible ok case: custom serializer
+                // (something OTHER than {@link BeanSerializerBase}
+                if (ser instanceof BeanSerializerBase) {
+                    prov.reportBadDefinition(getType(), "Direct self-reference leading to cycle");
+                }
+            } else if (prov.isEnabled(SerializationFeature.WRITE_SELF_REFERENCES_AS_NULL)) {
+                if (_nullSerializer != null) {
+                    // 23-Oct-2019, tatu: Tricky part -- caller does not specify if it's
+                    //   "as property" (in JSON Object) or "as element" (JSON array, via
+                    //   'POJO-as-array'). And since Afterburner calls method can not easily
+                    //   start passing info either. So check generator to see...
+                    //   (note: not considering ROOT context as possibility, does not seem legal)
+                    if (!gen.getOutputContext().inArray()) {
+                        gen.writeFieldName(_name);
+                    }
+                    _nullSerializer.serialize(null, gen, prov);
+                }
+                return true;
             }
         }
         return false;
