@@ -3,6 +3,7 @@ package com.fasterxml.jackson.databind.ser;
 import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.annotation.JsonIncludeProperties;
 import com.fasterxml.jackson.annotation.ObjectIdGenerator;
 import com.fasterxml.jackson.annotation.ObjectIdGenerators;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
@@ -16,11 +17,14 @@ import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.ser.impl.FilteredBeanPropertyWriter;
 import com.fasterxml.jackson.databind.ser.impl.ObjectIdWriter;
 import com.fasterxml.jackson.databind.ser.impl.PropertyBasedObjectIdGenerator;
+import com.fasterxml.jackson.databind.ser.impl.UnsupportedTypeSerializer;
 import com.fasterxml.jackson.databind.ser.std.MapSerializer;
 import com.fasterxml.jackson.databind.ser.std.StdDelegatingSerializer;
 import com.fasterxml.jackson.databind.type.ReferenceType;
+import com.fasterxml.jackson.databind.util.BeanUtil;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.Converter;
+import com.fasterxml.jackson.databind.util.IgnorePropertiesUtil;
 
 /**
  * Factory class that can provide serializers for any regular Java beans
@@ -228,16 +232,12 @@ public class BeanSerializerFactory
                     // And this is where this class comes in: if type is not a
                     // known "primary JDK type", perhaps it's a bean? We can still
                     // get a null, if we can't find a single suitable bean property.
-                    ser = findBeanSerializer(prov, type, beanDesc);
-                    // Finally: maybe we can still deal with it as an implementation of some basic JDK interface?
+                    ser = findBeanOrAddOnSerializer(prov, type, beanDesc, staticTyping);
+                    // 18-Sep-2014, tatu: Actually, as per [jackson-databind#539], need to get
+                    //   'unknown' serializer assigned earlier, here, so that it gets properly
+                    //   post-processed
                     if (ser == null) {
-                        ser = findSerializerByAddonType(config, type, beanDesc, staticTyping);
-                        // 18-Sep-2014, tatu: Actually, as per [jackson-databind#539], need to get
-                        //   'unknown' serializer assigned earlier, here, so that it gets properly
-                        //   post-processed
-                        if (ser == null) {
-                            ser = prov.getUnknownTypeSerializer(beanDesc.getBeanClass());
-                        }
+                        ser = prov.getUnknownTypeSerializer(beanDesc.getBeanClass());
                     }
                 }
             }
@@ -260,23 +260,34 @@ public class BeanSerializerFactory
     /**********************************************************
      */
 
-    /**
-     * Method that will try to construct a {@link BeanSerializer} for
-     * given class. Returns null if no properties are found.
-     */
+    @Deprecated // since 2.10
     public JsonSerializer<Object> findBeanSerializer(SerializerProvider prov, JavaType type,
             BeanDescription beanDesc)
         throws JsonMappingException
+    {
+        return findBeanOrAddOnSerializer(prov, type, beanDesc, prov.isEnabled(MapperFeature.USE_STATIC_TYPING));
+    }
+
+    /**
+     * Method that will try to construct a {@link BeanSerializer} for
+     * given class if at least one property is found, OR, if not,
+     * one of add-on types.
+     *<p>
+     * NOTE: behavior changed a bit
+     */
+    public JsonSerializer<Object> findBeanOrAddOnSerializer(SerializerProvider prov, JavaType type,
+            BeanDescription beanDesc, boolean staticTyping)
+        throws JsonMappingException
     {
         // First things first: we know some types are not beans...
         if (!isPotentialBeanType(type.getRawClass())) {
             // 03-Aug-2012, tatu: Except we do need to allow serializers for Enums,
-            //   as per [databind#24]
-            if (!type.isEnumType()) {
+            //   as per [databind#24], [databind#2576]
+            if (!ClassUtil.isEnumType(type.getRawClass())) {
                 return null;
             }
         }
-        return constructBeanSerializer(prov, beanDesc);
+        return constructBeanOrAddOnSerializer(prov, type, beanDesc, staticTyping);
     }
 
     /**
@@ -344,15 +355,24 @@ public class BeanSerializerFactory
     /**********************************************************
      */
 
-    /**
-     * Method called to construct serializer for serializing specified bean type.
-     * 
-     * @since 2.1
-     */
-    @SuppressWarnings("unchecked")
+    @Deprecated // since 2.10
     protected JsonSerializer<Object> constructBeanSerializer(SerializerProvider prov,
             BeanDescription beanDesc)
         throws JsonMappingException
+    {
+        return constructBeanOrAddOnSerializer(prov, beanDesc.getType(), beanDesc, prov.isEnabled(MapperFeature.USE_STATIC_TYPING));
+    }
+
+    /**
+     * Method called to construct serializer for serializing specified bean type if
+     * (but only if, as of 2.10), at least one property is found.
+     * 
+     * @since 2.10
+     */
+    @SuppressWarnings("unchecked")
+    protected JsonSerializer<Object> constructBeanOrAddOnSerializer(SerializerProvider prov,
+            JavaType type, BeanDescription beanDesc, boolean staticTyping)
+        throws JsonMappingException
     {
         // 13-Oct-2010, tatu: quick sanity check: never try to create bean serializer for plain Object
         // 05-Jul-2012, tatu: ... but we should be able to just return "unknown type" serializer, right?
@@ -360,6 +380,12 @@ public class BeanSerializerFactory
             return prov.getUnknownTypeSerializer(Object.class);
 //            throw new IllegalArgumentException("Cannot create bean serializer for Object.class");
         }
+
+        JsonSerializer<?> ser = _findUnsupportedTypeSerializer(prov, type, beanDesc);
+        if (ser != null) {
+            return (JsonSerializer<Object>) ser;
+        }
+
         final SerializationConfig config = prov.getConfig();
         BeanSerializerBuilder builder = constructBeanSerializerBuilder(beanDesc);
         builder.setConfig(config);
@@ -402,10 +428,9 @@ public class BeanSerializerFactory
 
         AnnotatedMember anyGetter = beanDesc.findAnyGetter();
         if (anyGetter != null) {
-            JavaType type = anyGetter.getType();
+            JavaType anyType = anyGetter.getType();
             // copied from BasicSerializerFactory.buildMapSerializer():
-            boolean staticTyping = config.isEnabled(MapperFeature.USE_STATIC_TYPING);
-            JavaType valueType = type.getContentType();
+            JavaType valueType = anyType.getContentType();
             TypeSerializer typeSer = createTypeSerializer(config, valueType);
             // last 2 nulls; don't know key, value serializers (yet)
             // 23-Feb-2015, tatu: As per [databind#705], need to support custom serializers
@@ -413,7 +438,8 @@ public class BeanSerializerFactory
             if (anySer == null) {
                 // TODO: support '@JsonIgnoreProperties' with any setter?
                 anySer = MapSerializer.construct(/* ignored props*/ (Set<String>) null,
-                        type, staticTyping, typeSer, null, null, /*filterId*/ null);
+                        anyType, config.isEnabled(MapperFeature.USE_STATIC_TYPING),
+                        typeSer, null, null, /*filterId*/ null);
             }
             // TODO: can we find full PropertyName?
             PropertyName name = PropertyName.construct(anyGetter.getName());
@@ -431,22 +457,31 @@ public class BeanSerializerFactory
             }
         }
 
-        JsonSerializer<Object> ser = null;
         try {
-            ser = (JsonSerializer<Object>) builder.build();
+            ser = builder.build();
         } catch (RuntimeException e) {
-            prov.reportBadTypeDefinition(beanDesc, "Failed to construct BeanSerializer for %s: (%s) %s",
+            return prov.reportBadTypeDefinition(beanDesc, "Failed to construct BeanSerializer for %s: (%s) %s",
                     beanDesc.getType(), e.getClass().getName(), e.getMessage());
         }
-        if (ser == null) {
-            // If we get this far, there were no properties found, so no regular BeanSerializer
-            // would be constructed. But, couple of exceptions.
-            // First: if there are known annotations, just create 'empty bean' serializer
-            if (beanDesc.hasKnownClassAnnotations()) {
+        if (ser == null) { // Means that no properties were found
+            // 21-Aug-2020, tatu: Empty Records should be fine tho
+            if (type.isRecordType()) {
                 return builder.createDummy();
             }
+
+            // 06-Aug-2019, tatu: As per [databind#2390], we need to check for add-ons here,
+            //    before considering fallbacks
+            ser = (JsonSerializer<Object>) findSerializerByAddonType(config, type, beanDesc, staticTyping);
+            if (ser == null) {
+                // If we get this far, there were no properties found, so no regular BeanSerializer
+                // would be constructed. But, couple of exceptions.
+                // First: if there are known annotations, just create 'empty bean' serializer
+                if (beanDesc.hasKnownClassAnnotations()) {
+                    return builder.createDummy();
+                }
+            }
         }
-        return ser;
+        return (JsonSerializer<Object>) ser;
     }
 
     protected ObjectIdWriter constructObjectIdHandler(SerializerProvider prov,
@@ -467,8 +502,9 @@ public class BeanSerializerFactory
 
             for (int i = 0, len = props.size() ;; ++i) {
                 if (i == len) {
-                    throw new IllegalArgumentException("Invalid Object Id definition for "+beanDesc.getBeanClass().getName()
-                            +": cannot find property with name '"+propName+"'");
+                    throw new IllegalArgumentException(String.format(
+"Invalid Object Id definition for %s: cannot find property with name %s",
+ClassUtil.getTypeDescription(beanDesc.getType()), ClassUtil.name(propName)));
                 }
                 BeanPropertyWriter prop = props.get(i);
                 if (propName.equals(prop.getName())) {
@@ -507,7 +543,7 @@ public class BeanSerializerFactory
     {
         return FilteredBeanPropertyWriter.constructViewBased(writer, inViews);
     }
-    
+
     protected PropertyBuilder constructPropertyBuilder(SerializationConfig config,
             BeanDescription beanDesc)
     {
@@ -517,7 +553,7 @@ public class BeanSerializerFactory
     protected BeanSerializerBuilder constructBeanSerializerBuilder(BeanDescription beanDesc) {
         return new BeanSerializerBuilder(beanDesc);
     }
-    
+
     /*
     /**********************************************************
     /* Overridable non-public introspection methods
@@ -607,17 +643,25 @@ public class BeanSerializerFactory
         //   just use it as is.
         JsonIgnoreProperties.Value ignorals = config.getDefaultPropertyIgnorals(beanDesc.getBeanClass(),
                 beanDesc.getClassInfo());
+        Set<String> ignored = null;
         if (ignorals != null) {
-            Set<String> ignored = ignorals.findIgnoredForSerialization();
-            if (!ignored.isEmpty()) {
-                Iterator<BeanPropertyWriter> it = props.iterator();
-                while (it.hasNext()) {
-                    if (ignored.contains(it.next().getName())) {
-                        it.remove();
-                    }
+            ignored = ignorals.findIgnoredForSerialization();
+        }
+        JsonIncludeProperties.Value inclusions = config.getDefaultPropertyInclusions(beanDesc.getBeanClass(),
+                beanDesc.getClassInfo());
+        Set<String> included = null;
+        if (inclusions != null) {
+            included = inclusions.getIncluded();
+        }
+        if (included != null || (ignored != null && !ignored.isEmpty())) {
+            Iterator<BeanPropertyWriter> it = props.iterator();
+            while (it.hasNext()) {
+                if (IgnorePropertiesUtil.shouldIgnore(it.next().getName(), ignored, included)) {
+                    it.remove();
                 }
             }
         }
+
         return props;
     }
 
@@ -642,7 +686,9 @@ public class BeanSerializerFactory
         for (int i = 0; i < propCount; ++i) {
             BeanPropertyWriter bpw = props.get(i);
             Class<?>[] views = bpw.getViews();
-            if (views == null) { // no view info? include or exclude by default?
+            if (views == null
+                    // [databind#2311]: sometimes we add empty array
+                    || views.length == 0) { // no view info? include or exclude by default?
                 if (includeByDefault) {
                     filtered[i] = bpw;
                 }
@@ -791,4 +837,21 @@ public class BeanSerializerFactory
         return pb.buildWriter(prov, propDef, type, annotatedSerializer,
                         typeSer, contentTypeSer, accessor, staticTyping);
     }
+
+    protected JsonSerializer<?> _findUnsupportedTypeSerializer(SerializerProvider ctxt,
+            JavaType type, BeanDescription beanDesc)
+        throws JsonMappingException
+    {
+        // 05-May-2020, tatu: Should we check for possible Shape override to "POJO"?
+        //   (to let users force 'serialize-as-POJO'?
+        final String errorMsg = BeanUtil.checkUnsupportedType(type);
+        if (errorMsg != null) {
+            // 30-Sep-2020, tatu: [databind#2867] Avoid checks if there is a mix-in
+            //    which likely providers a handler...
+            if (ctxt.getConfig().findMixInClassFor(type.getRawClass()) == null) {
+                return new UnsupportedTypeSerializer(type, errorMsg);
+            }
+        }
+        return null;
+    }
 }
