@@ -9,6 +9,7 @@ import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.base.ParserMinimalBase;
 import com.fasterxml.jackson.core.json.JsonWriteContext;
 import com.fasterxml.jackson.core.util.ByteArrayBuilder;
+import com.fasterxml.jackson.core.util.JacksonFeatureSet;
 import com.fasterxml.jackson.databind.*;
 
 /**
@@ -157,7 +158,7 @@ public class TokenBuffer
         _hasNativeTypeIds = hasNativeIds;
         _hasNativeObjectIds = hasNativeIds;
 
-        _mayHaveNativeIds = _hasNativeTypeIds | _hasNativeObjectIds;
+        _mayHaveNativeIds = _hasNativeTypeIds || _hasNativeObjectIds;
     }
 
     /**
@@ -181,7 +182,7 @@ public class TokenBuffer
         _appendAt = 0;
         _hasNativeTypeIds = p.canReadTypeId();
         _hasNativeObjectIds = p.canReadObjectId();
-        _mayHaveNativeIds = _hasNativeTypeIds | _hasNativeObjectIds;
+        _mayHaveNativeIds = _hasNativeTypeIds || _hasNativeObjectIds;
         _forceBigDecimal = (ctxt == null) ? false
                 : ctxt.isEnabled(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS);
     }
@@ -321,7 +322,7 @@ public class TokenBuffer
         if (!_hasNativeObjectIds) {
             _hasNativeObjectIds = other.canWriteObjectId();
         }
-        _mayHaveNativeIds = _hasNativeTypeIds | _hasNativeObjectIds;
+        _mayHaveNativeIds = _hasNativeTypeIds || _hasNativeObjectIds;
         
         JsonParser p = other.asParser();
         while (p.nextToken() != null) {
@@ -477,7 +478,7 @@ public class TokenBuffer
      */
     public TokenBuffer deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        if (p.getCurrentTokenId() != JsonToken.FIELD_NAME.id()) {
+        if (!p.hasToken(JsonToken.FIELD_NAME)) {
             copyCurrentStructure(p);
             return this;
         }
@@ -535,7 +536,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
                     sb.append(t.toString());
                     if (t == JsonToken.FIELD_NAME) {
                         sb.append('(');
-                        sb.append(jp.getCurrentName());
+                        sb.append(jp.currentName());
                         sb.append(')');
                     }
                 }
@@ -639,7 +640,16 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
     public boolean canWriteBinaryNatively() {
         return true;
     }
-    
+
+    // 20-May-2020, tatu: This may or may not be enough -- ideally access is
+    //    via `DeserializationContext`, not parser, but if latter is needed
+    //    then we'll need to pass this from parser contents if which were
+    //    buffered.
+    @Override
+    public JacksonFeatureSet<StreamWriteCapability> getWriteCapabilities() {
+        return DEFAULT_WRITE_CAPABILITIES;
+    }
+
     /*
     /**********************************************************
     /* JsonGenerator implementation: low-level output handling
@@ -667,14 +677,28 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
     public final void writeStartArray() throws IOException
     {
         _writeContext.writeValue();
-        _append(JsonToken.START_ARRAY);
+        _appendStartMarker(JsonToken.START_ARRAY);
         _writeContext = _writeContext.createChildArrayContext();
     }
 
+    @Override // since 2.10.1
+    public void writeStartArray(Object forValue) throws IOException {
+        _writeContext.writeValue();
+        _appendStartMarker(JsonToken.START_ARRAY);
+        _writeContext = _writeContext.createChildArrayContext(forValue);
+    }
+
+    @Override // since 2.10.1
+    public void writeStartArray(Object forValue, int size) throws IOException {
+        _writeContext.writeValue();
+        _appendStartMarker(JsonToken.START_ARRAY);
+        _writeContext = _writeContext.createChildArrayContext(forValue);
+    }
+
     @Override
     public final void writeEndArray() throws IOException
     {
-        _append(JsonToken.END_ARRAY);
+        _appendEndMarker(JsonToken.END_ARRAY);
         // Let's allow unbalanced tho... i.e. not run out of root level, ever
         JsonWriteContext c = _writeContext.getParent();
         if (c != null) {
@@ -686,7 +710,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
     public final void writeStartObject() throws IOException
     {
         _writeContext.writeValue();
-        _append(JsonToken.START_OBJECT);
+        _appendStartMarker(JsonToken.START_OBJECT);
         _writeContext = _writeContext.createChildObjectContext();
     }
 
@@ -694,18 +718,24 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
     public void writeStartObject(Object forValue) throws IOException
     {
         _writeContext.writeValue();
-        _append(JsonToken.START_OBJECT);
-        JsonWriteContext ctxt = _writeContext.createChildObjectContext();
+        _appendStartMarker(JsonToken.START_OBJECT);
+        JsonWriteContext ctxt = _writeContext.createChildObjectContext(forValue);
+        _writeContext = ctxt;
+    }
+
+    @Override // since 2.10.1
+    public void writeStartObject(Object forValue, int size) throws IOException
+    {
+        _writeContext.writeValue();
+        _appendStartMarker(JsonToken.START_OBJECT);
+        JsonWriteContext ctxt = _writeContext.createChildObjectContext(forValue);
         _writeContext = ctxt;
-        if (forValue != null) {
-            ctxt.setCurrentValue(forValue);
-        }
     }
 
     @Override
     public final void writeEndObject() throws IOException
     {
-        _append(JsonToken.END_OBJECT);
+        _appendEndMarker(JsonToken.END_OBJECT);
         // Let's allow unbalanced tho... i.e. not run out of root level, ever
         JsonWriteContext c = _writeContext.getParent();
         if (c != null) {
@@ -717,16 +747,16 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
     public final void writeFieldName(String name) throws IOException
     {
         _writeContext.writeFieldName(name);
-        _append(JsonToken.FIELD_NAME, name);
+        _appendFieldName(name);
     }
 
     @Override
     public void writeFieldName(SerializableString name) throws IOException
     {
         _writeContext.writeFieldName(name.getValue());
-        _append(JsonToken.FIELD_NAME, name);
+        _appendFieldName(name);
     }
-    
+
     /*
     /**********************************************************
     /* JsonGenerator implementation: write methods, textual
@@ -1001,7 +1031,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
         if (_mayHaveNativeIds) {
             _checkNativeIds(p);
         }
-        switch (p.getCurrentToken()) {
+        switch (p.currentToken()) {
         case START_OBJECT:
             writeStartObject();
             break;
@@ -1015,7 +1045,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             writeEndArray();
             break;
         case FIELD_NAME:
-            writeFieldName(p.getCurrentName());
+            writeFieldName(p.currentName());
             break;
         case VALUE_STRING:
             if (p.hasTextCharacters()) {
@@ -1038,11 +1068,10 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             break;
         case VALUE_NUMBER_FLOAT:
             if (_forceBigDecimal) {
-                /* 10-Oct-2015, tatu: Ideally we would first determine whether underlying
-                 *   number is already decoded into a number (in which case might as well
-                 *   access as number); or is still retained as text (in which case we
-                 *   should further defer decoding that may not need BigDecimal):
-                 */
+                // 10-Oct-2015, tatu: Ideally we would first determine whether underlying
+                //   number is already decoded into a number (in which case might as well
+                //   access as number); or is still retained as text (in which case we
+                //   should further defer decoding that may not need BigDecimal):
                 writeNumber(p.getDecimalValue());
             } else {
                 switch (p.getNumberType()) {
@@ -1070,71 +1099,184 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             writeObject(p.getEmbeddedObject());
             break;
         default:
-            throw new RuntimeException("Internal error: should never end up through this code path");
+            throw new RuntimeException("Internal error: unexpected token: "+p.currentToken());
         }
     }
-    
+
     @Override
     public void copyCurrentStructure(JsonParser p) throws IOException
     {
-        JsonToken t = p.getCurrentToken();
+        JsonToken t = p.currentToken();
 
         // Let's handle field-name separately first
         if (t == JsonToken.FIELD_NAME) {
             if (_mayHaveNativeIds) {
                 _checkNativeIds(p);
             }
-            writeFieldName(p.getCurrentName());
+            writeFieldName(p.currentName());
             t = p.nextToken();
             // fall-through to copy the associated value
+        } else if (t == null) {
+            throw new IllegalStateException("No token available from argument `JsonParser`");
         }
 
-        if (_mayHaveNativeIds) {
-            _checkNativeIds(p);
-        }
-        
+        // We'll do minor handling here to separate structured, scalar values,
+        // then delegate appropriately.
+        // Plus also deal with oddity of "dangling" END_OBJECT/END_ARRAY
         switch (t) {
         case START_ARRAY:
-            writeStartArray();
-            while (p.nextToken() != JsonToken.END_ARRAY) {
-                copyCurrentStructure(p);
+            if (_mayHaveNativeIds) {
+                _checkNativeIds(p);
             }
-            writeEndArray();
+            writeStartArray();
+            _copyBufferContents(p);
             break;
         case START_OBJECT:
-            writeStartObject();
-            while (p.nextToken() != JsonToken.END_OBJECT) {
-                copyCurrentStructure(p);
+            if (_mayHaveNativeIds) {
+                _checkNativeIds(p);
             }
+            writeStartObject();
+            _copyBufferContents(p);
+            break;
+        case END_ARRAY:
+            writeEndArray();
+            break;
+        case END_OBJECT:
             writeEndObject();
             break;
         default: // others are simple:
-            copyCurrentEvent(p);
+            _copyBufferValue(p, t);
         }
     }
 
-    
-    private final void _checkNativeIds(JsonParser jp) throws IOException
+    protected void _copyBufferContents(JsonParser p) throws IOException
     {
-        if ((_typeId = jp.getTypeId()) != null) {
-            _hasNativeId = true;
+        int depth = 1;
+        JsonToken t;
+
+        while ((t = p.nextToken()) != null) {
+            switch (t) {
+            case FIELD_NAME:
+                if (_mayHaveNativeIds) {
+                    _checkNativeIds(p);
+                }
+                writeFieldName(p.currentName());
+                break;
+
+            case START_ARRAY:
+                if (_mayHaveNativeIds) {
+                    _checkNativeIds(p);
+                }
+                writeStartArray();
+                ++depth;
+                break;
+
+            case START_OBJECT:
+                if (_mayHaveNativeIds) {
+                    _checkNativeIds(p);
+                }
+                writeStartObject();
+                ++depth;
+                break;
+
+            case END_ARRAY:
+                writeEndArray();
+                if (--depth == 0) {
+                    return;
+                }
+                break;
+            case END_OBJECT:
+                writeEndObject();
+                if (--depth == 0) {
+                    return;
+                }
+                break;
+
+            default:
+                _copyBufferValue(p, t);
+            }
         }
-        if ((_objectId = jp.getObjectId()) != null) {
-            _hasNativeId = true;
+    }
+
+    // NOTE: Copied from earlier `copyCurrentEvent()`
+    private void _copyBufferValue(JsonParser p, JsonToken t) throws IOException
+    {
+        if (_mayHaveNativeIds) {
+            _checkNativeIds(p);
+        }
+        switch (t) {
+        case VALUE_STRING:
+            if (p.hasTextCharacters()) {
+                writeString(p.getTextCharacters(), p.getTextOffset(), p.getTextLength());
+            } else {
+                writeString(p.getText());
+            }
+            break;
+        case VALUE_NUMBER_INT:
+            switch (p.getNumberType()) {
+            case INT:
+                writeNumber(p.getIntValue());
+                break;
+            case BIG_INTEGER:
+                writeNumber(p.getBigIntegerValue());
+                break;
+            default:
+                writeNumber(p.getLongValue());
+            }
+            break;
+        case VALUE_NUMBER_FLOAT:
+            if (_forceBigDecimal) {
+                writeNumber(p.getDecimalValue());
+            } else {
+                // 09-Jul-2020, tatu: Used to just copy using most optimal method, but
+                //  issues like [databind#2644] force to use exact, not optimal type
+                final Number n = p.getNumberValueExact();
+                _appendValue(JsonToken.VALUE_NUMBER_FLOAT, n);
+            }
+            break;
+        case VALUE_TRUE:
+            writeBoolean(true);
+            break;
+        case VALUE_FALSE:
+            writeBoolean(false);
+            break;
+        case VALUE_NULL:
+            writeNull();
+            break;
+        case VALUE_EMBEDDED_OBJECT:
+            writeObject(p.getEmbeddedObject());
+            break;
+        default:
+            throw new RuntimeException("Internal error: unexpected token: "+t);
         }
     }
     
+    private final void _checkNativeIds(JsonParser p) throws IOException
+    {
+        if ((_typeId = p.getTypeId()) != null) {
+            _hasNativeId = true;
+        }
+        if ((_objectId = p.getObjectId()) != null) {
+            _hasNativeId = true;
+        }
+    }
+
     /*
     /**********************************************************
     /* Internal methods
     /**********************************************************
      */
 
+    /*// Not used in / since 2.10
     protected final void _append(JsonToken type)
     {
-        Segment next = _hasNativeId
-                ? _last.append(_appendAt, type, _objectId, _typeId)
-                : _last.append(_appendAt, type);
+        Segment next;
+
+        if (_hasNativeId) {
+            next =_last.append(_appendAt, type, _objectId, _typeId);
+        } else {
+            next =  _last.append(_appendAt, type);
+        }
         if (next == null) {
             ++_appendAt;
         } else {
@@ -1145,9 +1287,12 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
 
     protected final void _append(JsonToken type, Object value)
     {
-        Segment next = _hasNativeId
-                ? _last.append(_appendAt, type, value, _objectId, _typeId)
-                : _last.append(_appendAt, type, value);
+        Segment next;
+        if (_hasNativeId) {
+            next =  _last.append(_appendAt, type, value, _objectId, _typeId);
+        } else {
+            next = _last.append(_appendAt, type, value);
+        }
         if (next == null) {
             ++_appendAt;
         } else {
@@ -1155,19 +1300,23 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             _appendAt = 1;
         }
     }
+    */
 
     /**
-     * Similar to {@link #_append(JsonToken)} but also updates context with
-     * knowledge that a scalar value was written
+     * Method used for appending token known to represent a "simple" scalar
+     * value where token is the only information
      *
      * @since 2.6.4
      */
     protected final void _appendValue(JsonToken type)
     {
         _writeContext.writeValue();
-        Segment next = _hasNativeId
-                ? _last.append(_appendAt, type, _objectId, _typeId)
-                : _last.append(_appendAt, type);
+        Segment next;
+        if (_hasNativeId) {
+            next = _last.append(_appendAt, type, _objectId, _typeId);
+        } else {
+            next = _last.append(_appendAt, type);
+        }
         if (next == null) {
             ++_appendAt;
         } else {
@@ -1177,17 +1326,20 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
     }
 
     /**
-     * Similar to {@link #_append(JsonToken,Object)} but also updates context with
-     * knowledge that a scalar value was written
+     * Method used for appending token known to represent a scalar value
+     * where there is additional content (text, number) beyond type token
      *
      * @since 2.6.4
      */
     protected final void _appendValue(JsonToken type, Object value)
     {
         _writeContext.writeValue();
-        Segment next = _hasNativeId
-                ? _last.append(_appendAt, type, value, _objectId, _typeId)
-                : _last.append(_appendAt, type, value);
+        Segment next;
+        if (_hasNativeId) {
+            next = _last.append(_appendAt, type, value, _objectId, _typeId);
+        } else {
+            next = _last.append(_appendAt, type, value);
+        }
         if (next == null) {
             ++_appendAt;
         } else {
@@ -1196,13 +1348,59 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
         }
     }
 
-    // 21-Oct-2016, tatu: Does not seem to be used or needed
-    /*
-    protected final void _appendRaw(int rawType, Object value)
+    /**
+     * Specialized method used for appending a field name, appending either
+     * {@link String} or {@link SerializableString}.
+     *
+     * @since 2.10
+     */
+    protected final void _appendFieldName(Object value)
     {
-        Segment next = _hasNativeId
-                ? _last.appendRaw(_appendAt, rawType, value, _objectId, _typeId)
-                : _last.appendRaw(_appendAt, rawType, value);
+        // NOTE: do NOT clear _objectId / _typeId
+        Segment next;
+        if (_hasNativeId) {
+            next =  _last.append(_appendAt, JsonToken.FIELD_NAME, value, _objectId, _typeId);
+        } else {
+            next = _last.append(_appendAt, JsonToken.FIELD_NAME, value);
+        }
+        if (next == null) {
+            ++_appendAt;
+        } else {
+            _last = next;
+            _appendAt = 1;
+        }
+    }
+
+    /**
+     * Specialized method used for appending a structural start Object/Array marker
+     *
+     * @since 2.10
+     */
+    protected final void _appendStartMarker(JsonToken type)
+    {
+        Segment next;
+        if (_hasNativeId) {
+            next =_last.append(_appendAt, type, _objectId, _typeId);
+        } else {
+            next =  _last.append(_appendAt, type);
+        }
+        if (next == null) {
+            ++_appendAt;
+        } else {
+            _last = next;
+            _appendAt = 1; // since we added first at 0
+        }
+    }
+
+    /**
+     * Specialized method used for appending a structural end Object/Array marker
+     *
+     * @since 2.10
+     */
+    protected final void _appendEndMarker(JsonToken type)
+    {
+        // NOTE: type/object id not relevant
+        Segment next = _last.append(_appendAt, type);
         if (next == null) {
             ++_appendAt;
         } else {
@@ -1210,7 +1408,6 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             _appendAt = 1;
         }
     }
-    */
 
     @Override
     protected void _reportUnsupportedOperation() {
@@ -1298,24 +1495,39 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             _parsingContext = TokenBufferReadContext.createRootContext(parentContext);
             _hasNativeTypeIds = hasNativeTypeIds;
             _hasNativeObjectIds = hasNativeObjectIds;
-            _hasNativeIds = (hasNativeTypeIds | hasNativeObjectIds);
+            _hasNativeIds = (hasNativeTypeIds || hasNativeObjectIds);
         }
 
         public void setLocation(JsonLocation l) {
             _location = l;
         }
-        
+
         @Override
         public ObjectCodec getCodec() { return _codec; }
 
         @Override
         public void setCodec(ObjectCodec c) { _codec = c; }
 
+        /*
+        /**********************************************************
+        /* Public API, config access, capability introspection
+        /**********************************************************
+         */
+
         @Override
         public Version version() {
             return com.fasterxml.jackson.databind.cfg.PackageVersion.VERSION;
         }
 
+        // 20-May-2020, tatu: This may or may not be enough -- ideally access is
+        //    via `DeserializationContext`, not parser, but if latter is needed
+        //    then we'll need to pass this from parser contents if which were
+        //    buffered.
+        @Override
+        public JacksonFeatureSet<StreamReadCapability> getReadCapabilities() {
+            return DEFAULT_READ_CAPABILITIES;
+        }
+
         /*
         /**********************************************************
         /* Extended API beyond JsonParser
@@ -1382,6 +1594,8 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
                     || _currToken == JsonToken.END_ARRAY) {
                 // Closing JSON Object/Array? Close matching context
                 _parsingContext = _parsingContext.parentOrCopy();
+            } else {
+                _parsingContext.updateForValue();
             }
             return _currToken;
         }
@@ -1403,7 +1617,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
                 _parsingContext.setCurrentName(name);
                 return name;
             }
-            return (nextToken() == JsonToken.FIELD_NAME) ? getCurrentName() : null;
+            return (nextToken() == JsonToken.FIELD_NAME) ? currentName() : null;
         }
 
         @Override
@@ -1427,7 +1641,7 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
         }
 
         @Override
-        public String getCurrentName() {
+        public String currentName() {
             // 25-Jun-2015, tatu: as per [databind#838], needs to be same as ParserBase
             if (_currToken == JsonToken.START_OBJECT || _currToken == JsonToken.START_ARRAY) {
                 JsonStreamContext parent = _parsingContext.getParent();
@@ -1436,6 +1650,9 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
             return _parsingContext.getCurrentName();
         }
 
+        @Override // since 2.12 delegate to the new method
+        public String getCurrentName() { return currentName(); }
+
         @Override
         public void overrideCurrentName(String name)
         {
@@ -2045,14 +2262,14 @@ sb.append("NativeObjectIds=").append(_hasNativeObjectIds).append(",");
         /**
          * @since 2.3
          */
-        private Object findObjectId(int index) {
+        Object findObjectId(int index) {
             return (_nativeIds == null) ? null : _nativeIds.get(_objectIdIndex(index));
         }
         
         /**
          * @since 2.3
          */
-        private Object findTypeId(int index) {
+        Object findTypeId(int index) {
             return (_nativeIds == null) ? null : _nativeIds.get(_typeIdIndex(index));
         }
 
