@@ -9,7 +9,6 @@ import com.fasterxml.jackson.annotation.JsonCreator;
 
 import com.fasterxml.jackson.core.*;
 
-import com.fasterxml.jackson.databind.ObjectMapper;
 import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;
 import com.fasterxml.jackson.databind.json.JsonMapper;
 import com.fasterxml.jackson.databind.ser.std.StdScalarSerializer;
@@ -64,7 +63,7 @@ public abstract class BaseMapTest
         public DoubleWrapper() { }
         public DoubleWrapper(double value) { d = value; }
     }
-    
+
     /**
      * Simple wrapper around String type, usually to test value
      * conversions or wrapping
@@ -289,7 +288,7 @@ public abstract class BaseMapTest
         throws IOException
     {
         String str = m.writeValueAsString(value);
-        return (Map<String,Object>) m.readValue(str, Map.class);
+        return (Map<String,Object>) m.readValue(str, LinkedHashMap.class);
     }
     
     protected String serializeAsString(ObjectMapper m, Object value)
@@ -354,12 +353,4 @@ public abstract class BaseMapTest
             throw new IllegalArgumentException(e);
         }
     }
-
-    protected static String aposToQuotes(String json) {
-        return json.replace("'", "\"");
-    }
-
-    protected static String quotesToApos(String json) {
-        return json.replace("\"", "'");
-    }
 }
