@@ -7,6 +7,8 @@ import java.util.*;
 import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
 import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.annotation.JsonSetter;
+import com.fasterxml.jackson.annotation.JsonTypeInfo;
+import com.fasterxml.jackson.annotation.JsonTypeName;
 import com.fasterxml.jackson.annotation.Nulls;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.json.JsonWriteFeature;
@@ -51,7 +53,16 @@ public class ObjectMapperTest extends BaseMapTest
     @SuppressWarnings("serial")
     static class NoCopyMapper extends ObjectMapper { }
 
-    final ObjectMapper MAPPER = new ObjectMapper();
+    // [databind#2785]
+    @JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.EXISTING_PROPERTY,
+            property = "packetType")
+    public interface Base2785  {
+    }
+    @JsonTypeName("myType")
+    static class Impl2785 implements Base2785 {
+    }
+
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     /*
     /**********************************************************
@@ -115,7 +126,7 @@ public class ObjectMapperTest extends BaseMapTest
         assertTrue(m.isEnabled(JsonParser.Feature.IGNORE_UNDEFINED));
 
         // // First: verify that handling of features is decoupled:
-        
+
         ObjectMapper m2 = m.copy();
         assertFalse(m2.isEnabled(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES));
         m2.enable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
@@ -185,6 +196,14 @@ public class ObjectMapperTest extends BaseMapTest
         assertSame(customVis, config2.getDefaultVisibilityChecker());
     }
 
+    // [databind#2785]
+    public void testCopyOfSubtypeResolver2785() throws Exception {
+        ObjectMapper objectMapper = new ObjectMapper().copy();
+        objectMapper.registerSubtypes(Impl2785.class);
+        Object result = objectMapper.readValue("{ \"packetType\": \"myType\" }", Base2785.class);
+        assertNotNull(result);
+    }
+
     public void testFailedCopy() throws Exception
     {
         NoCopyMapper src = new NoCopyMapper();
@@ -196,7 +215,7 @@ public class ObjectMapperTest extends BaseMapTest
         }
     }
 
-    public void testAnnotationIntrospectorCopyin() 
+    public void testAnnotationIntrospectorCopying()
     {
         ObjectMapper m = new ObjectMapper();
         m.setAnnotationIntrospector(new MyAnnotationIntrospector());
@@ -253,6 +272,28 @@ public class ObjectMapperTest extends BaseMapTest
         assertTrue(dc.shouldSortPropertiesAlphabetically());
     }
 
+    // Test to ensure that we can check forced property ordering defaults...
+    public void testConfigForForcedPropertySorting() throws Exception
+    {
+        ObjectMapper m = new ObjectMapper();
+
+        // sort-alphabetically is disabled by default:
+        assertTrue(m.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+        SerializationConfig sc = m.getSerializationConfig();
+        assertTrue(sc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+        DeserializationConfig dc = m.getDeserializationConfig();
+        assertTrue(dc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+
+        // but when enabled, should be visible:
+        m = jsonMapperBuilder()
+                .disable(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST)
+                .build();
+        sc = m.getSerializationConfig();
+        assertFalse(sc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+        dc = m.getDeserializationConfig();
+        // and not just via SerializationConfig, but also via DeserializationConfig
+        assertFalse(dc.isEnabled(MapperFeature.SORT_CREATOR_PROPERTIES_FIRST));
+    }
 
     public void testJsonFactoryLinkage()
     {
@@ -266,7 +307,7 @@ public class ObjectMapperTest extends BaseMapTest
         assertSame(m, f.getCodec());
     }
 
-    public void testProviderConfig() throws Exception   
+    public void testProviderConfig() throws Exception
     {
         ObjectMapper m = new ObjectMapper();
         final String JSON = "{ \"x\" : 3 }";
@@ -314,7 +355,7 @@ public class ObjectMapperTest extends BaseMapTest
         assertEquals("[1,2]", m.writer().without(SerializationFeature.INDENT_OUTPUT)
                 .writeValueAsString(input));
     }
-    
+
     // For [databind#703], [databind#978]
     public void testNonSerializabilityOfObject()
     {
@@ -344,6 +385,13 @@ public class ObjectMapperTest extends BaseMapTest
                 .canSerialize(EmptyBean.class));
     }
 
+    // for [databind#2749]: just to check there's no NPE; method really not useful
+    public void testCanDeserialize()
+    {
+        assertTrue(MAPPER.canDeserialize(MAPPER.constructType(EmptyBean.class)));
+        assertTrue(MAPPER.canDeserialize(MAPPER.constructType(Object.class)));
+    }
+
     // for [databind#898]
     public void testSerializerProviderAccess() throws Exception
     {
@@ -451,4 +499,49 @@ public class ObjectMapperTest extends BaseMapTest
             objectMapper.getRegisteredModuleIds()
         );
     }
+
+    // since 2.12
+    public void testHasExplicitTimeZone() throws Exception
+    {
+        final TimeZone DEFAULT_TZ = TimeZone.getTimeZone("UTC");
+
+        // By default, not explicitly set
+        assertFalse(MAPPER.getSerializationConfig().hasExplicitTimeZone());
+        assertFalse(MAPPER.getDeserializationConfig().hasExplicitTimeZone());
+        assertEquals(DEFAULT_TZ, MAPPER.getSerializationConfig().getTimeZone());
+        assertEquals(DEFAULT_TZ, MAPPER.getDeserializationConfig().getTimeZone());
+        assertFalse(MAPPER.reader().getConfig().hasExplicitTimeZone());
+        assertFalse(MAPPER.writer().getConfig().hasExplicitTimeZone());
+
+        final TimeZone TZ = TimeZone.getTimeZone("GMT+4");
+
+        // should be able to set it via mapper
+        ObjectMapper mapper = JsonMapper.builder()
+                .defaultTimeZone(TZ)
+                .build();
+        assertSame(TZ, mapper.getSerializationConfig().getTimeZone());
+        assertSame(TZ, mapper.getDeserializationConfig().getTimeZone());
+        assertTrue(mapper.getSerializationConfig().hasExplicitTimeZone());
+        assertTrue(mapper.getDeserializationConfig().hasExplicitTimeZone());
+        assertTrue(mapper.reader().getConfig().hasExplicitTimeZone());
+        assertTrue(mapper.writer().getConfig().hasExplicitTimeZone());
+
+        // ... as well as via ObjectReader/-Writer
+        {
+            final ObjectReader r = MAPPER.reader().with(TZ);
+            assertTrue(r.getConfig().hasExplicitTimeZone());
+            assertSame(TZ, r.getConfig().getTimeZone());
+            final ObjectWriter w = MAPPER.writer().with(TZ);
+            assertTrue(w.getConfig().hasExplicitTimeZone());
+            assertSame(TZ, w.getConfig().getTimeZone());
+
+            // but can also remove explicit definition
+            final ObjectReader r2 = r.with((TimeZone) null);
+            assertFalse(r2.getConfig().hasExplicitTimeZone());
+            assertEquals(DEFAULT_TZ, r2.getConfig().getTimeZone());
+            final ObjectWriter w2 = w.with((TimeZone) null);
+            assertFalse(w2.getConfig().hasExplicitTimeZone());
+            assertEquals(DEFAULT_TZ, w2.getConfig().getTimeZone());
+        }
+    }
 }
