@@ -26,6 +26,12 @@ public class ObjectReaderTest extends BaseMapTest
         public Map<String, Object> name;
     }
 
+    /*
+    /**********************************************************
+    /* Test methods, simple read/write with defaults
+    /**********************************************************
+     */
+
     public void testSimpleViaParser() throws Exception
     {
         final String JSON = "[1]";
@@ -89,6 +95,22 @@ public class ObjectReaderTest extends BaseMapTest
         assertEquals(Collections.singletonMap("key", ABC.B), value);
     }
 
+    public void testNodeHandling() throws Exception
+    {
+        JsonNodeFactory nodes = new JsonNodeFactory(true);
+        ObjectReader r = MAPPER.reader().with(nodes);
+        // but also no further changes if attempting again
+        assertSame(r, r.with(nodes));
+        assertTrue(r.createArrayNode().isArray());
+        assertTrue(r.createObjectNode().isObject());
+    }
+
+    /*
+    /**********************************************************
+    /* Test methods, some alternative JSON settings
+    /**********************************************************
+     */
+
     public void testParserFeaturesComments() throws Exception
     {
         final String JSON = "[ /* foo */ 7 ]";
@@ -140,15 +162,11 @@ public class ObjectReaderTest extends BaseMapTest
         assertEquals(1, result.size());
     }
 
-    public void testNodeHandling() throws Exception
-    {
-        JsonNodeFactory nodes = new JsonNodeFactory(true);
-        ObjectReader r = MAPPER.reader().with(nodes);
-        // but also no further changes if attempting again
-        assertSame(r, r.with(nodes));
-        assertTrue(r.createArrayNode().isArray());
-        assertTrue(r.createObjectNode().isObject());
-    }
+    /*
+    /**********************************************************
+    /* Test methods, config setting verification
+    /**********************************************************
+     */
 
     public void testFeatureSettings() throws Exception
     {
@@ -268,6 +286,31 @@ public class ObjectReaderTest extends BaseMapTest
         assertEquals(MAPPER.constructType(String.class), r.getValueType());
     }
 
+    public void testParserConfigViaReader() throws Exception
+    {
+        try (JsonParser p = MAPPER.reader()
+                .with(StreamReadFeature.STRICT_DUPLICATE_DETECTION)
+                .createParser("[ ]")) {
+            assertTrue(p.isEnabled(StreamReadFeature.STRICT_DUPLICATE_DETECTION));
+        }
+
+        try (JsonParser p = MAPPER.reader()
+                .with(JsonReadFeature.ALLOW_JAVA_COMMENTS)
+                .createParser("[ ]")) {
+            assertTrue(p.isEnabled(JsonReadFeature.ALLOW_JAVA_COMMENTS.mappedFeature()));
+        }
+    }
+
+    public void testGeneratorConfigViaReader() throws Exception
+    {
+        StringWriter sw = new StringWriter();
+        try (JsonGenerator g = MAPPER.writer()
+                .with(StreamWriteFeature.IGNORE_UNKNOWN)
+                .createGenerator(sw)) {
+            assertTrue(g.isEnabled(StreamWriteFeature.IGNORE_UNKNOWN));
+        }
+    }
+
     /*
     /**********************************************************
     /* Test methods, JsonPointer
@@ -473,8 +516,8 @@ public class ObjectReaderTest extends BaseMapTest
         assertEquals("test", aObject.knownField);
     }
 
-    private static class A{
-        private String knownField;
+    private static class A {
+        String knownField;
 
         @JsonCreator
         private A(@JsonProperty("knownField") String knownField) {
