@@ -5,12 +5,8 @@ import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonAnyGetter;
 import com.fasterxml.jackson.annotation.JsonAnySetter;
-import com.fasterxml.jackson.databind.DeserializationConfig;
-import com.fasterxml.jackson.databind.JavaType;
-import com.fasterxml.jackson.databind.ObjectMapper;
-import com.fasterxml.jackson.databind.ObjectReader;
-import com.fasterxml.jackson.databind.ObjectWriter;
-import com.fasterxml.jackson.databind.SerializationConfig;
+import com.fasterxml.jackson.annotation.JsonPropertyOrder;
+
 import com.fasterxml.jackson.databind.type.TypeFactory;
 import com.fasterxml.jackson.databind.util.LRUMap;
 
@@ -22,6 +18,7 @@ import com.fasterxml.jackson.databind.util.LRUMap;
  */
 public class TestJDKSerialization extends BaseMapTest
 {
+    @JsonPropertyOrder({ "x", "y" })
     static class MyPojo {
         public int x;
         protected int y;
@@ -37,6 +34,7 @@ public class TestJDKSerialization extends BaseMapTest
     }
 
     // for [databind#899]
+    @JsonPropertyOrder({ "abc", "stuff" })
     static class EnumPOJO {
         public ABC abc = ABC.B;
 
@@ -72,7 +70,7 @@ public class TestJDKSerialization extends BaseMapTest
      *   ObjectMapper here can lead to strange unit test suite failures, so
      *   let's create a private copy for this class only.
      */
-    private final ObjectMapper MAPPER = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
     
     public void testConfigs() throws IOException
     {
@@ -97,7 +95,7 @@ public class TestJDKSerialization extends BaseMapTest
     // for [databind#899]
     public void testEnumHandlers() throws IOException
     {
-        ObjectMapper mapper = new ObjectMapper();
+        ObjectMapper mapper = newJsonMapper();
         // ensure we have serializers and/or deserializers, first
         String json = mapper.writerFor(EnumPOJO.class)
                 .writeValueAsString(new EnumPOJO());
@@ -218,7 +216,7 @@ public class TestJDKSerialization extends BaseMapTest
     
     protected byte[] jdkSerialize(Object o) throws IOException
     {
-        ByteArrayOutputStream bytes = new ByteArrayOutputStream(1000);
+        ByteArrayOutputStream bytes = new ByteArrayOutputStream(2000);
         ObjectOutputStream obOut = new ObjectOutputStream(bytes);
         obOut.writeObject(o);
         obOut.close();
