@@ -322,7 +322,7 @@ public class TestBeanDeserializer extends BaseMapTest
     /********************************************************
      */
 
-    private final ObjectMapper MAPPER = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     /**
      * Test to verify details of how trying to deserialize into
@@ -370,24 +370,6 @@ public class TestBeanDeserializer extends BaseMapTest
         assertEquals(2, Issue476Deserializer.propCount);
     }
 
-    public void testPOJOFromEmptyString() throws Exception
-    {
-        // first, verify default settings which do not accept empty String:
-        assertFalse(MAPPER.isEnabled(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT));
-        try {
-            MAPPER.readValue(quote(""), Bean.class);
-            fail("Should not accept Empty String for POJO");
-        } catch (JsonProcessingException e) {
-            verifyException(e, "from String value");
-            assertValidLocation(e.getLocation());
-        }
-        // should be ok to enable dynamically
-        ObjectReader r = MAPPER.readerFor(Bean.class)
-                .with(DeserializationFeature.ACCEPT_EMPTY_STRING_AS_NULL_OBJECT);
-        Bean result = r.readValue(quote(""));
-        assertNull(result);
-    }
-
     // [databind#120]
     public void testModifyArrayDeserializer() throws Exception
     {
