@@ -90,7 +90,7 @@ public class DateDeserializationTest
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = newObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
 
     public void testDateUtil() throws Exception
     {
@@ -432,44 +432,6 @@ public class DateDeserializationTest
         assertEquals(28, c.get(Calendar.DAY_OF_MONTH));
     }
 
-    @SuppressWarnings("deprecation")
-    public void testDateSql() throws Exception
-    {
-        java.sql.Date value = new java.sql.Date(0L);
-        value.setYear(99); // 1999
-        value.setDate(19);
-        value.setMonth(Calendar.APRIL);
-        long now = value.getTime();
-
-        // First from long
-        assertEquals(value, MAPPER.readValue(String.valueOf(now), java.sql.Date.class));
-
-        // then from default java.sql.Date String serialization:
-        
-        java.sql.Date result = MAPPER.readValue(quote(value.toString()), java.sql.Date.class);
-        Calendar c = gmtCalendar(result.getTime());
-        assertEquals(1999, c.get(Calendar.YEAR));
-        assertEquals(Calendar.APRIL, c.get(Calendar.MONTH));
-        assertEquals(19, c.get(Calendar.DAY_OF_MONTH));
-
-        /* [JACKSON-200]: looks like we better add support for regular date
-         *   formats as well
-         */
-        String expStr = "1981-07-13";
-        result = MAPPER.readValue(quote(expStr), java.sql.Date.class);
-        c.setTimeInMillis(result.getTime());
-        assertEquals(1981, c.get(Calendar.YEAR));
-        assertEquals(Calendar.JULY, c.get(Calendar.MONTH));
-        assertEquals(13, c.get(Calendar.DAY_OF_MONTH));
-
-        /* 20-Nov-2009, tatus: I'll be damned if I understand why string serialization
-         *   is off-by-one, but day-of-month does seem to be one less. My guess is
-         *   that something is funky with timezones (i.e. somewhere local TZ is
-         *   being used), but just can't resolve it. Hence, need to comment this:
-         */
-//        assertEquals(expStr, result.toString());
-    }
-
     public void testCalendar() throws Exception
     {
         // not ideal, to use (ever-changing) current date, but...
@@ -513,7 +475,6 @@ public class DateDeserializationTest
     {
         assertNull(MAPPER.readValue(quote(""), java.util.Date.class));
         assertNull(MAPPER.readValue(quote(""), java.util.Calendar.class));
-        assertNull(MAPPER.readValue(quote(""), java.sql.Date.class));
     }
 
     public void test8601DateTimeNoMilliSecs() throws Exception
@@ -623,7 +584,7 @@ public class DateDeserializationTest
 
         // Standard mapper with timezone UTC: shared instance should be ok.
         // ... but, Travis manages to have fails, so insist on newly created
-        ObjectMapper mapper = newObjectMapper();
+        ObjectMapper mapper = newJsonMapper();
         Date dateUTC = mapper.readValue(json, Date.class);  // 1970-01-01T00:00:00.000+00:00
     
         // Mapper with timezone GMT-2
@@ -702,7 +663,7 @@ public class DateDeserializationTest
             fail("Did not throw exception when reading a value from a single value array with the UNWRAP_SINGLE_VALUE_ARRAYS feature disabled");
         } catch (MismatchedInputException exp) {
             verifyException(exp, "Cannot deserialize");
-            verifyException(exp, "out of START_ARRAY");
+            verifyException(exp, "from Array value (token `JsonToken.START_ARRAY`)");
         }
 
         reader = reader.with(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS);
@@ -725,7 +686,7 @@ public class DateDeserializationTest
     /**********************************************************
      */
 
-    public void testLenientCalendar() throws Exception
+    public void testLenientJDKDateTypes() throws Exception
     {
         final String JSON = aposToQuotes("{'value':'2015-11-32'}");
 
@@ -743,8 +704,10 @@ public class DateDeserializationTest
             verifyException(e, "from String \"2015-11-32\"");
             verifyException(e, "expected format");
         }
+    }
 
-        // similarly with Date...
+    public void testLenientJDKDateTypesViaTypeOverride() throws Exception
+    {
         ObjectMapper mapper = new ObjectMapper();
         mapper.configOverride(java.util.Date.class)
             .setFormat(JsonFormat.Value.forLeniency(Boolean.FALSE));
@@ -758,6 +721,38 @@ public class DateDeserializationTest
         }
     }
 
+    public void testLenientJDKDateTypesViaGlobal() throws Exception
+    {
+        final String JSON = quote("2015-11-32");
+
+        // with lenient, can parse fine
+        Calendar value = MAPPER.readValue(JSON, Calendar.class);
+        assertEquals(Calendar.DECEMBER, value.get(Calendar.MONTH));
+        assertEquals(2, value.get(Calendar.DAY_OF_MONTH));
+
+        // but not so if default leniency disabled
+        ObjectMapper mapper = new ObjectMapper();
+        mapper.setDefaultLeniency(false);
+        try {
+            mapper.readValue(JSON, java.util.Date.class);
+            fail("Should not pass with invalid (with strict) date value");
+        } catch (MismatchedInputException e) {
+            verifyException(e, "Cannot deserialize value of type `java.util.Date`");
+            verifyException(e, "from String \"2015-11-32\"");
+            verifyException(e, "expected format");
+        }
+    
+        // Unless we actually had per-type override too
+        mapper = new ObjectMapper();
+        mapper.configOverride(Calendar.class)
+            .setFormat(JsonFormat.Value.forLeniency(Boolean.TRUE));
+        mapper.setDefaultLeniency(false);
+
+        value = mapper.readValue(JSON, Calendar.class);
+        assertEquals(Calendar.DECEMBER, value.get(Calendar.MONTH));
+        assertEquals(2, value.get(Calendar.DAY_OF_MONTH));
+    }
+
     /*
     /**********************************************************
     /* Tests to verify failing cases
@@ -787,18 +782,9 @@ public class DateDeserializationTest
 
     private String dateToString(java.util.Date value)
     {
-        /* Then from String. This is bit tricky, since JDK does not really
-         * suggest a 'standard' format. So let's try using something...
-         */
+        // Then from String. This is bit tricky, since JDK does not really
+        // suggest a 'standard' format. So let's try using something...
         DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ");
         return df.format(value);
     }
-
-    private static Calendar gmtCalendar(long time)
-    {
-        Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
-        c.setTimeInMillis(time);
-        return c;
-    }
-
 }
