@@ -359,8 +359,9 @@ public class POJOPropertiesCollectorTest
 
     public void testGlobalVisibilityForGetters()
     {
-        ObjectMapper m = new ObjectMapper();
-        m.configure(MapperFeature.AUTO_DETECT_GETTERS, false);
+        ObjectMapper m = jsonMapperBuilder()
+                .configure(MapperFeature.AUTO_DETECT_GETTERS, false)
+                .build();
         POJOPropertiesCollector coll = collector(m, SimpleGetterVisibility.class, true);
         // should be 1, expect that we disabled getter auto-detection, so
         Map<String, POJOPropertyBuilder> props = coll.getPropertyMap();
@@ -414,7 +415,7 @@ public class POJOPropertiesCollectorTest
 
     public void testInnerClassWithAnnotationsInCreator() throws Exception
     {
-        BasicBeanDescription beanDesc;
+        BeanDescription beanDesc;
         // first with serialization
         beanDesc = MAPPER.getSerializationConfig().introspect(MAPPER.constructType(Issue701Bean.class));
         assertNotNull(beanDesc);
@@ -426,9 +427,10 @@ public class POJOPropertiesCollectorTest
     public void testUseAnnotationsFalse() throws Exception
     {
         // note: need a separate mapper, need to reconfigure
-        ObjectMapper mapper = new ObjectMapper();
-        mapper.configure(MapperFeature.USE_ANNOTATIONS, false);
-        BasicBeanDescription beanDesc = mapper.getSerializationConfig().introspect(mapper.constructType(Jackson703.class));
+        ObjectMapper mapper = jsonMapperBuilder()
+                .configure(MapperFeature.USE_ANNOTATIONS, false)
+                .build();
+        BeanDescription beanDesc = mapper.getSerializationConfig().introspect(mapper.constructType(Jackson703.class));
         assertNotNull(beanDesc);
 
         Jackson703 bean = new Jackson703();
@@ -539,9 +541,9 @@ public class POJOPropertiesCollectorTest
         // no real difference between serialization, deserialization, at least here
         if (forSerialization) {
             return bci.collectProperties(m0.getSerializationConfig(),
-                    m0.constructType(cls), null, true, "set");
+                    m0.constructType(cls), null, true);
         }
         return bci.collectProperties(m0.getDeserializationConfig(),
-                m0.constructType(cls), null, false, "set");
+                m0.constructType(cls), null, false);
     }
 }
