@@ -77,7 +77,7 @@ public class TestNamingStrategyStd extends BaseMapTest
     }
 
     @JsonPropertyOrder({ "firstName", "lastName" })
-    @JsonNaming(PropertyNamingStrategy.LowerCaseStrategy.class)
+    @JsonNaming(PropertyNamingStrategies.LowerCaseStrategy.class)
     static class BoringBean {
         public String firstName = "Bob";
         public String lastName = "Burger";
@@ -168,8 +168,10 @@ public class TestNamingStrategyStd extends BaseMapTest
                 // [databind#1026]
                 {"usId", "us_id" },
                 {"uId", "u_id" },
+                // [databind#2267]
+                {"xCoordinate", "x_coordinate" },
     });
-    
+
     private ObjectMapper _lcWithUndescoreMapper;
     
     @Override
@@ -177,7 +179,7 @@ public class TestNamingStrategyStd extends BaseMapTest
     {
         super.setUp();
         _lcWithUndescoreMapper = new ObjectMapper();
-        _lcWithUndescoreMapper.setPropertyNamingStrategy(PropertyNamingStrategy.SNAKE_CASE);
+        _lcWithUndescoreMapper.setPropertyNamingStrategy(PropertyNamingStrategies.SNAKE_CASE);
     }
     
     /*
@@ -188,13 +190,13 @@ public class TestNamingStrategyStd extends BaseMapTest
 
     /**
      * Unit test to verify translations of 
-     * {@link PropertyNamingStrategy#SNAKE_CASE} 
+     * {@link PropertyNamingStrategies#SNAKE_CASE} 
      * outside the context of an ObjectMapper.
      */
     public void testLowerCaseStrategyStandAlone()
     {
         for (Object[] pair : SNAKE_CASE_NAME_TRANSLATIONS) {
-            String translatedJavaName = PropertyNamingStrategy.SNAKE_CASE.nameForField(null, null,
+            String translatedJavaName = PropertyNamingStrategies.SNAKE_CASE.nameForField(null, null,
                     (String) pair[0]);
             assertEquals((String) pair[1], translatedJavaName);
         }
@@ -263,25 +265,25 @@ public class TestNamingStrategyStd extends BaseMapTest
 
     /**
      * Unit test to verify translations of 
-     * {@link PropertyNamingStrategy#UPPER_CAMEL_CASE } 
+     * {@link PropertyNamingStrategies#UPPER_CAMEL_CASE } 
      * outside the context of an ObjectMapper.
      */
     public void testPascalCaseStandAlone()
     {
-        assertEquals("UserName", PropertyNamingStrategy.UPPER_CAMEL_CASE.nameForField(null, null, "userName"));
-        assertEquals("User", PropertyNamingStrategy.UPPER_CAMEL_CASE.nameForField(null, null, "User"));
-        assertEquals("User", PropertyNamingStrategy.UPPER_CAMEL_CASE.nameForField(null, null, "user"));
-        assertEquals("X", PropertyNamingStrategy.UPPER_CAMEL_CASE.nameForField(null, null, "x"));
+        assertEquals("UserName", PropertyNamingStrategies.UPPER_CAMEL_CASE.nameForField(null, null, "userName"));
+        assertEquals("User", PropertyNamingStrategies.UPPER_CAMEL_CASE.nameForField(null, null, "User"));
+        assertEquals("User", PropertyNamingStrategies.UPPER_CAMEL_CASE.nameForField(null, null, "user"));
+        assertEquals("X", PropertyNamingStrategies.UPPER_CAMEL_CASE.nameForField(null, null, "x"));
 
-        assertEquals("BADPublicName", PropertyNamingStrategy.UPPER_CAMEL_CASE.nameForField(null, null, "bADPublicName"));
-        assertEquals("BADPublicName", PropertyNamingStrategy.UPPER_CAMEL_CASE.nameForGetterMethod(null, null, "bADPublicName"));
+        assertEquals("BADPublicName", PropertyNamingStrategies.UPPER_CAMEL_CASE.nameForField(null, null, "bADPublicName"));
+        assertEquals("BADPublicName", PropertyNamingStrategies.UPPER_CAMEL_CASE.nameForGetterMethod(null, null, "bADPublicName"));
     }
 
     // [databind#428]
     public void testIssue428PascalWithOverrides() throws Exception
     {
         String json = new ObjectMapper()
-                .setPropertyNamingStrategy(PropertyNamingStrategy.UPPER_CAMEL_CASE)
+                .setPropertyNamingStrategy(PropertyNamingStrategies.UPPER_CAMEL_CASE)
                 .writeValueAsString(new Bean428());
         if (!json.contains(quote("fooBar"))) {
             fail("Should use name 'fooBar', does not: "+json);
@@ -313,22 +315,22 @@ public class TestNamingStrategyStd extends BaseMapTest
     public void testKebabCaseStrategyStandAlone()
     {
         assertEquals("some-value",
-                PropertyNamingStrategy.KEBAB_CASE.nameForField(null, null, "someValue"));
+                PropertyNamingStrategies.KEBAB_CASE.nameForField(null, null, "someValue"));
         assertEquals("some-value",
-                PropertyNamingStrategy.KEBAB_CASE.nameForField(null, null, "SomeValue"));
+                PropertyNamingStrategies.KEBAB_CASE.nameForField(null, null, "SomeValue"));
         assertEquals("url",
-                PropertyNamingStrategy.KEBAB_CASE.nameForField(null, null, "URL"));
+                PropertyNamingStrategies.KEBAB_CASE.nameForField(null, null, "URL"));
         assertEquals("url-stuff",
-                PropertyNamingStrategy.KEBAB_CASE.nameForField(null, null, "URLStuff"));
+                PropertyNamingStrategies.KEBAB_CASE.nameForField(null, null, "URLStuff"));
         assertEquals("some-url-stuff",
-                PropertyNamingStrategy.KEBAB_CASE.nameForField(null, null, "SomeURLStuff"));
+                PropertyNamingStrategies.KEBAB_CASE.nameForField(null, null, "SomeURLStuff"));
     }
     
     public void testSimpleKebabCase() throws Exception
     {
         final FirstNameBean input = new FirstNameBean("Bob");
         ObjectMapper m = new ObjectMapper()
-                .setPropertyNamingStrategy(PropertyNamingStrategy.KEBAB_CASE);
+                .setPropertyNamingStrategy(PropertyNamingStrategies.KEBAB_CASE);
 
         assertEquals(aposToQuotes("{'first-name':'Bob'}"), m.writeValueAsString(input));
 
@@ -336,6 +338,39 @@ public class TestNamingStrategyStd extends BaseMapTest
                 FirstNameBean.class);
         assertEquals("Billy", result.firstName);
     }
+    /*
+    /**********************************************************
+    /* Test methods for LOWER_DOT_CASE
+    /**********************************************************
+     */
+
+    public void testLowerCaseWithDotsStrategyStandAlone()
+    {
+        assertEquals("some.value",
+                PropertyNamingStrategies.LOWER_DOT_CASE.nameForField(null, null, "someValue"));
+        assertEquals("some.value",
+                PropertyNamingStrategies.LOWER_DOT_CASE.nameForField(null, null, "SomeValue"));
+        assertEquals("url",
+                PropertyNamingStrategies.LOWER_DOT_CASE.nameForField(null, null, "URL"));
+        assertEquals("url.stuff",
+                PropertyNamingStrategies.LOWER_DOT_CASE.nameForField(null, null, "URLStuff"));
+        assertEquals("some.url.stuff",
+                PropertyNamingStrategies.LOWER_DOT_CASE.nameForField(null, null, "SomeURLStuff"));
+    }
+
+    public void testSimpleLowerCaseWithDots() throws Exception
+    {
+        final ObjectMapper m = jsonMapperBuilder()
+            .propertyNamingStrategy(PropertyNamingStrategies.LOWER_DOT_CASE)
+            .build();
+
+        final FirstNameBean input = new FirstNameBean("Bob");
+        assertEquals(aposToQuotes("{'first.name':'Bob'}"), m.writeValueAsString(input));
+
+        FirstNameBean result = m.readValue(aposToQuotes("{'first.name':'Billy'}"),
+                FirstNameBean.class);
+        assertEquals("Billy", result.firstName);
+    }
 
     /*
     /**********************************************************
@@ -349,7 +384,7 @@ public class TestNamingStrategyStd extends BaseMapTest
     public void testNamingWithObjectNode() throws Exception
     {
         ObjectMapper m = new ObjectMapper()
-            .setPropertyNamingStrategy(PropertyNamingStrategy.LOWER_CASE);
+            .setPropertyNamingStrategy(PropertyNamingStrategies.LOWER_CASE);
         ClassWithObjectNodeField result =
             m.readValue(
                 "{ \"id\": \"1\", \"json\": { \"foo\": \"bar\", \"baz\": \"bing\" } }",
@@ -363,30 +398,32 @@ public class TestNamingStrategyStd extends BaseMapTest
 
     public void testExplicitRename() throws Exception
     {
-      ObjectMapper m = new ObjectMapper();
-      m.setPropertyNamingStrategy(PropertyNamingStrategy.SNAKE_CASE);
-      m.enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY);
-      // by default, renaming will not take place on explicitly named fields
-      assertEquals(aposToQuotes("{'firstName':'Peter','lastName':'Venkman','user_age':'35'}"),
-          m.writeValueAsString(new ExplicitBean()));
+        ObjectMapper m = jsonMapperBuilder()
+                .propertyNamingStrategy(PropertyNamingStrategies.SNAKE_CASE)
+                .enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY)
+                .build();
+        // by default, renaming will not take place on explicitly named fields
+        assertEquals(aposToQuotes("{'firstName':'Peter','lastName':'Venkman','user_age':'35'}"),
+                m.writeValueAsString(new ExplicitBean()));
 
-      m = new ObjectMapper();
-      m.setPropertyNamingStrategy(PropertyNamingStrategy.SNAKE_CASE);
-      m.enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY);
-      m.enable(MapperFeature.ALLOW_EXPLICIT_PROPERTY_RENAMING);
-      // w/ feature enabled, ALL property names should get re-written
-      assertEquals(aposToQuotes("{'first_name':'Peter','last_name':'Venkman','user_age':'35'}"),
-          m.writeValueAsString(new ExplicitBean()));
+        m = jsonMapperBuilder()
+                .propertyNamingStrategy(PropertyNamingStrategies.SNAKE_CASE)
+                .enable(MapperFeature.SORT_PROPERTIES_ALPHABETICALLY)
+                .enable(MapperFeature.ALLOW_EXPLICIT_PROPERTY_RENAMING)
+                .build();
+        // w/ feature enabled, ALL property names should get re-written
+        assertEquals(aposToQuotes("{'first_name':'Peter','last_name':'Venkman','user_age':'35'}"),
+                m.writeValueAsString(new ExplicitBean()));
 
-      // test deserialization as well
-      ExplicitBean bean =
-          m.readValue(aposToQuotes("{'first_name':'Egon','last_name':'Spengler','user_age':'32'}"),
-              ExplicitBean.class);
+        // test deserialization as well
+        ExplicitBean bean =
+                m.readValue(aposToQuotes("{'first_name':'Egon','last_name':'Spengler','user_age':'32'}"),
+                        ExplicitBean.class);
 
-      assertNotNull(bean);
-      assertEquals("Egon", bean.userFirstName);
-      assertEquals("Spengler", bean.userLastName);
-      assertEquals("32", bean.userAge);
+        assertNotNull(bean);
+        assertEquals("Egon", bean.userFirstName);
+        assertEquals("Spengler", bean.userLastName);
+        assertEquals("32", bean.userAge);
     }
 
     // Also verify that "no naming strategy" should be ok
