@@ -39,7 +39,7 @@ public class RaceCondition738Test extends BaseMapTest
     static class Wrapper {
         private final HasSubTypes hasSubTypes;
 
-        private Wrapper(HasSubTypes hasSubTypes) {
+        Wrapper(HasSubTypes hasSubTypes) {
             this.hasSubTypes = hasSubTypes;
         }
 
@@ -63,7 +63,7 @@ public class RaceCondition738Test extends BaseMapTest
     }
     
     void runOnce(int round, int max) throws Exception {
-        final ObjectMapper mapper = newObjectMapper();
+        final ObjectMapper mapper = newJsonMapper();
         Callable<String> writeJson = new Callable<String>() {
             @Override
             public String call() throws Exception {
