@@ -4,10 +4,11 @@ import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonProperty;
+
 import com.fasterxml.jackson.core.Version;
+
 import com.fasterxml.jackson.databind.BaseMapTest;
 import com.fasterxml.jackson.databind.ObjectMapper;
-import com.fasterxml.jackson.databind.module.SimpleModule;
 
 public class TestAbstractTypes extends BaseMapTest
 {
@@ -77,7 +78,7 @@ public class TestAbstractTypes extends BaseMapTest
             return value;
         }
     }
-    
+
     /*
     /**********************************************************************
     /* Test methods
@@ -157,9 +158,9 @@ public class TestAbstractTypes extends BaseMapTest
     }
 
     // [databind#2019]: mappings from multiple modules
-    public static void testAbstractMappingsFromTwoModules() throws Exception
+    public void testAbstractMappingsFromTwoModules() throws Exception
     {
-        ObjectMapper mapper = newObjectMapper();
+        ObjectMapper mapper = newJsonMapper();
         SimpleModule module1 = new SimpleModule("module1");
         module1.addAbstractTypeMapping(Datatype1.class, SimpleDatatype1.class);
 
