@@ -44,13 +44,15 @@ public class TestDuplicateRegistration extends BaseMapTest
         mapper.registerModule(module);
         assertEquals(2, module.regCount);
 
+        final MyModule module2 = new MyModule();
         // and ditto for a new instance
-        ObjectMapper mapper2 = new ObjectMapper();
-        mapper2.disable(MapperFeature.IGNORE_DUPLICATE_MODULE_REGISTRATIONS);
-        MyModule module2 = new MyModule();
-        mapper.registerModule(module2);
-        mapper.registerModule(module2);
-        mapper.registerModule(module2);
+        @SuppressWarnings("unused")
+        ObjectMapper mapper2 = jsonMapperBuilder()
+                .disable(MapperFeature.IGNORE_DUPLICATE_MODULE_REGISTRATIONS)
+                .addModule(module2)
+                .addModule(module2)
+                .addModule(module2)
+                .build();
         assertEquals(3, module2.regCount);
     }
 }
