@@ -80,7 +80,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
                 .readValue("{\"v\":[3]}");
             fail("Did not throw exception when reading a value from a single value array with the UNWRAP_SINGLE_VALUE_ARRAYS feature disabled");
         } catch (MismatchedInputException e) {
-            verifyException(e, "Cannot deserialize instance of `int`");
+            verifyException(e, "Cannot deserialize value of type `int`");
         }
 
         ObjectReader r = UNWRAPPING_READER.forType(IntBean.class);
@@ -117,7 +117,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
             noUnwrapR.readValue("{\"v\":[3]}");
             fail("Did not throw exception when reading a value from a single value array");
         } catch (MismatchedInputException e) {
-            verifyException(e, "Cannot deserialize instance of `long`");
+            verifyException(e, "Cannot deserialize value of type `long`");
         }
 
         LongBean result = unwrapR.readValue("{\"v\":[3]}");
@@ -154,7 +154,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
             noUnwrapR.readValue("{\"v\":[" + value + "]}");
             fail("Did not throw exception when reading a value from a single value array");
         } catch (MismatchedInputException e) {
-            verifyException(e, "Cannot deserialize instance of `double`");
+            verifyException(e, "Cannot deserialize value of type `double`");
         }
         
         DoubleBean result = unwrapR.readValue("{\"v\":[" + value + "]}");
@@ -359,8 +359,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
             NO_UNWRAPPING_READER.readValue("[\""+value+"\"]", String.class);
             fail("Exception not thrown when attempting to unwrap a single value 'String' array into a simple String");
         } catch (MismatchedInputException exp) {
-            verifyException(exp, "Cannot deserialize");
-            verifyException(exp, "out of START_ARRAY");
+            _verifyNoDeserFromArray(exp);
         }
  
         try {
@@ -385,8 +384,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
             r.readValue("[" + value.toString() + "]");
             fail("Exception was not thrown when attempting to read a single value array of BigDecimal when UNWRAP_SINGLE_VALUE_ARRAYS feature is disabled");
         } catch (MismatchedInputException exp) {
-            verifyException(exp, "Cannot deserialize");
-            verifyException(exp, "out of START_ARRAY");
+            _verifyNoDeserFromArray(exp);
         }
 
         r = UNWRAPPING_READER.forType(BigDecimal.class);
@@ -411,8 +409,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
             NO_UNWRAPPING_READER.readValue("[" + value.toString() + "]", BigInteger.class);
             fail("Exception was not thrown when attempting to read a single value array of BigInteger when UNWRAP_SINGLE_VALUE_ARRAYS feature is disabled");
         } catch (MismatchedInputException exp) {
-            verifyException(exp, "Cannot deserialize");
-            verifyException(exp, "out of START_ARRAY");
+            _verifyNoDeserFromArray(exp);
         }
 
         result = UNWRAPPING_READER.readValue("[" + value.toString() + "]", BigInteger.class);
@@ -438,7 +435,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
                 .readValue("[" + quote(String.class.getName()) + "]");
             fail("Did not throw exception when UNWRAP_SINGLE_VALUE_ARRAYS feature was disabled and attempted to read a Class array containing one element");
         } catch (MismatchedInputException e) {
-            verifyException(e, "out of START_ARRAY token");
+            _verifyNoDeserFromArray(e);
         }
 
         _verifyMultiValueArrayFail("[" + quote(Object.class.getName()) + "," + quote(Object.class.getName()) +"]",
@@ -456,7 +453,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
                 .readValue("[\""+value.toString()+"\"]");
             fail("Did not throw exception for single value array when UNWRAP_SINGLE_VALUE_ARRAYS is disabled");
         } catch (MismatchedInputException e) {
-            verifyException(e, "out of START_ARRAY token");
+            _verifyNoDeserFromArray(e);
         }
         
         _verifyMultiValueArrayFail("[\""+value.toString()+"\",\""+value.toString()+"\"]", URI.class);
@@ -471,7 +468,7 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
                 .readValue("[" + quote(uuidStr) + "]");
             fail("Exception was not thrown as expected");
         } catch (MismatchedInputException e) {
-            verifyException(e, "out of START_ARRAY token");
+            _verifyNoDeserFromArray(e);
         }
         assertEquals(uuid,
                 UNWRAPPING_READER.forType(UUID.class)
@@ -485,6 +482,12 @@ public class UnwrapSingleArrayScalarsTest extends BaseMapTest
     /**********************************************************
      */
 
+    private void _verifyNoDeserFromArray(Exception e) {
+        verifyException(e, "Cannot deserialize");
+        verifyException(e, "from Array value");
+        verifyException(e, "JsonToken.START_ARRAY");
+    }
+
     private void _verifyMultiValueArrayFail(String input, Class<?> type) throws IOException {
         try {
             UNWRAPPING_READER.forType(type).readValue(input);
