@@ -10,7 +10,6 @@ import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.BaseMapTest;
 import com.fasterxml.jackson.databind.JavaType;
 import com.fasterxml.jackson.databind.type.TypeFactory;
-import com.fasterxml.jackson.databind.util.ClassUtil;
 
 public class ClassUtilTest extends BaseMapTest
 {
@@ -33,7 +32,15 @@ public class ClassUtilTest extends BaseMapTest
 
     interface SubInt extends BaseInt { }
 
-    enum TestEnum { A; }
+    enum TestEnum {
+        A,
+        B {
+            @Override
+            public String toString() {
+                return "TestEnum{B}";
+            }
+        }
+    }
 
     abstract class InnerNonStatic { }
 
@@ -203,6 +210,7 @@ public class ClassUtilTest extends BaseMapTest
     public void testFindEnumType()
     {
         assertEquals(TestEnum.class, ClassUtil.findEnumType(TestEnum.A));
+        assertEquals(TestEnum.class, ClassUtil.findEnumType(TestEnum.B));
         // different codepaths for empty and non-empty EnumSets...
         assertEquals(TestEnum.class, ClassUtil.findEnumType(EnumSet.allOf(TestEnum.class)));
         assertEquals(TestEnum.class, ClassUtil.findEnumType(EnumSet.noneOf(TestEnum.class)));
