@@ -7,11 +7,13 @@ import java.util.UUID;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.JsonParser.NumberType;
+import com.fasterxml.jackson.core.exc.InputCoercionException;
 import com.fasterxml.jackson.core.io.SerializedString;
 import com.fasterxml.jackson.core.util.JsonParserSequence;
 
 import com.fasterxml.jackson.databind.*;
 
+@SuppressWarnings("resource")
 public class TestTokenBuffer extends BaseMapTest
 {
     private final ObjectMapper MAPPER = objectMapper();
@@ -39,11 +41,11 @@ public class TestTokenBuffer extends BaseMapTest
         buf.setCodec(null);
         assertNull(buf.getCodec());
 
-        assertFalse(buf.isEnabled(JsonGenerator.Feature.ESCAPE_NON_ASCII));
-        buf.enable(JsonGenerator.Feature.ESCAPE_NON_ASCII);
-        assertTrue(buf.isEnabled(JsonGenerator.Feature.ESCAPE_NON_ASCII));
-        buf.disable(JsonGenerator.Feature.ESCAPE_NON_ASCII);
-        assertFalse(buf.isEnabled(JsonGenerator.Feature.ESCAPE_NON_ASCII));
+        assertFalse(buf.isEnabled(JsonGenerator.Feature.WRITE_BIGDECIMAL_AS_PLAIN));
+        buf.enable(JsonGenerator.Feature.WRITE_BIGDECIMAL_AS_PLAIN);
+        assertTrue(buf.isEnabled(JsonGenerator.Feature.WRITE_BIGDECIMAL_AS_PLAIN));
+        buf.disable(JsonGenerator.Feature.WRITE_BIGDECIMAL_AS_PLAIN);
+        assertFalse(buf.isEnabled(JsonGenerator.Feature.WRITE_BIGDECIMAL_AS_PLAIN));
 
         buf.close();
         assertTrue(buf.isClosed());
@@ -58,7 +60,7 @@ public class TestTokenBuffer extends BaseMapTest
         
         // First, with empty buffer
         JsonParser p = buf.asParser();
-        assertNull(p.getCurrentToken());
+        assertNull(p.currentToken());
         assertNull(p.nextToken());
         p.close();
 
@@ -66,7 +68,7 @@ public class TestTokenBuffer extends BaseMapTest
         buf.writeString("abc");
 
         p = buf.asParser();
-        assertNull(p.getCurrentToken());
+        assertNull(p.currentToken());
         assertToken(JsonToken.VALUE_STRING, p.nextToken());
         assertEquals("abc", p.getText());
         assertNull(p.nextToken());
@@ -75,7 +77,7 @@ public class TestTokenBuffer extends BaseMapTest
         // Then, let's append at root level
         buf.writeNumber(13);
         p = buf.asParser();
-        assertNull(p.getCurrentToken());
+        assertNull(p.currentToken());
         assertToken(JsonToken.VALUE_STRING, p.nextToken());
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertEquals(13, p.getIntValue());
@@ -106,7 +108,7 @@ public class TestTokenBuffer extends BaseMapTest
         }
 
         JsonParser p = buf.asParser();
-        assertNull(p.getCurrentToken());
+        assertNull(p.currentToken());
 
         for (double v : values1) {
             assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
@@ -138,7 +140,7 @@ public class TestTokenBuffer extends BaseMapTest
                 try {
                     p.getIntValue();
                     fail("Expected failure for `int` overflow");
-                } catch (JsonParseException e) {
+                } catch (InputCoercionException e) {
                     verifyException(e, "Numeric value ("+big+") out of range of int");
                 }
             }
@@ -153,7 +155,7 @@ public class TestTokenBuffer extends BaseMapTest
                 try {
                     p.getIntValue();
                     fail("Expected failure for `int` overflow");
-                } catch (JsonParseException e) {
+                } catch (InputCoercionException e) {
                     verifyException(e, "Numeric value ("+big+") out of range of int");
                 }
             }
@@ -171,7 +173,7 @@ public class TestTokenBuffer extends BaseMapTest
                 try {
                     p.getLongValue();
                     fail("Expected failure for `long` overflow");
-                } catch (JsonParseException e) {
+                } catch (InputCoercionException e) {
                     verifyException(e, "Numeric value ("+big+") out of range of long");
                 }
             }
@@ -205,7 +207,7 @@ public class TestTokenBuffer extends BaseMapTest
         assertTrue(buf.getOutputContext().inRoot());
 
         JsonParser p = buf.asParser();
-        assertNull(p.getCurrentToken());
+        assertNull(p.currentToken());
         assertTrue(p.getParsingContext().inRoot());
         assertToken(JsonToken.START_ARRAY, p.nextToken());
         assertTrue(p.getParsingContext().inArray());
@@ -266,7 +268,7 @@ public class TestTokenBuffer extends BaseMapTest
         assertTrue(buf.getOutputContext().inRoot());
 
         JsonParser p = buf.asParser();
-        assertNull(p.getCurrentToken());
+        assertNull(p.currentToken());
         assertTrue(p.getParsingContext().inRoot());
         assertToken(JsonToken.START_OBJECT, p.nextToken());
         assertTrue(p.getParsingContext().inObject());
@@ -283,22 +285,22 @@ public class TestTokenBuffer extends BaseMapTest
         buf.writeEndObject();
 
         p = buf.asParser();
-        assertNull(p.getCurrentToken());
+        assertNull(p.currentToken());
         assertToken(JsonToken.START_OBJECT, p.nextToken());
-        assertNull(p.getCurrentName());
+        assertNull(p.currentName());
         assertToken(JsonToken.FIELD_NAME, p.nextToken());
-        assertEquals("num", p.getCurrentName());
+        assertEquals("num", p.currentName());
         // and override should also work:
         p.overrideCurrentName("bah");
-        assertEquals("bah", p.getCurrentName());
+        assertEquals("bah", p.currentName());
         
         assertToken(JsonToken.VALUE_NUMBER_FLOAT, p.nextToken());
         assertEquals(1.25, p.getDoubleValue());
         // should still have access to (overridden) name
-        assertEquals("bah", p.getCurrentName());
+        assertEquals("bah", p.currentName());
         assertToken(JsonToken.END_OBJECT, p.nextToken());
         // but not any more
-        assertNull(p.getCurrentName());
+        assertNull(p.currentName());
         assertNull(p.nextToken());
         p.close();
         buf.close();
@@ -349,10 +351,10 @@ public class TestTokenBuffer extends BaseMapTest
         JsonParser p = buf1.asParser();
         assertToken(JsonToken.START_OBJECT, p.nextToken());
         assertToken(JsonToken.FIELD_NAME, p.nextToken());
-        assertEquals("a", p.getCurrentName());
+        assertEquals("a", p.currentName());
         assertToken(JsonToken.VALUE_TRUE, p.nextToken());
         assertToken(JsonToken.FIELD_NAME, p.nextToken());
-        assertEquals("b", p.getCurrentName());
+        assertEquals("b", p.currentName());
         assertToken(JsonToken.VALUE_NUMBER_INT, p.nextToken());
         assertEquals(13, p.getIntValue());
         assertToken(JsonToken.END_OBJECT, p.nextToken());
@@ -402,7 +404,7 @@ public class TestTokenBuffer extends BaseMapTest
     {
         TokenBuffer buf = new TokenBuffer(null, false); // no ObjectCodec
         StringWriter w = new StringWriter();
-        JsonGenerator gen = MAPPER.getFactory().createGenerator(w);
+        JsonGenerator gen = MAPPER.createGenerator(w);
  
         // test content: [{"a":1,"b":{"c":2}},{"a":2,"b":{"c":3}}]
 
@@ -564,8 +566,8 @@ public class TestTokenBuffer extends BaseMapTest
         assertFalse(p.isClosed());
         
         assertFalse(seq.hasCurrentToken());
-        assertNull(seq.getCurrentToken());
-        assertNull(seq.getCurrentName());
+        assertNull(seq.currentToken());
+        assertNull(seq.currentName());
 
         assertToken(JsonToken.START_ARRAY, seq.nextToken());
         assertToken(JsonToken.VALUE_STRING, seq.nextToken());
@@ -599,7 +601,6 @@ public class TestTokenBuffer extends BaseMapTest
      * Test to verify that TokenBuffer and JsonParserSequence work together
      * as expected.
      */
-    @SuppressWarnings("resource")
     public void testWithMultipleJsonParserSequences() throws IOException
     {
         TokenBuffer buf1 = new TokenBuffer(null, false);
